<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LicenseController.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

class Admin_LicenseController extends Zend_Controller_Action
{

    public function indexAction()
    {
        $config = Zend_Registry::get('config');
        
        $form = $this->getLicenseLoginForm();
        $this->view->form = $form;
        
        $this->view->urlSelf = $this->_helper->url->url(array(
            'language' => Zend_Registry::get('Zend_Translate')->getLocale() , 
            'module' => 'admin' , 
            'controller' => 'license'
        ), null, true) . '/';
        
        $currVersion = explode('.', $config->version);
        $this->view->currVersionMajor = intval($currVersion[0]);

    }
    
    public static function clientinfoAction() 
    {
        $result['replyCode'] = 200;
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        if ($config->installed == 't') {
            
            $license = getLicense();

            //Type
            $licenseType = $license['properties']['type']['value'];
            $licenseId = $license['properties']['id']['value'];
            
            //Name
            $licenseConfigKey = 'license_' . $licenseType;
            $licenseName = $config->licenses->license->$licenseConfigKey;
            
            if ($licenseType == 0) {
                $expireDate = new Zend_Date($license['info']['FILE_EXPIRY']);
                $result['timeleft'] = $expireDate->get(GT_DATETIME_STYLE_PHP); 
            }
                        
            $result['type'] = $licenseType;
            $result['name'] = $licenseName;
            $result['id'] = $licenseId;
            
        } else {
            $result['type'] = -1;
            $result['name'] = '';
            $result['id'] = 0;
        }
        
        self::json($result);
    }
    
    public static function serverinfoAction()
    {
        $request = new Zend_Controller_Request_Http();

        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $url = $config->licenses->portalAdress . '/en/default/license/getinfo'; // .
            
        $data = array(
            'account' => $request->getParam('account'),
            'password' => $request->getParam('password'),
        );
            
        $options = array(
            'http' => array(
                'method' => "POST",
                'header' => "Accept-language: en\r\n" . "Content-type: application/x-www-form-urlencoded\r\n" ,
                'content' => http_build_query($data)
            )
        );
            
        if ($config->installed == 'f') {
            $proxyHost = $request->getParam('proxyHost');
            $proxyPort = $request->getParam('proxyPort');
            $proxyUsername = $request->getParam('proxyUsername');
            $proxyPassword = $request->getParam('proxyPassword');
        } else {
            $proxyHost = $config->proxy->host;
            $proxyPort = $config->proxy->port;
            $proxyUsername = $config->proxy->username;
            $proxyPassword = $config->proxy->password;
        }
        
        if (! empty($proxyHost) && !empty($proxyPort)) {
            $proxy = 'tcp://' . $proxyHost . ':' . $proxyPort;
            $options['http']['proxy'] = $proxy;
            $options['http']['request_fulluri'] = true;
            
            if (! empty($proxyUsername) && !empty($proxyPassword)) {
                $credentials = base64_encode($proxyUsername . ':' . $proxyPassword);
                $options['http']['header'] .= "Proxy-Authorization: Basic " . $credentials . "\r\n";
            }
        }
        
        $context = stream_context_create($options);
        $fp = fopen($url, 'r', false, $context);
        
        if($fp) {
            $jsonResponse = stream_get_contents($fp);
            $result = Zend_Json::decode($jsonResponse);
            
            if ($config->installed == 'f') {
                if (! empty($proxyHost) && !empty($proxyPort)) {
                    $config = new Zend_Config_Xml(GT_CONFIG_PATH . DIR_SEP . 'config.xml', null, array(
                        'skipExtends' => true , 
                        'allowModifications' => true
                    ));
    
                    $config->production->proxy->host = $proxyHost;
                    $config->production->proxy->port = $proxyPort;
                    if (! empty($proxyUsername) && !empty($proxyPassword)) {
                        $config->production->proxy->username = $proxyUsername;
                        $config->production->proxy->password = $proxyPassword;
                    }
                    
                    //
                    require_once GT_MODULES_PATH . DIR_SEP . 'admin' . DIR_SEP . 'controllers' . DIR_SEP . 'GenconfController.php';
                    Admin_GenconfController::escapeConfig($config->production);
                    
                    //
                    $writer = new Zend_Config_Writer_Xml(array(
                        'config' => $config , 
                        'filename' => GT_CONFIG_PATH . DIR_SEP . 'config.xml'
                    ));
                    $writer->write();
                }
            }
        } else {
            $result['replyCode'] = 200;
            $result['error'] = vsprintf($translate->_('unableToOpen'), $url);
        }
        
        self::json($result);
    }
    
    public static function getlicenseAction()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $request = new Zend_Controller_Request_Http();
        
        if ($config->installed == 't') {
            $license = getLicense();
        } else {
            $license['properties']['type']['value'] = -1;    
        }
        
        $url = $config->licenses->portalAdress . '/en/default/license/getlicense';
        $data = array(
            'account' => $request->getParam('account'),
            'password' => $request->getParam('password'),
            'info' => ioncube_server_data(),
            'id' => $request->getParam('id'),
            'curr_version' => $config->version ,
            'curr_type' => $license['properties']['type']['value']
        );
        
        $options = array(
            'http' => array(
                'method' => "POST" , 
                'header' => "Accept-language: en\r\n" . "Content-type: application/x-www-form-urlencoded\r\n" , 
                'content' => http_build_query($data)
            )
        );
        

        if (! empty($config->proxy->host) && !empty($config->proxy->port)) {
            $proxy = 'tcp://' . $config->proxy->host . ':' . $config->proxy->port;
            $options['http']['proxy'] = $proxy;
            $options['http']['request_fulluri'] = true;
            
            if (! empty($config->proxy->username) && !empty($config->proxy->password)) {
                $credentials = base64_encode($config->proxy->username . ':' . $config->proxy->password);
                $options['http']['header'] .= "Proxy-Authorization: Basic " . $credentials . "\r\n";
            }
        }
        
        $context = stream_context_create($options);
        @$fp = fopen($url, 'r', false, $context);

        if($fp) {
            $jsonResponse = stream_get_contents($fp);
            $result = Zend_Json::decode($jsonResponse);
        } else {
            $result['replyCode'] = 200;
            $result['error'] = vsprintf($translate->_('unableToOpen'), $url);
        }

        self::json($result);
    }
    
    public static function revokeAction()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $request = new Zend_Controller_Request_Http();
        
        if ($config->installed == 't') {
            $license = getLicense();
        } else {
            throw new Giscuit_Exception("Giscuit is not installed");  
        }
        
        if (! isset($license['properties']['id'])) {
            throw new Giscuit_Exception("License id is unknown");  
        }
        
        $url = $config->licenses->portalAdress . '/en/default/license/revoke';
        $data = array(
            'account' => $request->getParam('account'),
            'password' => $request->getParam('password'),
            'id' => $license['properties']['id']['value']
        );
        
        $options = array(
            'http' => array(
                'method' => "POST" , 
                'header' => "Accept-language: en\r\n" . "Content-type: application/x-www-form-urlencoded\r\n" , 
                'content' => http_build_query($data)
            )
        );
        

        if (! empty($config->proxy->host) && !empty($config->proxy->port)) {
            $proxy = 'tcp://' . $config->proxy->host . ':' . $config->proxy->port;
            $options['http']['proxy'] = $proxy;
            $options['http']['request_fulluri'] = true;
            
            if (! empty($config->proxy->username) && !empty($config->proxy->password)) {
                $credentials = base64_encode($config->proxy->username . ':' . $config->proxy->password);
                $options['http']['header'] .= "Proxy-Authorization: Basic " . $credentials . "\r\n";
            }
        }
        
        $context = stream_context_create($options);
        @$fp = fopen($url, 'r', false, $context);

        if($fp) {
            $jsonResponse = stream_get_contents($fp);
            $result = Zend_Json::decode($jsonResponse);
            
            if (! isset($result['error'])) {
                file_put_contents(GT_APP_PATH . DIR_SEP . "license", '');
                file_put_contents(GT_APP_PATH . DIR_SEP . "LICENSE.txt", '');
                //Clear cache
                if (Zend_Registry::isRegistered('cache')) {
                    Zend_Registry::get('cache')->clean(Zend_Cache::CLEANING_MODE_ALL);    
                }
            }
        } else {
            $result['replyCode'] = 200;
            $result['error'] = vsprintf($translate->_('unableToOpen'), $url);
        }

        self::json($result);
    }
    
    public static function addlicenseAction() 
    {
        $request = new Zend_Controller_Request_Http();
        
        $result['replyCode'] = 200;
        
        $config = new Zend_Config_Xml(GT_CONFIG_PATH . DIR_SEP . 'config.xml', null, array(
            'skipExtends' => true , 
            'allowModifications' => true
        ));
        $type = intval($request->getParam('type'));
        
        if ($config->production->installed == 't') {
            if(Zend_Registry::isRegistered('db')) {
                $db = Zend_Registry::get('db');            
            } else {
                Giscuit_Controller_Plugin_Init::parseDbParams($config->production->database);
                $db = Zend_Db::factory($config->production->database);
                $db->getConnection();
                Zend_Db_Table_Abstract::setDefaultAdapter($db);
            }
    
            if($type == 2 || $type == 3) {
                $role = 'registered';    
            } else {
                $role = 'guest';
            }
    
            $db->query("UPDATE \"{$config->production->database->schema}\".\"rules\" SET
                role = '$role'
                WHERE resource = 'map_menu'");
        }

        file_put_contents(GT_APP_PATH . DIR_SEP . "license", $request->getParam('file'));
        //Clear cache
        if (Zend_Registry::isRegistered('cache')) {
            Zend_Registry::get('cache')->clean(Zend_Cache::CLEANING_MODE_ALL);    
        }
        
        //Save agreement
        file_put_contents(GT_APP_PATH . DIR_SEP . "LICENSE.txt", $request->getParam('agreement'));
        
        //Save license type
        $config->production->licenses->type = $type;

        //
        require_once GT_MODULES_PATH . DIR_SEP . 'admin' . DIR_SEP . 'controllers' . DIR_SEP . 'GenconfController.php';
        Admin_GenconfController::escapeConfig($config->production);
        
        //
        $writer = new Zend_Config_Writer_Xml(array(
            'config' => $config , 
            'filename' => GT_CONFIG_PATH . DIR_SEP . 'config.xml'
        ));
        $writer->write();

        self::json($result);
    }
    
    public static function getagreementAction()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');

        $request = new Zend_Controller_Request_Http();

        $url = $config->licenses->portalAdress . '/en/default/license/getagreement';
        $data = array(
            'type' => intval($request->getParam('type'))
        );
        
        $options = array(
            'http' => array(
                'method' => "POST" , 
                'header' => "Accept-language: en\r\n" . "Content-type: application/x-www-form-urlencoded\r\n" , 
                'content' => http_build_query($data)
            )
        );

        if (! empty($config->proxy->host) && !empty($config->proxy->port)) {
            $proxy = 'tcp://' . $config->proxy->host . ':' . $config->proxy->port;
            $options['http']['proxy'] = $proxy;
            $options['http']['request_fulluri'] = true;
            
            if (! empty($config->proxy->username) && !empty($config->proxy->password)) {
                $credentials = base64_encode($config->proxy->username . ':' . $config->proxy->password);
                $options['http']['header'] .= "Proxy-Authorization: Basic " . $credentials . "\r\n";
            }
        }
        
        $context = stream_context_create($options);
        @$fp = fopen($url, 'r', false, $context);

        if($fp) {
            $jsonResponse = stream_get_contents($fp);
            $result = Zend_Json::decode($jsonResponse);
        } else {
            $result['replyCode'] = 200;
            $result['error'] = vsprintf($translate->_('unableToOpen'), $url);
        }

        self::json($result);
    }
    
    public static function getLicenseLoginForm ()
    {
        $form = new Giscuit_Form_LicenseLogin();
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'licenseForm'  
        ));
        
        return $form;
    }
    
    public static function json($data) 
    {
        $response = Zend_Controller_Front::getInstance()->getResponse();
        if($response) {
            Zend_Controller_Action_HelperBroker::getStaticHelper('json')->direct($data);    
        } else {
            header('Content-Type: application/json');
            echo Zend_Json::encode($data);
            exit;
        }
    }
    
}

?>