<?php

class Tool_IncidentsController extends Zend_Controller_Action
{

    public function getAction ()
    {
        $result['replyCode'] = 200;
        
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $config = Zend_Registry::get('config');
        $request = $this->getRequest();
        
        $incidentType = intval($request->getParam('type'));
        $incidentCategory = intval($request->getParam('category'));

        $incidentsTable = new Giscuit_Db_Table_Layer_Incidents();
        
        if ($incidentType == 1) {
            $records = $incidentsTable->getItems(array(
                'whereActive' => 't' ,
                'whereCategory' => $incidentCategory
            ));
        } else if ($incidentType == 2) {
            $records = $incidentsTable->getItems(array(
                'whereActive' => 'f' ,
                'whereCategory' => $incidentCategory
            ));
        } else {
            $records = $incidentsTable->getItems(array('whereCategory' => $incidentCategory));
        }
        $records = $records->toArray();

        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
        for ($i = 0; $i < count($records); $i++) {
            if ($records[$i]['image_id'] != NULL) {
                $fileLink = $url->url(array(
                    'language' => Zend_Registry::get('Zend_Translate')->getLocale() , 
                    'module' => 'default' ,
                    'controller' => 'uploads' , 
                    'action' => 'index' ,
                    'id' => $records[$i]['image_id']
                ), null, true);
                $records[$i]['image_url'] = $fileLink;
                $records[$i]['image'] = "<a href=\"javascript: void(0);\" title=\"{$records[$i]['image_name']}\" " .
                    " onclick=\"Giscuit.Util.photobox('{$records[$i]['image_name']}', '{$fileLink}');\">" .
                    "<img src=\"{$this->view->staticPath('/camera_photo.png', 'img')}\" alt=\"{$records[$i]['image_name']}\"></a>";
            }
            $records[$i]['category_name'] = $records[$i]['name'];
            unset($records[$i]['name']);
            if (! $isAllowed->direct('tool_incidents_controller', 'setstatus')) {
                unset($records[$i]['fname']);
                unset($records[$i]['lname']);
                unset($records[$i]['email']);
            }
        }

        /*
        if ($isAllowed->direct('tool_incidents_controller', 'setstatus')) {
            $result['allowManageIncidents'] = 't';
        } else {
            $result['allowManageIncidents'] = 'f';
        }
        */
        
        $result['records'] = $records;

        $this->_helper->json($result);
    }
    
    public function addAction()
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        $config = Zend_Registry::get('config');
        $form = $this->getIncidentsForm();
        
        if ($request->isPost()) {
            $theGeomElement = $form->createElement('text', 'the_geom');
            $theGeomElement->setRequired(true);
            $theGeomElement->addValidator('Geometry');
            $form->addElement($theGeomElement);
            
            //Remove file element if $_FILES is empty
            if(empty($_FILES)) {   
                $form->removeElement('image');
            }
            
            $form->populate($request->getParams());

            if ($form->isValid(array_merge($request->getParams(), array('the_geom' => $request->getParam('geometry'))))) {
                $incidentsTable = new Giscuit_Db_Table_Layer_Incidents();
                $notificationsTable = new Giscuit_Db_Table_Layer_IncidentsNotifications();
                
                $incidentGid = $incidentsTable->addItem($form);
                
                if ($config->smtp->host && $config->smtp->port) {
                    $records = $notificationsTable->getItems(array(
                        'whereGeomIntersects' => $form->getValue('the_geom'),
                        'whereCategoryId' => $form->getValue('category_id'),
                        'withEmail' => true
                    ));
                    
                    $data = $incidentsTable->getItems(array(
                        'whereGid' => $incidentGid,
                        'withCentroid' => true
                    ))->current();

                    $this->sendMailNotification($records, $data);
                }
            } else {
                $form->removeElement('the_geom');
                
                $this->view->form = $form;
                $result['form'] = $this->view->render('incidents' . DIR_SEP . 'getform' . GT_SCRIPT_EXT);
            }
            
        } else {
            $this->view->form = $form;
            $result['form'] = $this->view->render('incidents' . DIR_SEP . 'getform' . GT_SCRIPT_EXT);
        }
        
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        if(GT_TESTING == false) {
            //FIXME: Avoid converting to html entities by the browser
            header("Content-Type: text/plain");
        }
        
        //FIXME: Json helper does not work file uploads
        echo Zend_Json::encode($result);
    }
    
    public function setstatusAction()
    {
        $result['replyCode'] = 200;
        $config = Zend_Registry::get('config');
        
        $request = $this->getRequest();
        $gid = intval($request->getParam('gid'));
        
        $incidentsTable = new Giscuit_Db_Table_Layer_Incidents();
        $editedGid = $incidentsTable->editStatus($gid);
        
        if ($config->smtp->host && $config->smtp->port) {
            $editedRecord = $incidentsTable->getItems(array('whereGid' => $editedGid))->current();
            $this->sendIncidentStatusMail($editedRecord);
        }
        
        $this->_helper->json($result);
    }
    
    public function deleteAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $gid = intval($request->getParam('gid'));

        $incidentsTable = new Giscuit_Db_Table_Layer_Incidents();
        $where = $incidentsTable->getAdapter()->quoteInto('gid = ?', $gid);
        $incidentsTable->delete($where);
        
        $this->_helper->json($result);
    }
    
    public function getnotificationsAction()
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        $notificationsTable = new Giscuit_Db_Table_Layer_IncidentsNotifications();
        $categoryTable = new Giscuit_Db_Table_Layer_IncidentsCategory();
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $userNotifications = $notificationsTable->getItems(array('whereId' => $userId));
        $categoryRecords = $categoryTable->getItems(array('withName' => true));
        
        $categories = array();
        foreach ($categoryRecords as $key => $categoryArr) {
            $category['label'] = $categoryArr['name'];
            $category['value'] = $categoryArr['id'];
            $categories[] = $category;
        }
        
        $result['records'] = $userNotifications->toArray();
        $result['categories'] = $categories;
        
        $this->_helper->json($result);
    }
    
    public function addnotificationAction()
    {
        $result['replyCode'] = 200;
        
        $notificationsTable = new Giscuit_Db_Table_Layer_IncidentsNotifications();
        
        $data['user_id'] = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $data['id'] = $notificationsTable->addItem($data);
        
        $result['record'] = $data;
        
        $this->_helper->json($result);
    }
    
    public function updatenotificationAction()
    {
        $result['replyCode'] = 200;
        $translate = Zend_Registry::get('Zend_Translate');
        
        $params = $this->getRequest()->getParams();
        $notificationsTable = new Giscuit_Db_Table_Layer_IncidentsNotifications();
        
        $trimFilter = new Zend_Filter_StringTrim();
        foreach($params as $param) {
            $trimFilter->filter($param);
        }
        
        $cols = $notificationsTable->info('cols');
        $columnValidator = new Zend_Validate_InArray($cols);
        if (! $columnValidator->isValid($params['column'])) {
            $result['replyCode'] = 500;
            $errors = $columnValidator->getErrors();
            $result['error'] = $translate->_('error') . ': ' . $errors[0];
        }
        
        if ($params['column'] == 'category_id') {
            $incidentsCategoryTable = new Giscuit_Db_Table_Layer_IncidentsCategory();
            $categoryItems = $incidentsCategoryTable->getItems()->toArray();
            $categories = array();
            for ($i = 0; $i < count($categoryItems); $i++) {
                $categories[] = $categoryItems[$i]['id'];
            }
            
            $categoryValidator = new Zend_Validate_InArray($categories);
            if (! $categoryValidator->isValid($params['data'])) {
                $result['replyCode'] = 500;
                $errors = $categoryValidator->getErrors();
                $result['error'] = $translate->_('error') . ': ' . $errors[0];
            }
        } else if ($params['column'] == 'the_geom') {
            if (! empty($params['data'])) {
                $geomValidator = new Giscuit_Validate_Geometry();
                if (! $geomValidator->isValid($params['data'])) {
                    $result['replyCode'] = 500;
                    $errors = $geomValidator->getErrors();
                    $result['error'] = $translate->_('error') . ': ' . $errors[0];
                }
            }
        } else {
            throw new Giscuit_Exception("Unknown column: " . $params['column']);
        }
        
        $params['id'] = intval($params['id']);
        
        if (! isset($result['error'])) {
            $notificationsTable->editItem($params['id'], $params);
        }
        
        $this->_helper->json($result);
    }
    
    public function deletenotificationAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $id = intval($request->getParam('id'));
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();

        $notificationsTable = new Giscuit_Db_Table_Layer_IncidentsNotifications();
        $notificationsTable->delete("id = {$id} AND user_id = {$userId}");
        
        $this->_helper->json($result);
    }
    
    public function getcategoriesAction()
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        if ($request->getParam('onlycolumns')) {
            $result['records'] = Zend_Registry::get('config')->language->toArray();
        } else {
            $categoryTable = new Giscuit_Db_Table_Layer_IncidentsCategory();
            
            $categoryRecords = $categoryTable->getItems(array('withAllNames' => true));

            $result['records'] = $categoryRecords->toArray();
            $result['currentKey'] = Zend_Registry::get('Zend_Translate')->getLocale();
        }

        $this->_helper->json($result);
    }
    
    public function addcategoryAction()
    {
        $result['replyCode'] = 200;

        $categoryTable = new Giscuit_Db_Table_Layer_IncidentsCategory();
        
        $result['record'] = $categoryTable->addItem();
        
        $this->_helper->json($result);
    }
    
    public function updatecategoryAction()
    {
        $result['replyCode'] = 200;
        
        $categoryTable = new Giscuit_Db_Table_Layer_IncidentsCategory();
        $textTable = new Giscuit_Db_Table_Text();
        
        $cols = array_merge($categoryTable->info('cols'), $textTable->info('cols'));
        
        $filters = array(
            '*' => 'StringTrim'
        );
        
        $validators = array(
            'column' => new Zend_Validate_InArray($cols) , 
            'data' => array(
                'allowEmpty' => true
            ) , 
            'id' => 'Digits'
        );
        
        $input = new Zend_Filter_Input($filters, $validators, $this->getRequest()->getParams());
        
        if ($input->isValid()) {
            if ($input->column == 'color') {
                $colorValidator = new Giscuit_Validate_Color();
                if (! $colorValidator->isValid($input->data)) {
                    $input->data = '#000000';
                }
                $categoryTable->editItem($input->id, $input);
            } else {
                $rows = $textTable->find($input->id);
                
                if ($rows->count() == 0) {
                    throw new Giscuit_Exception("Invalid text id :" . $input->id);
                }
                
                $row = $rows->current();
                $column = $input->column;
                $row->$column = $input->data;
                $row->save();
            }
        } else {
            $result['replyCode'] = 500;
            $result['errors'] = $input->getMessages();
        }
        
        $this->_helper->json($result);
    }
    
    public function deletecategoryAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $id = intval($request->getParam('id'));

        $categoryTable = new Giscuit_Db_Table_Layer_IncidentsCategory();
        $where = $categoryTable->getAdapter()->quoteInto('id = ?', $id);
        $categoryTable->delete($where);
        
        $this->_helper->json($result);
    }
    
    public function sendMailNotification($recipients, $data)
    {
        $config = Zend_Registry::get('config');
        
        $latLon = substr($data->centroid, 6);
        $latLon = substr($latLon, 0, strlen($latLon) - 1);
        $latLon = explode(' ', $latLon);

        $mailView = new Zend_View();
        $mailView->host = $_SERVER['HTTP_HOST'];
        $mailView->fname = $data->fname;
        $mailView->lname = $data->lname;
        $mailView->zoom = count(explode(',', $config->map->resolutions)) - 3;
        $mailView->lon = $latLon[0];
        $mailView->lat = $latLon[1];
        $mailView->title = $data->title;
        $mailView->text = $data->description;
        
        if (! empty($data->image_id)) {
            $mailView->image_id = $data->image_id;
        }

        $mailView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'tool' . DIR_SEP . 
            'views' . DIR_SEP . 'scripts' . DIR_SEP . 'incidents');
        
        $mailBody = $mailView->render('add_mail' . GT_SCRIPT_EXT);

        $mailArr = array();
        for ($i = 0; $i < count($recipients); $i++) {
            if ($recipients[$i]['email_active'] == 'f') {
                continue;
            }
            $mailData['to'] = $recipients[$i]['email'];
            $mailData['subject'] = $data->title;
            $mailData['text'] = $mailBody;
            $mailArr[] = $mailData;
        }
        
        if (count($mailArr) > 0) {
            Giscuit_Mail::batch($mailArr);
        }
    }
    
    public function sendIncidentStatusMail($data)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        if ($data->active == 'f') {
            $mailTitle = vsprintf($translate->_('reportClosed'), '');
            $mailText = vsprintf($translate->_('reportClosed'), '"' . $data->title . '"');
        } else {
            $mailTitle = vsprintf($translate->_('reportReopened'), '');
            $mailText = vsprintf($translate->_('reportReopened'), '"' . $data->title . '"');
        }
        
        $mailView = new Zend_View();
        $mailView->text = $mailText;
        
        $mailView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'tool' . DIR_SEP . 
            'views' . DIR_SEP . 'scripts' . DIR_SEP . 'incidents');
        
        $mailBody = $mailView->render('close_mail' . GT_SCRIPT_EXT);

        Giscuit_Mail::batch(array(
            array(
                'to' => $data->email ,
                'subject' => $mailTitle , 
                'text' => $mailBody
            )
        ));
    }
    
    public function getIncidentsForm() 
    {
        $form = new Giscuit_Form_Layer_Incidents();
        $form->setAttrib('enctype', 'multipart/form-data');
        
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtIncidentsForm'
        ));

        return $form;
    }
}