<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: InfoController.php 3514 2013-12-03 15:29:38Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Tool
 * @copyright  Copyright (c) 2012 VEC
 */

class Tool_InfoController extends Zend_Controller_Action
{

    protected $_maxResultsPerLayer = 5;
    
    protected $_toleranceModifier = 0;

    public function indexAction ()
    {
        $request = $this->getRequest();
        $x = floatval($request->getParam('x'));
        $y = floatval($request->getParam('y'));
        $lon = floatval($request->getParam('lon'));
        $lat = floatval($request->getParam('lat'));
        $width = floatval($request->getParam('width'));
        $height = floatval($request->getParam('height'));
        $bbox = array_map('floatval', explode(',', $request->getParam('bbox')));
        
        $result['replyCode'] = 200;
        $result['records'] = array();
        
        $layerTable = new Giscuit_Db_Table_Layers();
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        if(! empty($userId)) {
            $mapOptions = array(
                'orderDirection' => 'asc' ,
                'whereActive' => 't' , 
                'whereSelect' => 't' ,
                'withLayerSettings' => true , 
                'withAcl' => true , 
                'withFields' => true , 
            );
        } else {
            $layers = explode(',', $request->getParam('layers'));
            
            $mapOptions = array(
                'orderDirection' => 'asc' ,
                'wherePublish' => 't' , 
                'whereActive' => 't' , 
                'withFields' => true , 
                'withPublish' => true
            );
        }
        
        $rows = $layerTable->getItems($mapOptions);
             
        foreach ($rows as $row) {
            if (empty($userId) && ! in_array($row['class'], $layers)) {
                continue;
            }
            
            $record = $this->getByPoint($row, $x, $y, $lon, $lat, $width, $height, $bbox);
                      
            if ($record) {
                $result['records'] = array_merge($result['records'], $record);
            }
        }
        
        $this->_helper->json($result);
    
    }

    public function parseResult ($result)
    {
        return $result;
    }

    public function getByPoint ($record, $x, $y, $lon, $lat, $width, $height, $bbox)
    {
        $db = Zend_Registry::get('db');
        $config = Zend_Registry::get('config');

        $request = $this->getRequest();
        
        $result = array();
        $translate = Zend_Registry::get('Zend_Translate');
        
        $layerName = 'Giscuit_Map_Layer_' . $record['class'];
        
        if (class_exists('Giscuit_Map_Layer_Vector') && class_exists($layerName) && is_subclass_of($layerName, 'Giscuit_Map_Layer_Vector')) {
        
            if(empty($record['default_field'])) {
                return false;    
            }
            
            $deniedFields = array(
                'gid' , 
                'the_geom'
            );
            
            //Table
            $tableName = 'Giscuit_Db_Table_Layer_' . $record['class'];
            
            $tableOptions = array(
                'name' => $record['table']
            );
            
            if(! empty(Zend_Registry::get('settings')->historyDateEnd)) {
                $tableOptions['withHistory'] = true;
            }
            $table = new $tableName($tableOptions);
            $table->setSettings($record);
            
            //Form
            $formName = 'Giscuit_Form_Layer_' . $record['class'];
            $formConfigPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $record['class'] . '.xml';
            if (file_exists($formConfigPath)) {
                $configContent = new Zend_Config_Xml($formConfigPath);
                $form = new $formName($configContent);
            } else {
                $form = new $formName();
            }

            //Prepare group data
            $groups = array();
            $displayGroups = $form->getDisplayGroups();
            foreach($displayGroups as $group) {
                $groupData = $group->getAttribs();
                $groupData['fields'] = array();
                foreach($group->getElements() as $groupEl) {
                    $groupData['fields'][] = $groupEl->getName();
                }

                $groups[$group->getName()] = $groupData;
            }

            if (empty($record['tolerance'])) {
                $record['tolerance'] = 5;
            }
            $buffer = (($bbox[2] - $bbox[0]) / $width) * ($record['tolerance'] + $this->_toleranceModifier);
            
            $rows = $table->getByPoint($lon, $lat, $buffer, array(
                'limit' => $this->_maxResultsPerLayer ,
                'form' => $form
            ))->toArray();
            
            if (count($rows) == 0) {
                return false;
            }
            
            $fields = $table->getPublicFields();
            $defaultField = $table->getDefaultField();
            
            //Build yui tree
            $result = new stdClass();
            $result->type = 'HTML';
            $result->html = (string) $record['name'];
            $result->expanded = true;
            $result->children = array();

            //Fetch subform data
            $groupResult = array();
            foreach($groups as $groupKey => $group) {
                if(! (isset($group['subform']) && $group['subform'])) {
                    continue;
                }

                //Gid list
                $gidList = array();
                foreach ($rows as $row) {
                    $gidList[] = $row['gid'];
                }
                $gidListStr = implode(', ', $gidList);

                //Select
                $groupFields = $group['fields'];
                $groupFields[] = 'id';
                $groupFields[] = 'parent_id';

                $tableName = $config->database->schema . '_layers.' . $group['database']['table'];
                $subformSelect = $db->select();
                $subformSelect->from($tableName, $groupFields);
                $subformSelect->where("parent_id IN ({$gidListStr})");

                //Result
                $groupResult[$groupKey] = $db->fetchAssoc($subformSelect);
            }

            foreach ($rows as $row) {
                                
                $item = new stdClass();
                $item->type = 'HTML';
                if (! empty($row[$defaultField])) {
                    //Escape
                    if(! (isset($fields[$defaultField]['escape']) && $fields[$defaultField]['escape'] == 'f')) {
                        $item->html = htmlspecialchars($row[$defaultField], ENT_QUOTES); 
                    } else {
                        $item->html = (string) $row[$defaultField];
                    }
                } else {
                    $item->html = $translate->_('emptyField');
                }
                $item->the_geom = $row['the_geom'];
                $item->children = array();

                //Build group item
                foreach($groups as $groupKey => $group) {
                    //
                    $groupItem = new stdClass();
                    $groupItem->type = 'HTML';
                    $groupItem->children = array();
                    $groupItem->html = $translate->_($group['legend']);

                    //
                    if(isset($group['subform']) && $group['subform']) {
                        foreach($groupResult[$groupKey] as $groupValues) {

                            if($groupValues['parent_id'] != $row['gid']) {
                                continue;
                            }

                            //FIXME: Add the ability to specify the default field from layer.xml
                            $groupFeatureFirstVal = reset($groupValues);

                            //Build feature group
                            $groupFeature = new stdClass();
                            $groupFeature->type = 'HTML';
                            $groupFeature->children = array();
                            $groupFeature->html = $groupFeatureFirstVal;

                            //Build feature items
                            foreach($groupValues as $groupAttrKey => $groupAttrValue) {
                                if($groupAttrKey == 'id' || $groupAttrKey == 'parent_id') {
                                    continue;
                                }

                                $groupAttr = new stdClass();
                                $groupAttr->type = 'HTML';
                                $groupAttr->hasIcon = false;
                                $table->formatInfoResults($groupAttr, $groupAttrKey, $groupAttrValue, $form);


                                $groupFeature->children[] = $groupAttr;
                            }

                            //
                            $groupItem->children[] = $groupFeature;
                        }
                    }

                    //
                    $item->children[] = $groupItem;

                    //
                    $groups[$groupKey]['item'] = $groupItem;
                }

                foreach ($row as $field => $value) {
                    $itemRoot = &$item->children;

                    //Check if fields are a part of a group
                    foreach($groups as $group) {
                        if(in_array($field, $group['fields'])) {
                            $itemRoot = &$group['item']->children;
                        }
                    }

                    //Build item
                    if($field != 'gid' 
                        && $field != 'the_geom'
                        && substr($field, 0, 10) != '__filename') {
                        
                        $itemField = new stdClass();
                        $itemField->type = 'HTML';
                        $itemField->hasIcon = false;
                        
                        //Escape
                        if(! (isset($fields[$field]['escape']) && $fields[$field]['escape'] == 'f')) {
                            $value = htmlspecialchars($value, ENT_QUOTES);       
                        }
                        
                        //
                        if($form->getElement($field) instanceof Zend_Form_Element_File) {
                            $value = array(
                                'id' => $value ,
                                'name' => $row['__filename_' . $field]
                            );
                        }
                        
                        $table->formatInfoResults($itemField, $field, $value, $form);
                        
                        if($itemField->html) {
                            $itemRoot[] = $itemField;
                        }

                    }

                }

                $result->children[] = $item;
            }
            
            return array(
                $result
            );
        
        } elseif (class_exists('Giscuit_Map_Layer_Raster') && class_exists($layerName) && is_subclass_of($layerName, 'Giscuit_Map_Layer_Raster')) {
            
            if($record['query'] == 'f') {
                return false; 
            }
            
            $map = $this->_getMap($bbox, $width, $height);
            
            $layer = new $layerName($map, $record);
                        
            $rows = $layer->getByPoint($lon, $lat);
            
            if (count($rows) == 0) {
                return false;
            }
            
            //Build yui tree
            $result = new stdClass();
            $result->type = 'HTML';
            $result->html = (string) $record['name'];
            $result->expanded = true;
            $result->children = array();
            
            $item = new stdClass();
            $item->type = 'HTML';
            
            $item->html = $translate->_('coordinates') . ': ' . $rows['x'] . ' ' . $rows['y'];
            
            $item->children = array();
            
            foreach ($rows['values'] as $key => $value) {
                $itemField = new stdClass();
                $itemField->type = 'HTML';
                
                if(is_integer($key)) {
                    $itemField->html = $translate->_('band') . ' ' . $key . ': ' . $value;    
                } else {
                    $itemField->html = $translate->_($key) . ': ' . $value;
                }
                
                $itemField->hasIcon = false;
                
                $item->children[] = $itemField;
            }
            
            $result->children[] = $item;
            
            return array(
                $result
            );
        
        } elseif (class_exists('Giscuit_Map_Layer_Wms') && class_exists($layerName) && is_subclass_of($layerName, 'Giscuit_Map_Layer_Wms')) {
            
            if($record['query'] == 'f') {
                return false; 
            }
            
            $map = $this->_getMap($bbox, $width, $height);
            
            $layer = new $layerName($map, $record);
            
            //Get WMS feature info
            $wmsOptions = array_merge(
                stream_context_get_options(stream_context_get_default()),
                array(
                    'http' => array(
                        'method' => 'GET',
                        'header' => "Accept-language: en\r\n",
                        'request_fulluri' => true
                    )
                )
            );
            if($record['owsclient_auth_type']) {
                $wmsCredentials = base64_encode($record['owsclient_auth_username'] . ':' . $record['owsclient_auth_password']);
                $wmsOptions['http']['header'] .= "Authorization: Basic " . $wmsCredentials . "\r\n";
            }
            $context = stream_context_create($wmsOptions);
            $wmsUrl = $layer->getWMSFeatureInfoURL($x, $y, NULL, $record['owsclient_info_format']);
            $contents = file_get_contents($wmsUrl, false, $context);
            
            //Format feature info
            if($record['owsclient_info_format'] == 'text/plain') {
                $contents = trim('<pre>' . $contents . '</pre>');
            }
                       
            //Build yui tree
            $result = new stdClass();
            $result->type = 'HTML';
            $result->html = (string) $record['name'];
            $result->expanded = false;
            $result->children = array();
            
            $item = new stdClass();
            $item->type = 'HTML';
            $item->html = $contents;
            
            $result->children[] = $item;
            
            return array(
                $result
            );
            
        }
    
    }
    
    protected function _getMap ($bbox, $width, $height)
    {
        $map = new Giscuit_Map_Map(GT_CONFIG_PATH . DIR_SEP . 'default.map');
        $map->setParamsFromConfig();
        
        $map->setExtent(array(
            'min' => array(
                'x' => $bbox[0] , 
                'y' => $bbox[1]
            ) , 
            'max' => array(
                'x' => $bbox[2] , 
                'y' => $bbox[3]
            )
        ));
        
        $map->setSize(array(
            'width' => $width , 
            'height' => $height
        ));
        
        return $map;
    }
    
}