<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Array2dToHierarchy.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Helper
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see  Zend_Controller_Action_Helper_Abstract
 */
require_once 'Zend/Controller/Action/Helper/Abstract.php';

/**
 * Convert a 2d array into a hierarchical array
 * 
 * @category   Giscuit
 * @package    Giscuit_Helper
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Action_Helper_Array2dToHierarchy extends Zend_Controller_Action_Helper_Abstract
{
    /**
     * Id field name
     * 
     * @var unknown_type
     */
    protected $_idField = 'id';

    /**
     * Parent id field name
     * 
     * @var unknown_type
     */
    protected $_parentField = 'parent_id';

    /**
     * Filter
     * 
     * @var unknown_type
     */
    protected $_filter;

    /**
     * Convert a 2d array into a hierarchical array
     * 
     * @param array $data
     * @param array $options Available options
     * @return array
     */
    public function array2dToHierarchy ($data, $options = null)
    {
        //Init
        $this->_filter = 0;
        $startDepth = 0;
        $maxDepth = - 1;
        $parent = 0;
        $arr = array();
        
        //Options
        if (! empty($options['idField'])) {
            $this->_idField = $options['idField'];
        }
            
        if (! empty($options['parentIdField'])) {
            $this->_parentField = $options['parentField'];
        }
            
        if (! empty($options['startFrom'])) {
            $parent = $options['startFrom'];
        }
            
        if (! empty($options['maxDepth'])) {
            $maxDepth = $options['maxDepth'];
        }
            
        if (! empty($options['startDepth'])) {
            $startDepth = $options['startDepth'];
        }
            
        $this->createNestedArray($data, $arr, $parent, $startDepth, $maxDepth);
        
        return $arr;
    
    }

    /**
     * Create the nested array
     * 
     * @param array $data
     * @param array $arr
     * @param array $parent
     * @param integer $startDepth
     * @param integer $maxDepth
     * @return array
     */
    protected function createNestedArray (&$data, &$arr, $parent, $startDepth, $maxDepth)
    {
        
        if ($maxDepth -- == 0) {
            return;
        }
        
        $index = 0;
        $startDepth ++;
        
        $this->_filter = $parent;
        
        $children = array_filter($data, array(
            $this , 
            'filterMethod'
        ));
        foreach ($children as $child) {
            
            $child['id'] = (string) $child['id'];
            
            $arr[$index] = $child;
            $arr[$index]['depth'] = $startDepth;
            
            $this->createNestedArray($data, $arr[$index]['pages'], $child[$this->_idField], $startDepth, $maxDepth);
            
            $index ++;
        }
    }
    
    /**
     * Strategy pattern: call helper as helper broker method
     *
     * Convert a 2d array into a hierarchical array
     * 
     * @param array $data
     * @param array $options Available options
     * @return array
     */
    public function direct ($data, $options = null)
    {
        return $this->array2dToHierarchy($data, $options);
    }
}