<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for coordinates import
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Coords extends Giscuit_Controller_Plugin_Abstract
{
    /**
     * Plugin display name
     *
     * @var string
     */
    public static $label = 'Quick import';
    
    protected $_translations = array(
        'en' => array(
            'text' => 'Text',
            'gpx' => 'GPX',
            'kml' => 'KML',
            'csv' => 'CSV',
            'coords' => 'Import coordinates',
            'coordsShort' => 'Import coords.',
            'xCoord' => 'X (Longitude)',
            'yCoord' => 'Y (Latitude)',
            'label' => 'Label',
            'coordinatesIn' => 'Coordinates in',
            'wgs' => 'WGS 84',
            'mapProjection' => '<a target="_blank" href="http://www.epsg-registry.org/report.htm?type=selection&entity=urn:ogc:def:crs:%1$s&reportDetail=short&style=urn:uuid:report-style:default-with-code&style_name=OGP%20Default%20With%20Code&title=%2$s">%2$s</a>',
            'selectFile' => 'Select file',
            'csvFormatWarning' => 'CSV data should be in the following format (no header): Longitude,Latitude,Label',
            'import' => 'Import'
        ),
        'ro' => array(
            'text' => 'Text',
            'gpx' => 'GPX',
            'kml' => 'KML',
            'csv' => 'CSV',
            'coords' => 'Importă coordonatele',
            'coordsShort' => 'Importă coord.',
            'xCoord' => 'X (Longitudine)',
            'yCoord' => 'Y (Latitudine)',
            'label' => 'Denumire',
            'coordinatesIn' => 'Coordonatele în',
            'wgs' => 'WGS 84',
            'mapProjection' => '<a target="_blank" href="http://www.epsg-registry.org/report.htm?type=selection&entity=urn:ogc:def:crs:%1$s&reportDetail=short&style=urn:uuid:report-style:default-with-code&style_name=OGP%20Default%20With%20Code&title=%2$s">%2$s</a>',
            'selectFile' => 'Selectați fișier',
            'csvFormatWarning' => 'Datele CSV trebuie să fiu în format următor (fără antet): Longitudine, Latitudine, Denumire',
            'import' => 'Importarea'
        ),
        'ru' => array(
            'text' => 'Текст',
            'gpx' => 'GPX',
            'kml' => 'KML',
            'csv' => 'CSV',
            'coords' => 'Импорт координат',
            'coordsShort' => 'Импорт коорд.',
            'xCoord' => 'X (Долгота)',
            'yCoord' => 'Y (Широта)',
            'label' => 'Название',
            'coordinatesIn' => 'Координаты в формате',
            'wgs' => 'WGS 84',
            'mapProjection' => '<a target="_blank" href="http://www.epsg-registry.org/report.htm?type=selection&entity=urn:ogc:def:crs:%1$s&reportDetail=short&style=urn:uuid:report-style:default-with-code&style_name=OGP%20Default%20With%20Code&title=%2$s">%2$s</a>',
            'selectFile' => 'Выберите файл',
            'csvFormatWarning' => 'CSV данные должны быть в следующем формате (без заголовка): Долгота, Широта, Название',
            'import' => 'Загрузить'
        )
    );
    
    /**
     * List of the javascript files to include
     * 
     * @return array
     */
    public function getJsFilesList ()
    {
        return array(
            'Control/Coords.js' 
        );
    }
    
    /**
     * List of the javascript files to include for mobile
     * 
     * @return array
     */
    public function getMobileJsFilesList ()
    {
        return array(
            'Control/Coords.js',
            'Mobile/Control/Coords.js' 
        );
    }
    
    /**
     * Install plugin
     * 
     * @return boolean
     */
    public function install($newConfig) 
    {
        $rulesTable = new Giscuit_Db_Table_Rules();
        $data = array(
            'role' => 'guest',
            'resource' => 'tool_coords_controller',
            'privilege' => '',
            'group' => 'controller'
        );
        $rulesTable->insert($data);
        
        //Add default section translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
        
            $newMessages = array_merge($translate->getMessages(), $languageValue);
            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
            
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Uninstall plugin
     *  
     * @return boolean
     */
    public function uninstall($newConfig) 
    {
        $rulesTable = new Giscuit_Db_Table_Rules();
        $where = $rulesTable->getAdapter()->quoteInto('resource = ?', 'tool_coords_controller');
        $rulesTable->delete($where);
        
        //Delete translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
            
            $translations = $translate->getMessages();
            foreach ($languageValue as $key => $value) {
                unset($translations[$key]);
            }

            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $translations);
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }

    /**
     * Get parameters
     * 
     * @return array
     */
    public function getParams()
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        return array(
            'coordsPluginIsAllowed' => $isAllowed->direct('tool_coords_controller') ? true : false
        );    
    }
}