<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for Csw
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Csw extends Giscuit_Controller_Plugin_Abstract
{
    /**
     * Plugin display name
     *
     * @var string
     */
    public static $label = 'Catalog';

    protected $_translations = array(
        'en' => array(
            'images' => 'Images',
            'catalog' => 'Catalog' , 
            'what' => 'What' , 
            'sort' => 'Sort' , 
            'alphabetically' => 'Alphabetically' , 
            'newestFirst' => 'Newest first' , 
            'oldestFirst' => 'Oldest first' , 
            'cswHome' => 'Home' , 
            'popularKeywords' => 'Popular keywords' , 
            'noRecordsFound' => 'No records are found' , 
            'metadataFileIdentifier' => 'Metadata file identifier' , 
            'keywords' => 'Keywords' , 
            'description' => 'Description' , 
            'modifiedDate' => 'Modified date' , 
            'webAddress' => 'Web address' , 
            'addWmsLayerToMap' => 'Add WMS layer to map' , 
            'source' => 'Source' , 
            'downloadAsXML' => 'Download as XML' , 
            'license' => 'License' , 
            'seeMore' => 'See more' , 
            'recentlyAdded' => 'Recently added',
            'welcomeMessage' => 'Welcome to our catalogue of geospatial data. You can use this service to find out how to access geo-referenced information resources. Geo-referenced information is information that includes reference to a place on, about, or below the Earth\'s surface.<br>
                Do you want to harvest these metadata records into your own catalogue using Catalog Services for the Web (CSW)? If yes, use <a href="%1$s" target="_blank">this service URL</a>.'
        ),
        'ro' => array(
            'images' => 'Imagini',
            'catalog' => 'Catalog' , 
            'what' => 'Ce' , 
            'sort' => 'Sortare' , 
            'alphabetically' => 'Alfabetic' , 
            'newestFirst' => 'Cele mai recente primele' , 
            'oldestFirst' => 'Cele mai vechi primele' , 
            'cswHome' => 'Acasă' , 
            'popularKeywords' => 'Cuvinte cheie populare' , 
            'noRecordsFound' => 'Date nu au fost găsite' , 
            'metadataFileIdentifier' => 'Identificator fişierului metadatelor' , 
            'keywords' => 'Cuvinte cheie' , 
            'description' => 'Descriere' , 
            'modifiedDate' => 'Data modificării' , 
            'webAddress' => 'Adresa web' , 
            'addWmsLayerToMap' => 'Adaugă strat WMS pe hartă' , 
            'source' => 'Sursă' , 
            'downloadAsXML' => 'Descracă în XML' , 
            'license' => 'Licenţa' , 
            'seeMore' => 'Vizualizează mai mult' , 
            'recentlyAdded' => 'Adăugate recent' ,
            'welcomeMessage' => 'Bine venit la catalogul nostru de date geospaţiale. Puteţi folosi acest serviciu pentru a afla cum de accesat resurse informaţionale geo-referenţiate. Informaţia geo-referenţiată este informaţia ce include referenţe spre locaţia pe, împrejur sau dedesubt suprafeţei Pământului.<br>
                Doriţi să colectaţi aceste metadate în catalogul dumneavoastră folosind Web Servicii pentru Catalog (CSW)? Dacă da, folosiţi <a href="%1$s" target="_blank">URL acestui serviciu</a>.'
        ),
        'ru' => array(
            'images' => 'Изображения',
            'catalog' => 'Каталог' , 
            'what' => 'Что' , 
            'sort' => 'Сортировать' , 
            'alphabetically' => 'В алфавитном порядке' , 
            'newestFirst' => 'Самые новые первые' , 
            'oldestFirst' => 'Самые старые первые' , 
            'cswHome' => 'Домой' , 
            'popularKeywords' => 'Популярные ключевые слова' , 
            'noRecordsFound' => 'Результатов не найдено' , 
            'metadataFileIdentifier' => 'Идентификатор файла метаданных' , 
            'keywords' => 'Ключевые слова' , 
            'description' => 'Описание' , 
            'modifiedDate' => 'Дата изменения' , 
            'webAddress' => 'Адрес веб' , 
            'addWmsLayerToMap' => 'Добавить WMS слой на карту' , 
            'source' => 'Источник' , 
            'downloadAsXML' => 'Скачать в XML' , 
            'license' => 'Лицензия' , 
            'seeMore' => 'Подробная информация' , 
            'recentlyAdded' => 'Недавно добавленные' ,
            'welcomeMessage' => 'Добро пожаловать в наш каталог геопространственных данных. Вы можете воспользоваться этой услугой, чтобы узнать, как получить доступ к геосправочным информационным ресурсам. Гео-справочной информацией является информация, которая включает в себя ссылку на место, около или ниже поверхности Земли.<br>
                Вы хотите собрать эти записи метаданных в свой каталог с помощью Каталога Услуг для Веб (CSW)? Если да, то используйте <a href="%1$s" target="_blank">URL этой услуги</a>.'
        )
    );
    
    /**
     * List of the javascript files to include
     * 
     * @return array
     */
    public function getJsFilesList ()
    {
        return array(
            'Control/Csw.js'
        );
    }
    
    /**
     * Install plugin
     * 
     * @return boolean
     */
    public function install($newConfig) 
    {
        //Set plugin inactive after install
        $pluginName = $this->name;
        $newConfig->production->plugins->$pluginName->url = '';
        $newConfig->production->plugins->$pluginName->brand = '';
        $newConfig->production->plugins->$pluginName->active = 'f';
        
        //Add menu entry
        $menuTable = new Giscuit_Db_Table_Menu();
        $textTable = new Giscuit_Db_Table_Text();
        $newRow = $menuTable->createRow();
        
        $languages = array_keys(Zend_Registry::get('config')->language->toArray());
        
        $titleRow = $textTable->createRow();
        foreach ($languages as $lang) {
            $titleRow->$lang = 'Catalog';
        }
        $newRow->title_id = $titleRow->save();
                   
        $textRow = $textTable->createRow();
        $newRow->text_id = $textRow->save();
                   
        $tagsRow = $textTable->createRow();
        $newRow->tags_id = $tagsRow->save();
           
        $newRow->url = '<front>/map#ac=csw';
        $newRow->active = 't';
        $newRow->type = 3;
        $newRow->target = 'f';
        $newRow->resource = 'map_menu';
           
        $newRow->save();
        $newRow->order = $newRow->id;
        $newRow->save();
        
        //Add default section translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
        
            $newMessages = array_merge($translate->getMessages(), $languageValue);
            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
            
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Uninstall plugin
     *  
     * @return boolean
     */
    public function uninstall($newConfig) 
    {
        //Delete translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
            
            $translations = $translate->getMessages();
            foreach ($languageValue as $key => $value) {
                unset($translations[$key]);
            }

            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $translations);
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
        
        //Remove menu entry
        $menuTable = new Giscuit_Db_Table_Menu();
        $select = $menuTable->select();
        $select->where('url = ?', '<front>/map#ac=csw');
        $row = $menuTable->fetchRow($select);
           
        $menuTable->delete('id = ' . $row->id);
    }
    
    /**
     * Parse settings in form
     *  
     * @return boolean
     */
    public function parseSettingsForm($form) {
        $settings = $this->getSettings();
        $url = $settings['url'];
        $brand = $settings['brand'];
        
        //url
        $form->addElement('text', 'cswUrl', array(
            'required' => true , 
            'label' => 'url',
            'validators' => array(
                array(
                    'NotEmpty'
                )
            ),
            'value' => $url
        ));

        //brand
        $brandsArray = array(
            '' => 'none',
            'gn' => 'Geonetwork'
        );
        $form->addElement('radio', 'cswBrand', array(
            'required' => false , 
            'label' => 'brand',
            'multiOptions' => $brandsArray,
            'value' => $brand
        ));
        
        return true;
    }
    
    /**
     * Parse settings from received parameters.
     * This is the place where we can save received parameters to config file
     *  
     * @return boolean 
     */
    public function parseSettingsParams($newConfig, $form) 
    {
        $pluginName = $this->name;
    
        $newConfig->production->plugins->$pluginName->url = $form->getValue('cswUrl');
        $newConfig->production->plugins->$pluginName->brand = $form->getValue('cswBrand');
        
        return true;
    }
    
    /**
     * Get parameters
     * 
     * @return array
     */
    public function getParams() 
    {
        $config = Zend_Registry::get('config');
        $pluginName = $this->name;
        
        return array(
            'cswUrl' => $config->plugins->$pluginName->url,
            'cswBrand' => $config->plugins->$pluginName->brand
        );    
    }

}