<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Comm.php 2896 2012-08-01 15:28:28Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for comments table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Comm extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'comm';

    /**
     * Get items
     * 
     * @param integer $part The id of the article related to this comment
     * @param array $options Available options:
     * <ul><li>limitStart integer - Offset of the first row to return</li>
     * <li>limit integer - Maximum number of rows to return</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems ($part, array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'date' => "to_char({$this->_name}.date, '"  . GT_DATETIME_STYLE_DB . "')" ,
            'text'
        );
        $select->from($this, $fields);
        
        $select->joinLeft(array(
            'users' => $this->_schema . '.users'
        ), "users.id = {$this->_name}.user_id", array(
            'name' => "(users.lname||' '||users.fname)" , 
            //'account' => 'users.account'
        ));
        
        $select->where('part = ?', $part);
        $select->order($this->_name . '.date desc');

        //Limit
        if (! empty($options['limit'])) {
            if (empty($options['limitStart'])) {
                $options['limitStart'] = 0;
            }
            $select->limit($options['limit'], $options['limitStart']);
        }
        
        $result = $this->fetchAll($select);
        
        //Escape
        foreach ($result as $row) {
            $row->text = htmlspecialchars($row->text, ENT_QUOTES);
        }
        
        return $result;
    }

    /**
     * Add item
     * 
     * @param integer $part The id of the article related to this comment
     * @param Zend_Form $data Comment data
     * @return integer The id of the new element
     */
    public function addItem ($part, $data)
    {
        if($data instanceof Zend_Form) {
            $data = $data->getValues();
        }
        
        //Get current user id
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $newRow = $this->createRow();
        
        //$newRow->name = $data->getValue('name');
        //$newRow->email = $data->getValue('email');
        $newRow->user_id = $userId;
        
        $newRow->text = $data['text'];

        $newRow->part = $part;
        $newRow->date = new Zend_Db_Expr('now()');
        
        return $newRow->save();
    }

    /**
     * Count the number of items in the table
     * 
     * @param array $options Available options:
     * <ul><li>part integer - The id of the article related to this comment</li></ul>
     * @return integer Number of items
     */
    public function count ($options = array())
    {
        $select = $this->select();
        $select->from($this, array(
            'count' => 'count(*)'
        ));
        
        if (! empty($options['part'])) {
            $select->where('part = ?', $options['part']);
        }
        
        $count = $this->fetchRow($select);
        
        return $count->count;
    }

}