<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: JobsActionsWorkflows.php 2894 2012-08-01 14:23:23Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for Jobs Actions Workflows table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_JobsActionsWorkflows extends Giscuit_Db_Table_Multilang
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'jobs_actions_workflows';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Contains the names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array(
        'name'
    );
    
    /**
     * Get items
     * 
     * @param array $options Available options:
     * <ul><li>withAllNames bool - Get name in all installed languages</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' ,
            'name_id'
        );
        
        $select->from($this, $fields);
        
        if (! empty($options['withMultilangFields'])) {
            $this->withMultilangFields($select, $options);
        } else {
            $this->joinMultilangField($select, 'name');
        }
        
        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Add item
     * 
     * @param array $options Available options:
     * <ul></ul>
     * @return $data array - id and name_id of the new element
     */
    public function addItem (array $options = array())
    {
        $newRow = $this->createRow();

        $textTable = new Giscuit_Db_Table_Text();
        $newTextRow = $textTable->createRow();
        
        $newRow->name_id = $newTextRow->save();
        $data['name_id'] = $newRow->name_id;
        
        $data['id'] = $newRow->save();
        
        //Create default reserved actions
        $actionsMapTable = new Giscuit_Db_Table_JobsActionsMap();
        
        $defaultActions = array (
            'accept' => array(
                'id' => '',
                'order' => 1,
                'theme' => 'e',
                'reserved' => 1,
                'text' => 'Accept'
            ),
            'decline' => array(
                'id' => '',
                'order' => 2,
                'theme' => 'e',
                'reserved' => 2,
                'text' => 'Decline'
            ),
            'checkin' => array(
                'id' => '',
                'order' => 3,
                'theme' => 'c',
                'reserved' => NULL,
                'text' => 'Check in'
            ),
            'checkout' => array(
                'id' => '',
                'order' => 4,
                'theme' => 'c',
                'reserved' => NULL,
                'text' => 'Check out'
            ),
            'complete' => array(
                'id' => '',
                'order' => 5,
                'theme' => 'b',
                'reserved' => 3,
                'text' => 'Complete'
            )
        );
        
        $languages = Zend_Registry::get('config')->language->toArray();
        foreach ($defaultActions as $key => $action) {
            $newAction = $actionsMapTable->createRow();
            $newAction->order = $action['order'];
            $newAction->theme = $action['theme'];
            $newAction->workflow_id = $data['id'];
            $newAction->reserved = $action['reserved'];
            $newAction->parent = '';
            
            if ($key == 'checkin') {
                $newAction->parent = $defaultActions['accept']['id'];
            } else if ($key == 'checkout') {
                $newAction->parent = $defaultActions['checkin']['id'];
            } else if ($key == 'complete') {
                $newAction->parent = $defaultActions['checkout']['id'];
            }
            
            $newTextRow = $textTable->createRow();
            foreach ($languages as $langKey => $langValue) {
                $newTextRow->$langKey = $action['text'];
            }
            $newAction->name_id = $newTextRow->save();
            
            $defaultActions[$key]['id'] = $newAction->save();
        }

        return $data;
    }
}
