<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LayerPublish.php 2894 2012-08-01 14:23:23Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for layers publish table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_LayerPublish extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'layer_publish';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Update tilecache config
     * 
     * @return void
     */
    public static function updateTilecacheConfig ()
    {
        
        //Get base
        $baseTable = new Giscuit_Db_Table_Base();
        $rows = $baseTable->getItems();
        
        $baseLayers = array();
        foreach ($rows as $row) {
            $baseLayers[$row->id] = array(
                'id' => $row->id ,
                'parent_id' => $row->parent_id , 
                'format' => $row->format
            );
        }
        
        //Get layers
        $layersTable = new Giscuit_Db_Table_Layers();
        $rows = $layersTable->getItems(array(
            'wherePublish' => 't' , 
            'orderDirection' => 'asc' , 
            'withPublish' => true
        ));
        
        $tilecachePath = GT_PUBLIC_PATH . DIR_SEP . 'tiles';
        $result = "[cache]\ntype=TmsDisk\nbase=$tilecachePath\n\n";
        
        //Overlays
        foreach ($rows as $row) {
            if (! empty($row['class']) && 
                ! is_subclass_of('Giscuit_Map_Layer_' . $row['class'], 'Giscuit_Map_Layer_JavaScript') && 
                is_array($row['destination'])) {
                
                foreach ($row['destination'] as $destination) {
                    if ($destination == 'overlay') {
                        $result .= "[{$row['class']}]\n";
                        
                        $result .= self::getTilecacheParams(array(
                            'class' => $row['class'],
                            'format' => $row['format']
                        ));

                        $result .= "\n";
                    } else {
                        $baseLayers[$destination]['classes'][] = $row['class'];
                    }
                }
            
            }
        }
        
        //Base
        foreach ($baseLayers as $layer) {
            
            if (! empty($layer['id']) && ! empty($layer['classes'])) {
                $result .= "[_base{$layer['id']}]\n";
                $result .= "layers=" . implode(',', array_reverse($layer['classes'])) . "\n";
                $result .= self::getTilecacheParams(array(
                    'class' => $row['class'],
                    'format' => $layer['format']
                ));
                $result .= "\n";
            }
        }
        
        $tilecacheCfg = file_get_contents(GT_PUBLIC_PATH . DIR_SEP . 'tilecache' . DIR_SEP . 'tilecache.cfg');
        if(! stristr($tilecacheCfg, "; Temporary\n")) {
            $tilecacheCfg .= "\n; Temporary\n";
        }

        $tilecacheCfg = explode("\n; Temporary\n", $tilecacheCfg);
        $tilecacheCfg[0] = $result;
        $tilecacheCfg = implode("\n; Temporary\n", $tilecacheCfg);
        
        file_put_contents(GT_PUBLIC_PATH . DIR_SEP . 'tilecache' . DIR_SEP . 'tilecache.cfg', $tilecacheCfg);
    }

    /**
     * Get tilecache params
     * 
     * @param array $options Available options:
     * <ul><li>class string - </li>
     * <li>format string - </li></ul>
     * @return string
     */
    public static function getTilecacheParams (array $options = array())
    {
        $config = Zend_Registry::get('config');
        $result = array();
        
        $resolutions = $config->map->resolutions;
        
        $bbox = array(
            $config->map->extent->min->x , 
            $config->map->extent->min->y , 
            $config->map->extent->max->x , 
            $config->map->extent->max->y
        );
        $bbox = implode(',', $bbox);
        
        $urlHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
        
        if (empty($options['action'])) {
            $action = 'tilecache';
        } else {
            $action = $options['action'];
        }
        
        $url = 'http://' . $_SERVER['HTTP_HOST'] . $urlHelper->url(array(
            'language' => $config->defaultLanguage,
            'module' => 'default' ,
            'controller' => 'wms' , 
            'action' => $action
        ), null, true) . '?';
        
        if($options['format'] == 'png' || $options['format'] == 'png24') {
            $url .= 'TRANSPARENT=TRUE&';    
        }
        
        if($options['format'] != 'png') {
            $result['metaTile'] = 'true';
            $result['metaSize'] = '5,5';
            $result['metaBuffer'] = '10';
        }
        
        $url .= 'server_origin=' . $config->server_origin . '&';
        
        if ($options['format'] == 'jpeg') {
            $extension = 'jpeg';
            $mime_type="image/jpeg";
        } elseif($options['format'] == 'png') {
            $extension = 'png';
            $mime_type="image/png";
        } elseif($options['format'] == 'png24') {
            $extension = 'png';
            $mime_type="image/png; mode=24bit";
        }
        
        $result['type'] = 'WMS';
        $result['extent_type'] = 'loose';
        $result['extension'] = $extension;
        $result['mime_type'] = $mime_type;
        $result['url'] = $url;
        $result['resolutions'] = $resolutions;
        $result['bbox'] = $bbox;
        $result['srs'] = "EPSG:{$config->map->srid}";
        $result['debug'] = 'off';
        
        //Parse
        $className = 'Giscuit_Map_Layer_' . $options['class'];
        if(@class_exists($className)) {
            $result = call_user_func(array($className, 'parseTilecacheParams'), $result, $options);
        }
        
        //Implode
        $str = '';
        foreach ($result as $key => $value) {
            $str .= "{$key}={$value}\n";    
        }
                
        return $str;
    }

}
