<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Articles.php 2182 2012-01-20 12:01:13Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Multilang
 */
require_once 'Giscuit/Db/Table/Multilang.php';

/**
 * Abstract class for routing cost columns table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_RoutingCostColumns extends Giscuit_Db_Table_Multilang
{
    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'routing_cost_columns';
    
    /**
     * Names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array('name');

    /**
     * Get items
     * 
     * @param array $options Available options:
     * <ul><li>withCurrentLanguage bool - Get elements in current language only</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        //Set fields
        $fields = array(
            'id' ,
            'column' ,
            'reverse_column' ,
            'layer_id'
        );
        
        $select->from($this, $fields);
        
        $this->whereId($select, $options);
        $this->whereLayerId($select, $options);
        
        if (! empty($options['withCurrentLanguage'])) {
            $this->joinMultilangField($select, 'name');
        } else {
            $this->addMultilangFields($select);
        }
        
        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Get column
        $column = $data->column;

        $row->$column = $data->data;
        
        return $row->save();
    }

    /**
     * Add item
     * 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return Zend_Db_Table_Row
     */
    public function addItem ($data)
    {
        $newRow = $this->createRow();
        $newRow->layer_id = $data->layer_id;
        
        $this->manageMultilangText(array('name' => array()), $newRow);
        $newRow->save();
        
        return $newRow;
    }
    
    /**
     * Fetch rows that match the layer id
     * 
     * @param Zend_Db_Select $select
     * @param array $options Available options:
     * <ul><li>whereLayerId integer - If option is set selects row with given layer id</li></ul>
     * @return void
     */
    public function whereLayerId ($select, array $options = array())
    {
        if (! isset($options['whereLayerId'])) {
            return;
        }
        
        $select->where($this->_name . '.layer_id = ?', $options['whereLayerId']);
    }
    
    /**
     * Check if cost column belongs to current user
     * 
     * @param integer $id Cost column id
     * @return void
     */
    public function checkCurrUserOwnCostColumn($id)
    {
        $rows = $this->getItems(array(
            'whereId' => $id
        ))->toArray();
        if(count($rows) == 0) {
            throw new Exception("Invalid cost column id ({$id})");
        }
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $layersTable->checkCurrUserOwnLayer($rows[0]['layer_id']);
    }
}