<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Rules.php 2896 2012-08-01 15:28:28Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for rules table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Rules extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'rules';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Get rules
     * 
     * @param array $options Available options:
     * <ul><li>getResourceList string - Get only resource list that match the given group</li>
     * <li>getPrivilegeList string - Get only priviledge list match the given resource</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'role' , 
            'resource' , 
            'privilege' , 
            'group'
        );
        
        if(isset($options['getResourceList'])) {
            $fields = array('resource');
            
            $select->where($this->_name . '.group = ?', $options['getResourceList']);
            $select->group($this->_name . '.resource');
        } 
        
        if(isset($options['getPrivilegeList'])) {
            $fields = array('privilege');
            $select->where($this->_name . '.resource = ?', $options['getPrivilegeList']);
        }
            
        if(!isset($options['getResourceList']) && !isset($options['getPrivilegeList'])) {                    
            $select->order($this->_name . '.id desc');    
        }
                        
        $select->from($this, $fields);
        
        $result = $this->fetchAll($select);
        
        return $result;
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Get column
        $column = $data->column;
        
        $row->$column = $data->data;
        
        Giscuit_Acl::clearCache();

        return $row->save();
    
    }
    
    /**
     * Add item
     * 
     * @return integer Id of the new item
     */
    public function addItem ()
    {
        $newRow = $this->createRow();
        
        Giscuit_Acl::clearCache();
        
        return $newRow->save();
    }

}
