<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Exec.php 3301 2013-02-13 10:46:20Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Exec
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Program execution Functions
 * 
 * @category   Giscuit
 * @package    Giscuit_Exec
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Exec
{

    /**
     * Execute an external program
     * 
     * @param string $binary Command path
     * @param string $arguments Command arguments
     * @param string $workdir The initial working directory for the command
     * @param array $descriptorspec An indexed array where the key represents the descriptor number and the value represents how PHP will pass that descriptor to the child process. 0 is stdin, 1 is stdout, while 2 is stderr.
     * @param string Command extension
     * @throws Exception
     * 
     * @return string Output from the result of the command
     */
    public static function exec ($binary, $arguments, $workdir = NULL, $descriptorspec = NULL, $extension = '')
    {
        
        if (empty($descriptorspec)) {
            $descriptorspec = array(
                0 => array(
                    "pipe" , 
                    "r"
                ) , 
                1 => array(
                    "pipe" , 
                    "w"
                ) , 
                2 => array(
                    "pipe" , 
                    "w"
                )
            );
        }
        
        $env = array();
        
        if (! empty($workdir)) {
            $workdir = realpath($workdir) . DIR_SEP;
        }
        
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            if (empty($extension)) {
                $extension = '.exe';
            }
            
            $env = array(
                'GT_PGSQL_BIN_PATH' => GT_PGSQL_BIN_PATH ,
                'GT_GDAL_BIN_PATH' => GT_GDAL_BIN_PATH ,
                'GT_GDAL_LIB_PATH' => GT_GDAL_LIB_PATH ,
                'GT_GDAL_DATA_PATH' => GT_GDAL_DATA_PATH ,
                'GT_PHP_PATH' => GT_PHP_PATH ,
                'PATH' => getenv('PATH') . ';' . GT_GDAL_LIB_PATH . ';' . GT_GDAL_DATA_PATH
            );
            
            $command = '"' . $workdir . $binary . $extension . '" ' . $arguments;
        } else {
            $command = $workdir . $binary . $extension . ' ' . $arguments;
        }
        
        $pipes = NULL;
        
        $handle = proc_open($command, $descriptorspec, $pipes, $workdir, array_merge($_ENV, $env), array(
            "bypass_shell" => true
        ));
        
        if (! is_resource($handle)) {
            throw new Exception("Process handle creation failed");
        }
        
        fclose($pipes[0]);
        
        $result = array();
        
        if (isset($pipes[1])) {
            $result['output'] = stream_get_contents($pipes[1]);
            fclose($pipes[1]);
        }
        
        if (isset($pipes[2])) {
            $result['error'] = stream_get_contents($pipes[2]);
            fclose($pipes[2]);
        }
        
        proc_close($handle);
        
        //Filter errors
        $result['error'] = str_replace('could not find a "psql" to execute' . PHP_EOL, '', $result['error']);
        
        return $result;
    }
    
    /**
     * Execute an external program in background
     * 
     * @param string $cmd The command that will be executed
     * @param string $stderr STDERR path
     * 
     * @return void
     */
    public static function background($cmd, $stderr = '') {
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            if($stderr) {
                $stderr = "2> \"{$stderr}\"";
            }
            pclose(popen("start /B {$cmd} {$stderr}", 'r')); 
            
        } else {
            if($stderr) {
                $stderr = "2> \"{$stderr}\"";
            }
            pclose(popen("{$cmd} {$stderr} &", 'r'));
            
        }
    }
    
}