<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Install.php 3521 2013-12-23 15:06:20Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Form_Abstract
 */
require_once 'Giscuit/Form/Abstract.php';

/**
 * Installation process form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Install extends Giscuit_Form_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ($options = array())
    {
        $this->addDatabase();
        $this->addGeneral();
        $this->addMap();
        $this->addAdmin();
        $this->addSignup();
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            $this->addExternalPaths();    
        }
        if (Zend_Registry::isRegistered('config')) {
            $this->addProxy();
            $this->addSmtp();
            $this->addLanguageUpload();
            $this->addTitle();
            
            //Current tab
            $this->addElement('hidden', 'current_tab', array(
                'required' => false,
                'ignore' => true ,
                'decorators' => array(
                    'ViewHelper'
                )
            ));
        }
        
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'install'
        ));
        
        $this->setAttrib('enctype', 'multipart/form-data');
    }
    
    /**
     * Add database fields
     *
     * @return void
     */
    public function addDatabase ()
    {
        //Error
        $this->addElement('hidden', 'db_error', array(
            'required' => false , 
            'decorators' => array(
                'ViewHelper' , 
                'Errors'
            )
        ));
        
        //Database name
        $this->addElement('text', 'db_name', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'required' => true , 
            'label' => 'dbName'
        ));
        
        //Username
        $this->addElement('text', 'db_username', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'required' => true , 
            'label' => 'username'
        ));
        
        //Password
        $this->addElement('password', 'db_password', array(
            'required' => false , 
            'label' => 'password'
        ));
        
        //Host
        $this->addElement('text', 'db_host', array(
            'filters' => array(
                'StringTrim'
            ) , 
            //'required' => true , 
            'label' => 'host' 
        ));
        
        //Port
        $this->addElement('text', 'db_port', array(
            'filters' => array(
                'StringTrim'
            ) , 
            //'required' => true ,
            'value' => '5432' ,
            'label' => 'port' 
        ));
        
        //Schema
        $this->addElement('text', 'db_schema', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'required' => true , 
            'label' => 'schema'
        ));
        
        $this->addDisplayGroup(array(
            'db_error' , 
            'db_name' , 
            'db_username' , 
            'db_password' , 
            'db_host' ,
            'db_port' ,  
            'db_schema'
        ), 'database');
        
        $dbconn = $this->getDisplayGroup('database');
        $dbconn->setLegend('dbConnection');
    
    }

    /**
     * Add general fields
     *
     * @return void
     */
    public function addGeneral ()
    {
        
        //Date format
        $this->addElement('select', 'dateFormat', array(
            'multiOptions' => array(
                'iso8601' => 'YYYY-MM-DD',
                'german' => 'DD.MM.YYYY',
                'sqlMdy' => 'MM/DD/YYYY',
            ) , 
            'required' => true , 
            'label' => 'dateFormat'
        ));
        
        //Time zone
        $timeZoneArr = array_combine(DateTimeZone::listIdentifiers(), DateTimeZone::listIdentifiers());
        $this->addElement('select', 'timeZone', array(
            'required' => true , 
            'label' => 'timeZone' , 
            'multiOptions' => $timeZoneArr , 
            'value' => @date_default_timezone_get()
        ));
        
        //Http path
        $this->addElement('text', 'httpPath', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'required' => false , 
            'label' => 'httpPath'
        ));
        
        //Ssl
        $this->addElement('checkbox', 'ssl', array(
            'validators' =>  $this->checkboxValidator ,
            'required' => true , 
            'label' => 'ssl' , 
            'checkedValue' => 't' , 
            'uncheckedValue' => 'f'
        ));
        
        $this->addDisplayGroup(array(
            'dateFormat' , 
            'timeZone' , 
            'httpPath' , 
            'ssl'
        ), 'general');
        
        $general = $this->getDisplayGroup('general');
        $general->setLegend('general');
    }

    /**
     * Add map fields
     *
     * @return void
     */
    public function addMap ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        //Srid
        $this->addElement('text', 'srid', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => true , 
            'label' => 'srid'
        ));
        
        //Left
        $this->addElement('text', 'minx', array(
            'filters' => array(
                'Float'
            ) , 
            'required' => true , 
            'label' => 'minx'
        ));
        
        //Bottom
        $this->addElement('text', 'miny', array(
            'filters' => array(
                'Float'
            ) , 
            'required' => true , 
            'label' => 'miny' 
        ));
        
        //Right
        $this->addElement('text', 'maxx', array(
            'filters' => array(
                'Float'
            ) , 
            'required' => true , 
            'label' => 'maxx' 
        ));
        
        //Top
        $this->addElement('text', 'maxy', array(
            'filters' => array(
                'Float'
            ) , 
            'required' => true , 
            'label' => 'maxy' 
        ));
        
        //Resolutions
        $this->addElement('text', 'resolutions', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'required' => true , 
            'label' => 'resolutions'
        ));

        //Zoom x
        $this->addElement('text', 'zoomx', array(
            'filters' => array(
                'Float'
            ) ,
            'label' => 'zoomx'
        ));

        //Zoom y
        $this->addElement('text', 'zoomy', array(
            'filters' => array(
                'Float'
            ) ,
            'label' => 'zoomy'
       ));

        //Zoom z
        $this->addElement('text', 'zoomz', array(
            'filters' => array(
                'Float'
            ) ,
            'label' => 'zoomz',
        ));

        //Units
        $this->addElement('select', 'units', array(
            'multiOptions' => array(
                'm' => $translate->_('meters') , 
                'dd' => $translate->_('degrees') , 
                'inches' => $translate->_('inches') , 
                'ft' => $translate->_('feet') , 
                'mi' => $translate->_('miles') , 
                'km' => $translate->_('kilometers')
            ) , 
            'required' => true , 
            'label' => 'units' , 
            'value' => 'm'
        ));
        
        //Display units
        $this->addElement('select', 'displayUnits', array(
            'multiOptions' => array(
                'metric' => $translate->_('metric') , 
                'geographic' => $translate->_('geographic') , 
                'english' => $translate->_('english')
            ) , 
            'required' => true , 
            'label' => 'displayUnits' , 
            'value' => 'metric'
        ));
        
        //Watermarks
        $this->addElement('checkbox', 'watermarks', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'watermark' , 
            'checkedValue' => 't' , 
            'uncheckedValue' => 'f' 
        ));

        //Map
        $this->addDisplayGroup(array(
            'srid' ,
            'minx' ,
            'miny' ,
            'maxx' ,
            'maxy' ,
            'resolutions' ,
            'units' ,
            'displayUnits' ,
            'watermarks'
        ), 'map');
        
        $map = $this->getDisplayGroup('map');
        $map->setLegend('map');

        //Zoom
        $this->addDisplayGroup(array(
            'zoomx', 'zoomy', 'zoomz'
        ), 'zoom');

        $zoom = $this->getDisplayGroup('zoom');
        $zoom->setLegend('initialZoom');

    }
    
    /**
     * Add path fields
     *
     * @return void
     */
    public function addExternalPaths()
    {
        //8.4
        $pgsqlBinPath = 'C:\Program Files (x86)\PostgreSQL\8.4\bin';
        if(! file_exists($pgsqlBinPath)) {
            $pgsqlBinPath = 'C:\Program Files\PostgreSQL\8.4\bin';
        }
        
        //9.2
        if(! file_exists($pgsqlBinPath)) {
            $pgsqlBinPath = 'C:\Program Files (x86)\PostgreSQL\9.2\bin';
        }
        if(! file_exists($pgsqlBinPath)) {
            $pgsqlBinPath = 'C:\Program Files\PostgreSQL\9.2\bin';
        }
        
        //Pgsql bin path
        $this->addElement('text', 'pgsqlBinPath', array(
            'validators' => array(
                array(
                    'PsqlExists'
                )                
            ) ,
            'label' => 'pgsqlBinPath' ,
            'value' => $pgsqlBinPath
        ));
        
        $this->addDisplayGroup(array(
            'pgsqlBinPath' 
        ), 'externalPaths');
        
        $externalPaths = $this->getDisplayGroup('externalPaths');
        $externalPaths->setLegend('externalPaths');
    }

    /**
     * Add administrator account fields
     *
     * @return void
     */
    public function addAdmin()
    {
        $this->addElement('text', 'admin_account', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'validators' => array(
                array(
                    'EmailAddress'
                )                
            )
             , 
            'required' => true , 
            'label' => 'adminAccount'
        ));
        
        
        $this->addElement('password', 'admin_password', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        6 , 
                        20
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'password'
        ));
        
        $this->addElement('password', 'admin_confirm_password', array(
            'validators' => array(
                array(
                    'ConfirmPassword' , 
                    false , 
                    array(
                        'admin_password'
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'confirmPassword'
        ));
        
        $this->addElement('text', 'admin_fname', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars',
                array(
                    'Callback',
                    'ucwords'
                )
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        3 , 
                        20
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'fname'
        ));

        $this->addElement('text', 'admin_lname', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars',
                array(
                    'Callback',
                    'ucwords'
                )
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        3 , 
                        20
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'lname'
        ));
        
        $this->addDisplayGroup(array(
            'admin_account' , 
            'admin_password' , 
            'admin_confirm_password' , 
            'admin_fname' ,
            'admin_lname'
        ), 'admin');
        
        $general = $this->getDisplayGroup('admin');
        $general->setLegend('administratorAccount');
    }
    
    /**
     * Add signup options fields
     *
     * @return void
     */
    public function addSignup ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $this->addElement('select', 'signup_resource', array(
            'required' => true , 
            'label' => 'defaultRole'
        ));
        
        $this->addElement('select', 'signup_activate', array(
            'multiOptions' => array(
                1 => $translate->_('manual'),
                2 => $translate->_('email'),
                3 => $translate->_('automaticallyOnRegister')
            ) , 
            'required' => true , 
            'label' => 'signupActivate'
        ));
        
        $this->addDisplayGroup(array(
            'signup_resource' , 
            'signup_activate' 
        ), 'signup');
        
        $signup = $this->getDisplayGroup('signup');
        $signup->setLegend('signupOptions');
    }
    
    /**
     * Add proxy options fields
     *
     * @return void
     */
    public function addProxy ()
    {
        $this->addElement('text', 'proxy_host', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) ,
            'required' => false , 
            'label' => 'host'
        ));
        
        $this->addElement('text', 'proxy_port', array(
            'required' => false , 
            'label' => 'port'
        ));
        
        $this->addElement('text', 'proxy_username', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) , 
            'required' => false , 
            'label' => 'username'
        ));
        
        $this->addElement('password', 'proxy_password', array(
            'required' => false ,
            'renderPassword' => true, 
            'label' => 'password'
        ));
        
        $this->addDisplayGroup(array(
            'proxy_host' , 
            'proxy_port' ,
            'proxy_username' ,
            'proxy_password'
        ), 'proxy');
        
        $proxy = $this->getDisplayGroup('proxy');
        $proxy->setLegend('proxy');
    }
    
    /**
     * Add smtp options fields
     *
     * @return void
     */
    public function addSmtp ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $this->addElement('text', 'smtp_host', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) ,
            'required' => false , 
            'label' => 'host'
        ));
        
        $this->addElement('text', 'smtp_port', array(
            'required' => false , 
            'label' => 'port'
        ));
        
        $this->addElement('select', 'smtp_ssl', array(
            'multiOptions' => array(
                '' => '',
                'tls' => $translate->_('tsl'),
                'ssl' => $translate->_('ssl'),
            ) , 
            'required' => false , 
            'label' => 'ssl'
        ));
        
        $this->addElement('select', 'smtp_auth', array(
            'multiOptions' => array(
                '' => '',
                'login' => $translate->_('login'),
                'plain' => $translate->_('plain'),
                'crammd5' => $translate->_('crammd5'),
            ) , 
            'required' => false , 
            'label' => 'authType'
        ));
        
        $this->addElement('text', 'smtp_username', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) , 
            'required' => false ,
            'label' => 'username'            
        ));
        
        $this->addElement('password', 'smtp_password', array(
            'required' => false , 
            'renderPassword' => true, 
            'label' => 'password'
        ));
        
        $this->addElement('text', 'smtp_from_email', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) , 
            'validators' => array(
                'EmailAddress'
            ),
            'required' => false , 
            'label' => 'fromEmail'
        ));
        
        $this->addElement('text', 'smtp_from_name', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars'
            ) , 
            'required' => false , 
            'label' => 'fromName'
        ));
        
        $this->addDisplayGroup(array(
            'smtp_host' , 
            'smtp_port' ,
            'smtp_ssl' ,
            'smtp_auth' ,
            'smtp_username' ,
            'smtp_password',
            'smtp_from_email',
            'smtp_from_name',
        ), 'smtp');
        
        $smtp = $this->getDisplayGroup('smtp');
        $smtp->setLegend('smtp');
    }
    
    public function addLanguageUpload() 
    {
        //Archive
        $langpack = new Zend_Form_Element_File('langpack');
        $langpack->setLabel('addLanguageFile');
        $langpack->setRequired(false);
        $langpack->addValidator('Count', false, 1);
        $langpack->setDestination(GT_TMP_PATH);
        $langpack->addValidator('FilesSize', false, array(
            'min' => 1 , 
            'max' => '3MB'
        ));
        $langpack->addValidator('Extension', false, array(
            'ext' => 'zip' , 
            'case' => true
        ));
        $this->addElement($langpack, 'langpack');
        
    }
    
    public function addTitle() 
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        $displayGroup = array();
        
        //Title
        foreach($config->language as $key => $value) {
            $elName = 'title_name_' . $key;       
            $displayGroup[] = $elName; 
            
            $this->addElement('text', $elName, array(
                'filters' => array(
                    'StringTrim'
                ) ,
                'required' => false ,
                'label' => $translate->_('name') . ' (' . $value . ')'
            ));
        }
        
        //Logo
        $logo = new Zend_Form_Element_File('title_logo');
        $logo->setLabel('addLogo');
        $logo->setRequired(false);
        $logo->setDestination(GT_PUBLIC_PATH . DIR_SEP . 'static' . DIR_SEP . 'upload' . DIR_SEP);
        $logo->addFilter('Rename', 'logo.png');
        $logo->addValidator('FilesSize', false, array(
            'min' => 1 , 
            'max' => '2MB'
        ));
        $logo->addValidator('Extension', false, array(
            'ext' => 'jpg,png,gif'
        ));
        $logo->addValidator('Count', false, 1);
        $this->addElement($logo, 'title_logo');
        $displayGroup[] = 'title_logo';

        //
        $this->addDisplayGroup($displayGroup, 'title');
        $title = $this->getDisplayGroup('title');
        $title->setLegend('header');
    }
}