<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Abstract.php 3514 2013-12-03 15:29:38Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Abstract layer form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Layer_Abstract extends Giscuit_Form_Abstract
{
    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */
    public function populate(array $values)
    {
        //Subforms
        $values = $this->subformPopulate($values);

        return $this->setDefaults($values);
    }

    /**
     * Populate subform
     *
     * @param  array $values
     * @return array $values
     */
    public function subformPopulate(array $values)
    {
        //
        $db = Zend_Registry::get('db');
        $config = Zend_Registry::get('config');

        //
        $frontController = Zend_Controller_Front::getInstance();
        $params = array_merge($frontController->getRequest()->getParams(), $values);

        //
        $groups = $this->getDisplayGroups();

        //
        foreach($groups as $group) {
            $attrs = $group->getAttribs();

            if(isset($attrs['subform']) && $attrs['subform']) {
                $els = $group->getElements();

                //Add params from DB
                if(count($values) && $params['action'] == 'get') {
                    $gid = $params['gid'];

                    //
                    $dbParams = $attrs['database'];
                    $table = $dbParams['table'];
                    $schema = isset($dbParams['schema']) ? $options['database']['schema'] : $config->database->schema . '_layers';

                    $select = $db->select();
                    $select->from($schema . '.' . $table);
                    $select->where("{$table}.parent_id = ?", $gid);
                    $rows = $db->fetchAll($select, NULL, Zend_Db::FETCH_ASSOC);

                    $idx = 0;
                    foreach($rows as $row) {
                        foreach($row as $fieldKey => $fieldVal) {
                            if($fieldKey == 'id' || $fieldKey == 'parent_id') {
                                continue;
                            }

                            $params[$fieldKey . '_' . $idx] = $fieldVal;
                        }

                        $idx++;
                    }
                }

                //
                foreach($els as $elName => $el) {
                    $ddClass = $el->getDecorator('HtmlTag')->getOption('class');
                    $el->getDecorator('HtmlTag')->setOption('class', $ddClass . ' gtSubformEl gtSubformDefault');

                    $el->setRequired(false);
                    $el->setValidators(array());
                }
                $lastEl = $el;
                $lastElName = $lastEl->getName();

                //
                $this->_addSubformSeparator($el, 'gtSubformDefault ');

                //Get indexes
                $idxArr = array();
                foreach($params as $paramKey => $paramVal) {
                    if (preg_match("/^{$lastElName}_(?P<digit>\d+)$/", $paramKey, $match)) {
                        $idxArr[] = $match[1];
                    }
                }

                if(! count($values)) {
                    $group->setAttrib('data-count', 0);
                } else {
                    $group->setAttrib('data-count', count($idxArr));

                    foreach($idxArr as $idx) {
                        foreach($els as $elName => $el) {
                            //Deep clone
                            $newEl = unserialize(serialize($el));

                            $newEl->setName($elName . '_' . $idx);

                            $ddClass = $el->getDecorator('HtmlTag')->getOption('class');
                            $ddClass = str_replace('gtSubformDefault', '', $ddClass);
                            $newEl->getDecorator('HtmlTag')->setOption('class', $ddClass);

                            //Set value and validate
                            if(count($params)) {
                                $val = $params[$elName . '_' . $idx];

                                //Gtmultiselect & Gtmulticheckbox
                                if($newEl instanceof Giscuit_Form_Element_Gtmultiselect ||
                                    $newEl instanceof Giscuit_Form_Element_Gtmulticheckbox) {

                                    $val = is_string($val) ? explode(', ', $val) : array();
                                }

                                //
                                $newEl->setValue($val);
                                $newEl->isValid($val);
                            }

                            $group->addElement($newEl);
                        }
                        $this->_addSubformSeparator($newEl);
                    }
                }
            }
        }

        return $values;
    }

    /**
     * Add sunform separator
     *
     * @param Zend_Form_Element $el Element
     * @param string $class Class name
     *
     * @return void
     */
    protected function _addSubformSeparator($el, $class = '')
    {
        $staticPathHelper = new Giscuit_View_Helper_StaticPath();
        $translate = Zend_Registry::get('Zend_Translate');

        $el->addDecorator(
            array('subformDelBtn' => 'HtmlTag'),
            array(
                 'tag' => 'img',
                 'class' => 'gtSubformDel ' . $class,
                 'src' => $staticPathHelper->staticPath('/list_remove.png', 'img'),
                 'placement' => 'append',
                 'title' => $translate->_('remove')
            )
        );

        $el->addDecorator(
            array('subformHr' => 'HtmlTag'),
            array(
                 'tag' => 'hr',
                 'class' => 'gtSubformHr ' . $class,
                 'placement' => 'append',
            )
        );
    }

}