<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Search.php 3281 2013-01-22 14:10:46Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Form_Abstract
 */
require_once 'Giscuit/Form/Abstract.php';

/**
 * Search form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Search extends Giscuit_Form_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        //
        $this->setMethod('post');
        $this->addAttribs(array(
            'id' => 'gtSearchForm'
        ));
        
        //Layer
        $this->addElement('select', 'layer', array(
            'required' => true , 
            'label' => 'layerOnlyVisibleLayersAreShown' , 
            'attribs' => array(
                'id' => 'gtSearchLayer'
            ) ,
        ));
       
        //Field
        $this->addElement('select', 'field', array(
            'required' => true , 
            'label' => 'field' , 
            'attribs' => array(
                'id' => 'gtSearchField'
            ) , 
        ));
        
        //Where
        $whereArr = array(
            'everywhere' => $translate->_('everywhere') , 
            'bbox' => $translate->_('withInCurrentBbox')
        );
        $this->addElement('radio', 'where', array(
            'required' => false , 
            'multiOptions' => $whereArr , 
            'label' => 'where' , 
            'value' => 'everywhere' ,
            'attribs' => array(
                'id' => 'gtSearchWhere'
            ) ,
            'registerInArrayValidator' => false ,
        ));

        //Text
        $this->addElement('text', 'text', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'label' => 'text' , 
            'attribs' => array(
                'id' => 'gtSearchText' ,
                'onkeypress' => 'return event.keyCode != 13' ,
            )
        ));
        
        //Submit
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'search' , 
            'attribs' => array(
                'id' => 'gtSearchSubmit'
            ) ,
            'decorators' => array(
                'ViewHelper'
            )
        ));
        
        //Back
        $this->addElement('button', 'back_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'back' , 
            'attribs' => array(
                'id' => 'gtSearchBack'
            ) , 
            'decorators' => array(
                'ViewHelper'
            )
        ));
    
    }

    /**
     * Get layers
     * 
     * @return array
     */
    public function getLayers ()
    {
        $result = array();
        
        $layersTable = new Giscuit_Db_Table_Layers();
        
        $rows = $layersTable->getItems($this->getMapOptions());
        
        foreach ($rows as $row) {
            
            $path = $row['path'];
            
            if (! empty($path) && $path != '/') {
                $path = trim($path, '/');
                $path = explode('/', $path);
                if (count($path) > 1) {
                    foreach ($path as $key => $value) {
                        if(isset($rows[$path[$key]])) {
                            $path[$key] = $rows[$path[$key]]['name'];
                        }
                    }
                    $row['name'] = implode('/', $path);
                }
            }
            
            if(empty($row['class'])) {
                continue;
            }
            
            $layerName = 'Giscuit_Map_Layer_' . $row['class'];
            $tableName = 'Giscuit_Db_Table_Layer_' . $row['class'];
            
            if (class_exists($layerName) && class_exists('Giscuit_Map_Layer_Vector') && is_subclass_of($layerName, 'Giscuit_Map_Layer_Vector')) {
                $table = new $tableName();
                $table->setSettings($row);
                
                $defaultField = $table->getDefaultField();
                
                if (! empty($defaultField))
                    $result[$row['id']] = $row['name'];
            }
        }
        
        return $result;
    }

    /**
     * Get fields
     * 
     * @return array
     */
    public function getFields ($layerId)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $fields = array();
        $defaultField = NULL;
        
        //Get layer object
        $table = Giscuit_Db_Table_Layers::getLayerTableById($layerId, $this->getMapOptions());
        
        if($table) {
            $publicFields = $table->getPublicFields();
        
            foreach ($publicFields as $field) {
                $fields[$field['name']] = $translate->_($field['name']);
            }
            
            $defaultField = $table->getDefaultField();
        }
        
        return array(
            'fields' => $fields , 
            'defaultField' => $defaultField
        );
    
    }

    /**
     * Get map options
     * 
     * @return array
     */
    public function getMapOptions ()
    {
        $mapOptions = array(
            'orderDirection' => 'asc' , 
            'whereActive' => 't' , 
            'withLayerSettings' => true , 
            'withAcl' => true , 
            'withFields' => true ,
            'withGroups' => true
        );
        
        return $mapOptions;
    }
    
    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */
    public function populate (array $values)
    {
        //Layer
        $layers = $this->getLayers();
        $this->getElement('layer')->setMultiOptions($layers);
        
        //Field
        if(! isset($values['layer'])) {
            $layerId = key($layers);        
        } else {
            $layerId = $values['layer'];
        }
        $fields = $this->getFields($layerId);
        $this->getElement('field')->setMultiOptions($fields['fields'])
            ->setValue('defaultField');
        
        $result = $this->setDefaults($values);
        
        return $result;
    }

}