<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Settings.php 2844 2012-07-09 08:54:35Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Form_Abstract
 */
require_once 'Giscuit/Form/Abstract.php';

/**
 * User settings form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Settings extends Giscuit_Form_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        //Set method and id
        $this->setMethod('post');
        $this->addAttribs(array(
            'id' => 'gtSettingsForm'
        ));
        
        //Snap
        $this->addElement('checkbox', 'snap', array(
            'validators' => $this->checkboxValidator ,
            'required' => true ,
            'label' => 'snap' ,
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Tooltip
        $this->addElement('checkbox', 'tooltip', array(
            'validators' => $this->checkboxValidator , 
            'required' => true ,
            'label' => 'showHelpMessages' ,
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Language
        $languages = $config->language->toArray();
        $this->addElement('select', 'giscuit_language', array(
            /*
            'filters' => array(
                'StringTrim'
            ) ,
            */ 
            'required' => true , 
            'label' => 'language' , 
            'multiOptions' => $languages
        ));
        
        //Time zone
        $timeZoneArr = array_combine(DateTimeZone::listIdentifiers(), DateTimeZone::listIdentifiers());
        $this->addElement('select', 'timezone', array(
            'required' => true , 
            'label' => 'Timezone' , 
            'multiOptions' => $timeZoneArr 
        ));
        
        //Touch offset
        $filteredOffsets = array(
            1 => 'none',
            2 => 'upperLeft',
            3 => 'upperLeftFurther',
            4 => 'upperRight',
            5 => 'upperRightFurther'
        );
        $this->addElement('select', 'touch_offset', array(
            'required' => true , 
            'label' => 'touchOffset' , 
            'multiOptions' => $filteredOffsets
        ));
        
        //Submit
        $this->addElement('button', 'submit_button', array(
            'attribs' => array(
                'id' => 'gtSettingsSubmit'
            ) , 
            'required' => false , 
            'ignore' => true , 
            'label' => 'save'
        ));
    
    }
}