<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Mail.php 2142 2012-01-04 12:40:53Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Mail
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class for sending an e-mail
 * 
 * @category   Giscuit
 * @package    Giscuit_Mail
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Mail
{
    /**
     * Execute e-mail delivery batch file
     * 
     * @param string $file File path
     * @return void
     */
    public static function batchProcess($file) 
    {
        //
        $data = json_decode(file_get_contents($file));

        //Config
        $config = new Zend_Config_Xml(GT_CONFIG_PATH . DIR_SEP . 'config.xml', 'production');
                
        //SMTP settings
        $smtp = $config->smtp->toArray();
        if(! $smtp['host'] && ! $smtp['port']) {
            return;
        }
        
        //
        if(! $smtp['auth']) {
            unset($smtp['auth']);
            unset($smtp['username']);
            unset($smtp['password']);
        }

        //
        if(! $smtp['ssl']) {
            unset($smtp['ssl']);
        }
            
        //
        $tr = new Zend_Mail_Transport_Smtp($smtp['host'], $smtp);
        Zend_Mail::setDefaultTransport($tr);
        Zend_Mail::setDefaultFrom($config->smtp->from_email, $config->smtp->from_name);
        
        //
        try {
            foreach ($data as $recipient) {
                $mail = new Zend_Mail('UTF-8');
                $mail->addTo($recipient->to);
                $mail->setSubject($recipient->subject);
                $mail->setBodyHtml($recipient->text);
                $mail->send();
            }
        } catch(Exception $e) {
            error_log($e->getMessage());
        }
        
        unlink($file);
    }
    
    /**
     * Create e-mail delivery batch file
     * 
     * @param array $data E-mails to send
     * @return void
     */
    public static function batch(array $data = array())
    {
        $config = Zend_Registry::get('config');
        
        if(! $config->smtp->host && ! $config->smtp->port) {
            return;
        }
        
        $file = tempnam(GT_TMP_PATH, 'gt_');
        file_put_contents($file, json_encode($data));
        
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            $cmd = GT_PHP_PATH . DIR_SEP . 'php.exe "' . __FILE__ . '" "' . $file . '"';
        } else {
            $cmd = 'php "' . __FILE__ . '" "' . $file . '"';
        }
        
        Giscuit_Exec::background($cmd);
    }
}

if(! defined('GT_APP_PATH')) {
    //
    define('DIR_SEP', DIRECTORY_SEPARATOR);
    define('GT_APP_PATH', realpath(dirname(__FILE__) . DIR_SEP . '..' . DIR_SEP . '..'));
    define('GT_LIBRARY_PATH', GT_APP_PATH . DIR_SEP . 'library');
    define('GT_CONFIG_PATH', GT_APP_PATH . DIR_SEP . 'configs');
    define('GT_TMP_PATH', GT_APP_PATH . DIR_SEP . 'tmp');
    
    //Errors
    error_reporting(E_ALL | E_STRICT);
    ini_set('display_startup_errors', 0);
    ini_set('display_errors', 0);
    ini_set('log_errors', 1);
    ini_set('error_log', GT_APP_PATH . DIR_SEP . 'logs' . DIR_SEP . 'error.log');
    
    //Include_path
    $paths = array(
        realpath(GT_LIBRARY_PATH) , 
        get_include_path()
    );
    set_include_path(implode(PATH_SEPARATOR, $paths));

    //Loader
    require_once 'Zend/Loader/Autoloader.php';
    $loader = Zend_Loader_Autoloader::getInstance();
    $loader->registerNamespace('Giscuit_');
    
    Giscuit_Mail::batchProcess($argv[1]);
}