<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: StaticPath.php 3110 2012-11-08 08:02:59Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_View_Helper
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Generate a http path for a static file
 * 
 * @category   Giscuit
 * @package    Giscuit_View_Helper
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_View_Helper_StaticPath
{

    /**
     * Generate a http path for static a file
     * 
     * @param string $file Url relative to the base
     * @param string $base Base type
     * @return string
     */
    public function staticPath ($file, $base = null)
    {
        $config = Zend_Registry::get('config');
        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
        
        $frontController = Zend_Controller_Front::getInstance();
        //front controller
        $baseUrl = $frontController->getBaseUrl();
        
        if (empty($base)) {
            //Base
            $url = $baseUrl;
        } elseif ($base == 'js') {
            //Java script
            $url = $baseUrl . '/static-' . $config->version . '/js';
        } elseif ($base == 'css') {
            //Css
            $url = $baseUrl . '/static-' . $config->version . '/css';
        } elseif ($base == 'img') {
            //Image
            $url = $baseUrl . '/static-' . $config->version . '/img';
        } elseif ($base == 'yui') {
            //YUI
            $url = $baseUrl . '/static-' . $config->version . '/yui/build';
        } elseif ($base == 'other') {
            //Full path
        } elseif ($base == 'legend') {
            //Legend
            $url = $baseUrl . '/legend';
        } elseif ($base == 'upload') {
            //Upload
            $url = $baseUrl . '/static-' . $config->version . '/upload';
        } elseif ($base == 'jar') {
               //Jar
               $url = $baseUrl . '/static-' . $config->version . '/jar';
        }
        
        if(! empty($config->staticserverUrls)) {
            $staticserverUrls = $config->staticserverUrls->toArray();
            shuffle($staticserverUrls);

            return $staticserverUrls[0] . $url . $file;
        } else {
            return $url . $file;
        }
    }

    /**
     * Get full path
     * 
     * @param string $protocol
     * @return string
     */
    public function getFullPath ($protocol = NULL)
    {
        if (empty($protocol)) {
            $pageURL = 'http';
            if ($_SERVER["HTTPS"] == "on") {
                $pageURL .= "s";
            }
        } else {
            $pageURL = $protocol;
        }
        $pageURL .= "://";
        if ($_SERVER["SERVER_PORT"] != "80") {
            $pageURL .= $_SERVER["SERVER_NAME"] . ":" . $_SERVER["SERVER_PORT"] . $_SERVER["REQUEST_URI"];
        } else {
            $pageURL .= $_SERVER["SERVER_NAME"] . $_SERVER["REQUEST_URI"];
        }
        return $pageURL;
    }

}