/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editor.js 3520 2013-12-20 18:46:57Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */


/**
 * @private
 * @method _setInitialContent
 * @param {Boolean} raw Don't add events.
 * @description This method will open the iframes content document and write the textareas value into it, then start the body.onload checking.
 */
YAHOO.widget.Editor.prototype._setInitialContent = function(raw) {
    var Dom = YAHOO.util.Dom,
        Event = YAHOO.util.Event,
        Lang = YAHOO.lang;

    var value = ((this._textarea) ? this.get('element').value : this.get('element').innerHTML),
        doc = null;

    if (value === '') {
        value = '<br>';
    }

    var html = Lang.substitute(this.get('html'), {
            TITLE: this.STR_TITLE,
            CONTENT: this._cleanIncomingHTML(value),
            CSS: this.get('css'),
            HIDDEN_CSS: ((this.get('hiddencss')) ? this.get('hiddencss') : '/* No Hidden CSS */'),
            EXTRA_CSS: ((this.get('extracss')) ? this.get('extracss') : '/* No Extra CSS */')
        }),
        check = true;

    html = html.replace(/RIGHT_BRACKET/gi, '{');
    html = html.replace(/LEFT_BRACKET/gi, '}');

    if (document.compatMode != 'BackCompat') {
        html = this._docType + "\n" + html;
    } else {
    }


    if (this.browser.ie || this.browser.webkit || this.browser.opera || (navigator.userAgent.indexOf('Firefox/1.5') != -1)) {

        //FIXME: IE11 fix
        if(this.browser.newIe && this.browser.newIe >= 11) {
            this.browser.ie = 0;
        }

        //Firefox 1.5 doesn't like setting designMode on an document created with a data url
        try {
            //Adobe AIR Code
            if (this.browser.air) {
                doc = this._getDoc().implementation.createHTMLDocument();
                var origDoc = this._getDoc();
                origDoc.open();
                origDoc.close();
                doc.open();
                doc.write(html);
                doc.close();
                var node = origDoc.importNode(doc.getElementsByTagName("html")[0], true);
                origDoc.replaceChild(node, origDoc.getElementsByTagName("html")[0]);
                origDoc.body._rteLoaded = true;
            } else {
                doc = this._getDoc();
                doc.open();
                doc.write(html);
                doc.close();
            }
        } catch (e) {
            //Safari will only be here if we are hidden
            check = false;
        }

    } else {
        //This keeps Firefox 2 from writing the iframe to history preserving the back buttons functionality
        this.get('iframe').get('element').src = 'data:text/html;charset=utf-8,' + encodeURIComponent(html);
    }
    this.get('iframe').setStyle('visibility', '');
    if (check) {
        this._checkLoaded(raw);
    }
};

/**
 * 
 */
YAHOO.widget.Editor.prototype._render = function() {
    //FIXME: IE11 fix
    if(this.browser.newIe && this.browser.newIe >= 11) {
        this.browser.ie = this.browser.newIe;
    }

    this._defaultToolbar.titlebar = false;
        
    this.upload('insertimage', gt.config.httpPath + '/uploads/upload', 'file');
    this.upload('createlink', gt.config.httpPath + '/uploads/upload', 'file');

    YAHOO.widget.Editor.superclass._render.apply(this, arguments);
    var self = this;

    //Render the panel in another thread and delay it a little..
    window.setTimeout(function() {
        self._renderPanel.call(self);
    }, 800);
};

YAHOO.widget.Editor.prototype.upload = function (type, uploadUrl, fieldName) {
    
    var editorName = this.get('name');
    
    this.addListener('toolbarLoaded', function () {
        
        this.toolbar.addListener(type + 'Click', function (o) {

            var imgPanel = new YAHOO.util.Element(editorName + '-panel');
            imgPanel.on('contentReady', function () {

                var Dom = YAHOO.util.Dom;

                if (!Dom.get(editorName + '_' + type + '_upload')) {
                    var label = document.createElement('label');
                    label.innerHTML = '<strong>Upload:</strong>' + '<input type="file" id="' + editorName + '_' + type + '_upload' + '" name="' + fieldName + '">' + '</label>';

                    var elem = Dom.get(editorName + '_' + type + '_url');
                    Dom.getAncestorByTagName(elem, 'form').encoding = 'multipart/form-data';

                    Dom.insertAfter(label, elem.parentNode);

                    YAHOO.util.Event.on(editorName + '_' + type + '_upload', 'change', function (ev) {
                        YAHOO.util.Event.stopEvent(ev);
                        YAHOO.util.Connect.setForm(elem.form, true, true);
                        var c = YAHOO.util.Connect.asyncRequest('POST', uploadUrl, {
                            upload: function (r) {
                                //Strip pre tags
                                //var resp = r.responseText.replace(/<pre>/i, '').replace(/<\/pre>/i, '');
                                var resp = r.responseText;

                                var o = eval('(' + resp + ')');
                                if (o.status == 'UPLOADED') {
                                    Dom.get(editorName + '_' + type + '_upload').value = '';
                                    Dom.get(editorName + '_' + type + '_url').value = o.url;

                                    Dom.get(editorName + '_' + type + '_url').focus();
                                    Dom.get(editorName + '_' + type + '_upload').focus();
                                } else {
                                    debug.warn('Received invalid status value while uploading a file');
                                }
                                
                            }
                        });
                        return false;
                    });
                }
            });

        });

    });

};