/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Info.js 2844 2012-07-09 08:54:35Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.info = new Giscuit.Control.Info();
    }
});

/**
 * Class: Giscuit.Control.Info
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Info = OpenLayers.Class(Giscuit.Control, {

    panel: null,
    
    button: null,
    
    clickControl: null,
    
    tree: null,
    
    selected: null,
    
    tooltipShown: false,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        this.button = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon(10), 
            id: "gtControlInfo", 
            onclick: {
                fn: function() {
                    if(self.active) {
                        self.deactivate();
                    } else {
                        self.activate();
                    }
                }
            },
            title: gt.translate._('information'), 
            container: "gtMapControls",
            type: "checkbox"
        });
    },
    
    activate: function () {
        gt.map.deactivateAll();

        var activated = Giscuit.Control.prototype.activate.call(this);
                        
        if (activated == true) {
            //Check button
            this.button.set('checked', true);
            
            //Tooltip
            if (! this.tooltipShown && gt.config.tooltip == 't') {
                gt.tooltip.cfg.setProperty('text', gt.translate._('infoHelp'));
                gt.tooltip.showTooltip();
                this.tooltipShown = true;
            }
            
            this.clickControl = new Giscuit.Control.Click({
                'clickCallback': Giscuit.Util.bind(this.trigger, this)
            });
            gt.map.addControl(this.clickControl);
            this.clickControl.activate();
        }

        return activated;
    },

    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);

        if (deactivated == true) {
            //Destroy control
            this.clickControl.deactivate();
            this.clickControl.destroy();
            this.clickControl = null;
            
            //Uncheck button
            this.button.set('checked', false);
        }
        
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        return deactivated;
    },

    clear: function() {
        this.deactivate();
        
        //Destroy tree
        if(this.tree != null) {
            this.tree.destroy();
            this.tree = null;
        }
        
        Giscuit.Control.prototype.clear.call(this);
    },
    
    get: function (xy, lonlat, size, extent) {
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('information') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.info.pkg.html');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }
        gt.accord.openPanel(this.getPanelIndex());
       
        var getInfoCallback = {
            success: this.getSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var requestString = gt.config.httpToolPath + 
            "/info/index/x/" + xy.x + 
            "/y/" + xy.y + 
            "/width/" + size.w +
            "/height/" + size.h +
            "/lon/" + lonlat.lon +
            "/lat/" + lonlat.lat +
            "/bbox/" + extent.toBBOX() +
            gt.map.getLayersString();
              
        var request = YAHOO.util.Connect.asyncRequest('GET', requestString, getInfoCallback);

    },
    
    getSuccess: function (o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        gt.map.addJsInfoResults(responseText.records);
        
        if(responseText.records.length == 0) {
            gt.accord.setBody(this.getPanelIndex(), gt.translate._('emptySelectResult'));
            return;
        }
        gt.accord.setBody(this.getPanelIndex(), '<div id="gtInfoTree" class="ygtv-highlight"></div>');

        this.tree = new YAHOO.widget.TreeView("gtInfoTree", responseText.records);
        this.tree.subscribe('clickEvent', function () {
            return false;
        });
        
        this.tree.subscribe("focusChanged", function(oArgs) {
            if(oArgs.newNode == null) {
                return;
            }
            
            if(oArgs.newNode.depth == 1) {
                if(typeof(oArgs.newNode.data.the_geom) != 'undefined') {
                
                    var geometry = OpenLayers.Geometry.fromWKT(oArgs.newNode.data.the_geom);
                                            
                    if(this.selected != null) {
                        gt.layer.vector.destroyFeatures([this.selected]);
                        this.selected = null;
                    }
                    this.selected = new OpenLayers.Feature.Vector(geometry);
                    
                    if(this.selected.geometry.CLASS_NAME.toLowerCase().indexOf('polygon') != -1 &&
                        this.selected.geometry.getBounds().containsBounds(gt.map.getExtent(), true) == true) {
                        
                        //
                    } else {
                        gt.layer.vector.addFeatures([this.selected]);
                    }
                }

            } else {
                if(this.selected != null) {
                    gt.layer.vector.destroyFeatures([this.selected]);
                    this.selected = null;
                }
            }
                                
        }, null, this);
                        
        this.tree.render();
        
        var rootChildren = this.tree.getRoot().children;
        if(rootChildren.length == 1) {
            if(rootChildren[0].children.length == 1) {
                rootChildren[0].children[0].expand();
                rootChildren[0].children[0].focus();
            }
        }
         
    },
    
    trigger: function (evt) {
        var xy = gt.map.getLayerPxFromViewPortPx(evt.xy);
        var lonlat = gt.map.lastInfoClick = gt.map.getLonLatFromViewPortPx(evt.xy);
        var size = gt.map.getSize();
        var extent = gt.map.getExtent();
        
        this.get(xy, lonlat, size, extent);
    },
    
    CLASS_NAME: "Giscuit.Control.Info"
});