Giscuit.Layer.Jobs = OpenLayers.Class(OpenLayers.Layer.Vector, {
    
    defaultField: 'title',
    
    searchScale: 5000,

    initialize: function(name, options) {
        //Required
        this.visibility = true;
        
        var newArguments = [];
        newArguments.push(name, options);

        //Name
        newArguments[0] = 'Jobs';
        
        //Options
        newArguments[1] = {
            //Set layer projection
            'projection': 'EPSG:4326'
        };
        
        //Call constructor
        OpenLayers.Layer.Vector.prototype.initialize.apply(this, newArguments);
        
        //Events
        this.events.on({
            'visibilitychanged': this.load
        });
        
        var defaultStyle = OpenLayers.Util.applyDefaults({
            fillOpacity: 1 ,
            pointRadius: 7 ,
            strokeWidth: 2 ,
            fillColor: '${__getFillColor}' ,
            strokeColor: '#FFFFFF'
        }, OpenLayers.Feature.Vector.style['default']);
        
        var selectStyle = OpenLayers.Util.applyDefaults({
            fillOpacity: 1 ,
            pointRadius: 7 ,
            strokeWidth: 2 ,
            fillColor: '${__getFillColor}' ,
            strokeColor: '#FFFFFF'
        }, OpenLayers.Feature.Vector.style['select']);

        
        var style = new OpenLayers.StyleMap({
            'default': defaultStyle,
            'select': OpenLayers.Feature.Vector.style['select']
        });

        style.styles['select'].context = style.styles['default'].context = {
            '__getFillColor': function(feature) {
                var status = feature.attributes.status;
                var color = '#999999';
                if (status == 'pending') {
                    color = '#EE0000';
                } else if (status == 'in_progress') {
                    color = '#E69138';
                } else if (status == 'completed') {
                    color = '#38761D';
                } else if (status == 'declined') {
                    color = '#999999';
                }
                    
                return color;
            }
        };

        this.styleMap = style;
        
        //Legend
        gt.control.legend.events.register("draw", this, function() {
            //Visibility
            var visible = gt.control.layers.getSettings(
                'view',
                gt.control.layers.getLayerIdByClassName(this.name)
            );
            if(! visible) {
                return;
            }
            
            if(gt.config.layout == 'desktop') {
                var panel = gt.control.legend.panel.children[1];
            } else {
                var panel = $("#gtLegend > div[data-role='content']").get(0);
            }
            
            var html = '<ul><strong>' + gt.translate._('jobs') + '</strong>';
                html += '<li style="margin-bottom: 3px;">' +
                        '<span style="background-color: #EE0000; display:inline-block; width:30px; height:20px;">&nbsp;</span> ' +
                        '<span>' + gt.translate._('status') + ': ' + gt.translate._('pending') + '</span></li>';
                html += '<li style="margin-bottom: 3px;">' +
                        '<span style="background-color: #E69138; display:inline-block; width:30px; height:20px;">&nbsp;</span> ' +
                        '<span>' + gt.translate._('status') + ': ' + gt.translate._('in_progress') + '</span></li>';
                html += '<li style="margin-bottom: 3px;">' +
                        '<span style="background-color: #38761D; display:inline-block; width:30px; height:20px;">&nbsp;</span> ' +
                        '<span>' + gt.translate._('status') + ': ' + gt.translate._('completed') + '</span></li>';
                html += '<li style="margin-bottom: 3px;">' +
                        '<span style="background-color: #999999; display:inline-block; width:30px; height:20px;">&nbsp;</span> ' +
                        '<span>' + gt.translate._('status') + ': ' + gt.translate._('declined') + '</span></li>';
            html += '</ul>';
            
            panel.innerHTML += html;
        });
    },
    
    load: function(evt) {
        if(! this.visibility) {
            this.removeAllFeatures();
        }
        
        var jobsDt = gt.control.jobs.dt;
        jobsDt.getDataSource().sendRequest('', {
            success: jobsDt.onDataReturnInitializeTable,
            scope: jobsDt,
            argument: jobsDt.getState()
        });
    },
    
    add: function(records) {                    
        var newFeatures = [];
        for(var i = 0; i < records.length; i++) {
            //Geometry
            var point = new OpenLayers.Geometry.fromWKT(records[i].the_geom);
            var newFeature = new OpenLayers.Feature.Vector(point);
            
            //Parse date fields
            var date = new Date(parseFloat(records[i].date) * 1000);
            var currentDay = ( date.getDate() < 10 ? "0" : "" ) + date.getDate();
            var currentMonth = ( (date.getMonth() + 1) < 10 ? "0" : "" ) + (date.getMonth() + 1);
            var currentHours = ( date.getHours() < 10 ? "0" : "" ) + date.getHours();
            var currentMinutes = ( date.getMinutes() < 10 ? "0" : "" ) + date.getMinutes();
            var dateStr = date.getFullYear() + '-' + currentMonth + '-' + currentDay + ' ' +
                currentHours + ':' + currentMinutes;
            
            var deadline = new Date(parseFloat(records[i].deadline) * 1000);
            var currentDay = ( deadline.getDate() < 10 ? "0" : "" ) + deadline.getDate();
            var currentMonth = ( (deadline.getMonth() + 1) < 10 ? "0" : "" ) + (deadline.getMonth() + 1);
            var currentHours = ( deadline.getHours() < 10 ? "0" : "" ) + deadline.getHours();
            var currentMinutes = ( deadline.getMinutes() < 10 ? "0" : "" ) + deadline.getMinutes();
            var deadlineStr = deadline.getFullYear() + '-' + currentMonth + '-' + currentDay + ' ' +
                currentHours + ':' + currentMinutes;
            
            //Attributes
            newFeature.attributes = {
                deadline: deadlineStr,
                date: dateStr,
                status: records[i].status,
                priority: records[i].priority,
                title: records[i].title
            };

            newFeatures.push(newFeature);
        }
        
        //Add or update elements
        this.removeAllFeatures();
        this.addFeatures(newFeatures);
    },

    CLASS_NAME: "Giscuit.Layer.Jobs"
});
