/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editgeom.js 1600 2011-08-09 13:52:10Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
//
Giscuit.Mobile.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.editgeom = new Giscuit.Mobile.Control.Editgeom();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Editgeom
 * 
 *  Inherits from:
 *   - <Giscuit.Control.Editgeom>
 */
Giscuit.Mobile.Control.Editgeom = OpenLayers.Class(Giscuit.Control.Editgeom, {
    
    initialize: function (options) {
        
        //
        if(gt.config.guest == 't') {
            return;
        }
        
        this.initializeTouchOffset();
        
        $("#gtMapControls").append('<div>'
                + '<div class="ui-bar ui-bar-c" id="gtEditButton">'
                + '<div class="gtMapControlsElImage" style="background-position: -384px 50%;"></div>'
                + '<div class="gtMapControlsElText">' + gt.translate._('editing') + '</div>'
                + '</div></div>');
                    
        $("#gtEditButton").on(gt.config.clickEvt, this, function(evt) {
            //
            evt.stopImmediatePropagation();
            
            if(evt.data.active) {
                evt.data.deactivate();
            } else {
                //Hide map controls
                gt.init.layout.hideMapControls();
                
                evt.data.activate();
            }
            
            return false;
        });
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
    },
    
    activate: function (callback) {
        //
        if(typeof(callback) != 'function') {
            callback = function(){};
        }
        
        gt.map.deactivateAll();
        
        gt.map.globalSelectControl.deactivate();
        
        if (! gt.control.offline.isOnline && 
            (gt.control.offline.files.length || gt.control.offline.synchronizeIds.length)) {
            if (confirm(gt.translate._('synchronizationNotFinished'))) {
                gt.control.offline.files = [];
                gt.control.offline.synchronizeIds = [];
            } else {
                return false;
            }
        }
        
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        if (activated) {
            if (! gt.control.offline.isOnline) {
                this.loaded = true;
            }
            this.enableEditing(callback);
        }
        
        return activated;
    },
    
    enableEditing: function(callback) {
        if(! $('#gtEditattr').length) {
            var html = '<div data-role="page" id="gtEditattr" data-dom-cache="true" data-url="gtEditattr">' +
                '<div data-role="header" >' +
                    '<h1>' + gt.translate._('editAttributes') + '</h1>' +
                    '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                        gt.translate._('map') +
                        '</a>' + 
                    '</div>' +                 
                '<div data-role="content"></div>' +        
            '</div>';
            
            $.mobile.pageContainer.append(html);
            $('#gtEditattr').page();
        }
        
        var editLayerData = gt.control.layers.getEditLayer();
        
        if (editLayerData && gt.control.offline.isOnline) {
            this.editLayer = gt.layer.edit;
            
            $.mobile.showPageLoadingMsg();
            
            var url = gt.config.httpMobilePath + "/privatelayers/editenable/rand/" + Math.random();
            $.getJSON(url, Giscuit.Util.bind(function(data) {
                $("#gtEditButton").addClass('ui-btn-active');
                
                callback.call();
                
                this.initializeControls(data);
                
                if(gt.map.getScale() < data.edit_maxscale) {
                    $.mobile.hidePageLoadingMsg();
                }

            }, this))
            .error(Giscuit.Util.jqmAjaxError);
            
        } else if (editLayerData && ! gt.control.offline.isOnline && editLayerData.offline) {
            
            this.editLayer = gt.layer[editLayerData['class']];
            gt.snap.edit.layer = this.editLayer; 
            
            $("#gtEditButton").addClass('ui-btn-active');
            
            var mapGeometryType = Giscuit.Util.getStorageObject('gtMapGeometryType');
            
            callback.call();
            
            this.initializeControls({
                'editgeom': true,
                'type': mapGeometryType['layer_' + editLayerData.id],
                'changesCount': 0,
                'history': false
            });
            
        } else {
            this.deactivate(callback, false);
            Giscuit.Util.jqmMessage(gt.translate._('noEditLayerSelected'));
        }
    },
    
    disableEditing: function(callback) {
        if(gt.control.offline.isOnline) {
            var url = gt.config.httpToolPath + "/privatelayers/editdisable/rand/" + Math.random();
        
            $.getJSON(url, Giscuit.Util.bind(function(data) {
                //
                callback.call();
                
                //
                if(this.editLayer && this.editLayer.map) {
                    this.editLayer.setVisibility(false);
                    gt.map.refresh();
                }
                
                //
                $("#gtEditButton").removeClass('ui-btn-active');
                
            }, this))
            .error(Giscuit.Util.jqmAjaxError);
        
            
        } else {
            //
            $("#gtEditButton").removeClass('ui-btn-active');
            
            //
            callback.call();
        }
    },
    
    /**
     * Edit
     */
    editAction: function(evt) {
        if(evt.feature == this.controls.editControl.feature) {
            return;
        }
        
        evt.feature.originalGeometry = evt.feature.geometry.clone();
    },
    
    /**
     * Delete
     */
    deleteAction: function (evt) {
        setTimeout(Giscuit.Util.bind(function() {
            var answer = confirm(gt.translate._('deleteConfirm'));
            if (answer) {
                if(gt.control.offline.isOnline) {
                    $.mobile.showPageLoadingMsg();
                
                    this.editLayer.removeFeatures(evt.feature);
                
                    var url = gt.config.httpMobilePath + "/editgeom/delete/id/" + evt.feature.attributes.gid;
                    $.getJSON(url, Giscuit.Util.bind(function(data) {
                        this.events.triggerEvent("deletefeature", evt.feature.attributes.gid);
                        this.backActivate();
                        $.mobile.hidePageLoadingMsg();
                    }, this))
                        .error(Giscuit.Util.jqmAjaxError);
                } else {
                    this.editLayer.removeFeatures(evt.feature);
                    
                    //Delete file
                    var editLayerSettings = gt.control.layers.getEditLayer();
                    var editForm = Giscuit.Util.getStorageObject('gtEditForm');
                    var fileNames = gt.control.offline.getFileElementsNames(editForm['layer_' + editLayerSettings.id]);
                    
                    if (fileNames.length) {
                        var filesTable = new Giscuit.Util.Db.Files({tableName: 'files_' + editLayerSettings.id});
                        
                        for (var key in evt.feature.attributes) {
                            for (var i = 0; i < fileNames.length; i++) {
                                if (key == fileNames[i] && evt.feature.attributes[key]) {
                                    filesTable['delete'](evt.feature.attributes[key]);
                                }
                            }
                        }
                    }
                    
                    //Delete feature
                    if(evt.feature.attributes.hstatus == 'C') {
                        this.editLayer.protocol['delete'](evt.feature);
                    } else {
                        evt.feature.attributes.hstatus = 'D';
                        this.editLayer.protocol.update(evt.feature);
                    }
                }
            } else {
                this.controls.deleteControl.unselect(evt.feature);
            }
        }, this), 100);
    },
    
    /**
     * Back
     */
    backAction: function() {
        $.mobile.showPageLoadingMsg();
        
        var url =  gt.config.httpMobilePath + "/editgeom/back";
        
        $.getJSON(url, Giscuit.Util.bind(function(data) {
            if(data.changesCount == 0) {
                this.backDeactivate();
            } else {
                this.backActivate();
            }
            
            this.editLayer.mergeNewParams({
                'rand' : Math.random()
            });
            
            $.mobile.hidePageLoadingMsg();
        }, this))
            .error(Giscuit.Util.jqmAjaxError);
        
    } ,
    
    /**
     * Editsave
     */
    editsaveControl: function() {
        var panelDiv = null;
        
        this.controls.editsaveControl = new OpenLayers.Control({
            'displayClass': 'olControlEditsave',
            type: OpenLayers.Control.TYPE_BUTTON,
            trigger: Giscuit.Util.bind(function() {
                if(panelDiv.className.indexOf('olControlEditsaveItemActive') === -1) {
                    return;
                }
                
                var feature = this.controls.editControl.feature;
                
                if(gt.control.offline.isOnline) {
                    $.mobile.showPageLoadingMsg();
                    var url = gt.config.httpMobilePath + '/editgeom/editgeometry';
                    
                    $.ajax({
                        'type': 'POST',
                        'url': url,
                        'data': {
                            'id': feature.attributes.gid ,
                            'geometry': feature.geometry.toString()
                        },
                        'success': function(data) {
                            var error = null;
                            
                            //Destroy original
                            if(typeof(feature.originalGeometry) != 'undefined') {
                                feature.originalGeometry.destroy();
                                feature.originalGeometry = undefined;
                            }
                            feature.state = null;
                            
                            if (data.errors) {
                                for (var error in data.errors) {
                                    Giscuit.Util.jqmMessage(data.errors[error]);
                                }
                            } else {
                                this.backActivate();
                                
                                this.unselectAll();
                            }
                            
                            if(! error) {
                                $.mobile.hidePageLoadingMsg();
                            }
                        },
                        'error': Giscuit.Util.jqmAjaxError ,
                        'dataType': 'json' ,
                        'context': this
                    });
                } else {
                    feature.state = null;
                    
                    if(typeof(feature.attributes.hstatus) == 'undefined') {
                        feature.attributes.hstatus = 'U';
                    }
        
                    this.editLayer.protocol.update(feature);
                    
                    this.unselectAll();
                }
                                
                disable();
            }, this),
            title: gt.translate._('saveFeature')
        });   
        
        //
        this.controls.editControl.events.on({
            "activate": function() {
                panelDiv.style.display = 'block';
            },
            "deactivate": function() {
                panelDiv.style.display = 'none';
            },
            scope: this
        });
        
        //Unselect feature
        this.editLayer.events.on({
            'afterfeaturemodified': function() {
                disable();
            }
        });
        
        //Feature modified
        this.editLayer.events.on({'featuremodified': function() {
            enable();
        }});
        
        //Layer reload
        this.editLayer.events.on({'loadstart': function() {
            disable();
        }});
        
        //
        this.secondaryControlPanel.addControls(this.controls.editsaveControl);
        panelDiv = this.controls.editsaveControl.panel_div;
        panelDiv.style.display = 'none';
        
        //Disable
        var disable = function() {
            panelDiv.className = 'olControlEditsaveItemInactive olButton';
        };
        
        //Enable
        var enable = function() {
            panelDiv.className = 'olControlEditsaveItemActive olButton';
        };
    },
    
    /**
     * Editattr
     */
    editattrControl: function () {
        this.controls.editattrControl = new OpenLayers.Control.SelectFeature(this.editLayer, {
            'displayClass': 'olControlEditattr',
            title: gt.translate._('editAttributes')
        });
        
        this.controls.editattrControl.events.on({
            'featurehighlighted': Giscuit.Util.bind(function(evt) {
                evt.feature.originalGeometry = evt.feature.geometry.clone();
                gt.control.editattr = new Giscuit.Mobile.Control.Editattr(evt.feature, 0);
            }, this) ,
            'featureunhighlighted': this.editFeatureUnhighlighted,
            scope: this
        });
        
        //
        this.controlPanel.addControls(this.controls.editattrControl);
    },
    
    invalidEditScale: function(editMaxscale) {
        Giscuit.Util.jqmMessage(
            gt.translate._('layerIsEditableFromScale') + ' 1:' + editMaxscale 
        );
    },
    
    //Initialize Controls
    initializeControls: function (data) {
        //
        var self = this;
        this.controls = {};
        
        //Activate snap
        if(typeof(gt.layer.snap) != 'undefined') {
            gt.layer.snap.setVisibility(true);
        }
        if (typeof(gt.snap.edit) != 'undefined') {
            gt.snap.edit.activate();
        }
        
        if(gt.control.offline.isOnline) {
            //Invalid edit scale message
            if(gt.map.getScale() > data.edit_maxscale) {
                this.invalidEditScale(data.edit_maxscale);
            }
            
            //Style
            this.editLayer.styleMap.styles['default'].rules = [];
            this.editLayer.styleMap.styles['default'].addRules(gt.map.getStyleRules(data.sld, true));
            
            if(this.editLayer && this.editLayer.map) {
                this.editLayer.setVisibility(true);
                gt.map.refresh();
            }
        } else {
            var vertexStyle = new OpenLayers.Style();
            vertexStyle.defaultStyle = OpenLayers.Util.applyDefaults({
                pointRadius: gt.map.pointRadius ,
                strokeWidth: 3,
                display: 'block'
            }, OpenLayers.Feature.Vector.style['default']);
            this.editLayer.styleMap.styles['vertex'] = vertexStyle;
        }

        //Initialize panels
        var controlPanelDiv = document.createElement('div');
        controlPanelDiv.className = 'gtEditgeomToolbar';
        OpenLayers.Util.getElement('gtMap').appendChild(controlPanelDiv);
        this.controlPanel = new OpenLayers.Control.Panel({
            'div': controlPanelDiv
        });
        
        var secondaryControlPanelDiv = document.createElement('div');
        secondaryControlPanelDiv.className = 'gtEditgeomToolbar gtEditgeomSecondaryToolbar';
        OpenLayers.Util.getElement('gtMap').appendChild(secondaryControlPanelDiv);
        this.secondaryControlPanel = new OpenLayers.Control.Panel({
            'div': secondaryControlPanelDiv
        });

        //Primary panel controls
        if (data.editgeom) {
            this.addControl(data.type);
            
            this.editControl();
            this.controls.editControl.dragVertex = this.dragVertex;
            this.controls.editControl.handlers.drag.callbacks.down = function(px) {
                //
                this.vertex = null;
                var feature = this.layer.getFeatureFromEvent(this.handlers.drag.evt);
                
                //If no feature was selected try to select using Bbox
                if(! feature) {
                    feature = gt.map.getClosestFeature(self.editLayer, px, gt.map.selectTolerance);
                }
                
                //
                if (feature) {
                    this.dragStart(feature);
                } else if (this.feature && this.clickout) {
                    this.unselectFeature(this.feature);
                }
            };
            
            this.editattrControl();
            gt.map.addBboxHandler(this.controls.editattrControl, this.editLayer, gt.map.selectTolerance);
            
            this.backControl(data.history, data.changesCount);
            this.deleteControl();
            gt.map.addBboxHandler(this.controls.deleteControl, this.editLayer, gt.map.selectTolerance);

        } else {
            this.editattrControl();
        }

        this.layerNameControl();

        //
        gt.map.addControl(this.controlPanel);
        
        //Secondary panel controls
        if (data.editgeom) {
            this.finalizeControl(data.type);
            this.undovertexControl(data.type);
            this.addgpsnodeControl(data.type);
            
            this.editsaveControl();
            this.editgpsnodeControl(data.type);
            this.removevertexControl(data.type);
            
            gt.map.addControl(this.secondaryControlPanel);
        }
        
        //
        if(data.changesCount > 0) {
            this.backActivate();
        }
    },
    
    dragVertex: function(vertex, pixel) {
        var self = gt.control.editgeom;
        pixel.x += self.touchOffsetToleranceX;
        pixel.y += self.touchOffsetToleranceY;
        
        OpenLayers.Control.ModifyFeature.prototype.dragVertex.apply(this, [vertex, pixel]);
    },

    CLASS_NAME: "Giscuit.Mobile.Control.Editgeom"
});