/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Info.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.info = new Giscuit.Mobile.Control.Info();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Info
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Info = OpenLayers.Class(Giscuit.Control, {

    type: OpenLayers.Control.TYPE_TOGGLE,
    
    clickControl: null,
    
    selectControl: null,

    selected: null,
    
    fileFieldsCounter: 0,
    
    offlineData: {},
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        $("#gtMapControls").append('<div>'
            + '<div class="ui-bar ui-bar-c" id="gtInfoButton">'
            + '<div class="gtMapControlsElImage" style="background-position: -128px 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('info') + '</div>'
            + '</div></div>');
        
        // 
        $("#gtInfoButton").on(gt.config.clickEvt, this, function(evt) {
            //
            evt.stopImmediatePropagation();
            
            if(evt.data.active) {
                evt.data.deactivate();
            } else {
                evt.data.activate();
            }
            
            return false;
        });
        
    },
    
    render: function () {
        var html = '<div data-role="page" id="gtInfo" data-dom-cache="true" data-url="gtInfo">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('information') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' + 
            '</div>' +
         
            '<div data-role="content">' + 
            '</div>' +
            
        '</div>';
            
        $.mobile.pageContainer.append(html);
        $('#gtInfo').page();
    },

    activate: function () {
        gt.map.deactivateAll();

        var activated = Giscuit.Control.prototype.activate.call(this);
        
        //
        $("#gtInfoButton").addClass('ui-btn-active');
        
        //Hide map controls
        gt.init.layout.hideMapControls();
                        
        if (activated == true) {
            
            //
            if(! $('#gtInfo').length) {
                this.render();
            }
            
            if(gt.control.offline.isOnline) {
                this.clickControl = new Giscuit.Control.Click({
                    'clickCallback': Giscuit.Util.bind(this.trigger, this)
                });
                gt.map.addControl(this.clickControl);
                this.clickControl.activate();
            } else {
                var layerType = Giscuit.Util.getStorageObject('gtLayerType');
                
                //Select control
                var offlineLayersArr = [];
                for (var i = 0; i < gt.control.layers.records.length; i++) {
                    if (gt.control.layers.records[i].offline == true && 
                        layerType['layer_' + gt.control.layers.records[i].id] == 'vector') {
                        
                        offlineLayersArr.push(gt.layer[gt.control.layers.records[i]['class']]);
                    }
                }
                this.selectControl = new OpenLayers.Control.SelectFeature(offlineLayersArr);
                this.addBboxHandler(this.selectControl, gt.map.selectTolerance);
                gt.map.addControl(this.selectControl);
                
                this.selectControl.activate();
            }
        }

        return activated;
    },

    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        $("#gtInfoButton").removeClass('ui-btn-active');

        if (deactivated == true) {
            if(this.clickControl) {
                this.clickControl.destroy();
                this.clickControl = null;
            } else if (this.selectControl) {
                this.selectControl.unselectAll();
                this.selectControl.destroy();
                this.selectControl = null;
            }
        }
        
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        return deactivated;
    },
    
    get: function (xy, lonlat, size, extent) {
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        $.mobile.showPageLoadingMsg();
        
        var url = gt.config.httpMobilePath + 
            "/info/index/x/" + xy.x + 
            "/y/" + xy.y + 
            "/width/" + size.w +
            "/height/" + size.h +
            "/lon/" + lonlat.lon +
            "/lat/" + lonlat.lat +
            "/bbox/" + extent.toBBOX() +
            gt.map.getLayersString();
        
        $.getJSON(url, Giscuit.Util.bind(this.set, this))
            .error(Giscuit.Util.jqmAjaxError);
    },
    
    set: function (data) {
        if (gt.control.offline.isOnline) {
            gt.map.addJsInfoResults(data.records);
        }
        
        //
        Giscuit.Util.jqmCleanPage('gtInfo');
        
        //Empty select result
        if(data.records.length == 0) {
            Giscuit.Util.jqmMessage(gt.translate._('emptySelectResult'));

            return;
        }
        
        var html = '<ul data-role="listview">';

        for(var a=0; a < data.records.length; a++) {
            var layer = data.records[a];
            html += '<li data-role="list-divider">' + layer.html + '</li>';
            
            for(var b=0; b < layer.children.length; b++) {
                var item = layer.children[b];
                if (typeof item.children == 'undefined') {
                    html += '<li>' + item.html + '</li>';
                } else {
                    html += '<li>' + item.html + '<ul>';
                                    
                    for(var c=0; c < item.children.length; c++) {
                        var field = item.children[c];
                        
                        html += '<li>' + field.html + '</li>';
                    }
                    
                    if (gt.control.offline.isOnline && item.the_geom) {
                        html += '<li data-geometry="' + item.the_geom + '"><button type="button" class="gtInfoShowOnMapButton" style="z-index: 1100;">' + gt.translate._('showResultsOnMap') + '</button></li>';
                    } else if (! gt.control.offline.isOnline) {
                        html += '<li data-layer="' + item.layerClass + '" data-feature="' + item.featureId + '"><button type="button" class="gtInfoShowOnMapButton" style="z-index: 1100;">' + gt.translate._('showResultsOnMap') + '</button></li>';
                    }

                    
                    html += '</ul></li>';
                }
            }
        }

        html += '</ul>';
            
        //Render
        var content = $("#gtInfo > div[data-role='content']")
            .html(html)
            .trigger('create');
       
        //
        $('.gtInfoShowOnMapButton').off(gt.config.clickEvt).on(gt.config.clickEvt, this, function(evt) {
            if (gt.control.offline.isOnline) {
                var geometry = OpenLayers.Geometry.fromWKT(
                    $(this.parentNode.parentNode).jqmData('geometry')
                );
                
                if(evt.data.selected != null) {
                    gt.layer.vector.destroyFeatures([evt.data.selected]);
                    evt.data.selected = null;
                }
                evt.data.selected = new OpenLayers.Feature.Vector(geometry);
                
                //
                if(evt.data.selected.geometry.CLASS_NAME.toLowerCase().indexOf('polygon') != -1 &&
                    evt.data.selected.geometry.getBounds().containsBounds(gt.map.getExtent(), true) == true) {
                    
                } else {
                    gt.layer.vector.addFeatures([evt.data.selected]);
                }
            } else {
                var layerClass =  $(this.parentNode.parentNode).jqmData('layer');
                var featureId =  $(this.parentNode.parentNode).jqmData('feature');
                
                evt.data.selectControl.unselectAll();
                evt.data.selectControl.select(gt.layer[layerClass].getFeatureByFid(featureId));
            }
                
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });

        if (data.records.length == 1 && data.records[0].children.length == 1) {
            
            $("div[data-role='page']").each(function (index, value) {
                var el = $(value);
                var page = 'gtInfo&ui-page=';
                if(el.jqmData('url').substr(0, page.length) == page) {
                    $.mobile.changePage('#' + el.jqmData('url'), {
                        'changeHash': true
                    });
                }
            });
            
        } else { 
            $.mobile.changePage($('#gtInfo'), {
                'changeHash': true
            });
        }
        
        $.mobile.hidePageLoadingMsg();
    } ,
    
    trigger: function (evt) {
        var xy = gt.map.getLayerPxFromViewPortPx(evt.xy);
        var lonlat = gt.map.lastInfoClick = gt.map.getLonLatFromViewPortPx(evt.xy);
        var size = gt.map.getSize();
        var extent = gt.map.getExtent();
  
        this.get(xy, lonlat, size, extent);
    },
    
    addBboxHandler: function(control, tolerance) {
        if (control.multipleSelect()) {
            debug.warn('Multiple selection is not supported');
            return;
        }
        
        control.handlers.click = new OpenLayers.Handler.Click(
            control,
            {click: Giscuit.Util.bind(function(evt) {
                var layerType = Giscuit.Util.getStorageObject('gtLayerType');
                var editForm = Giscuit.Util.getStorageObject('gtEditForm');
                
                //Bounds
                var topLeft = gt.map.getLonLatFromViewPortPx({
                    'x': evt.xy.x - tolerance,
                    'y': evt.xy.y - tolerance
                });
                var bottomRight = gt.map.getLonLatFromViewPortPx({
                    'x': evt.xy.x + tolerance,
                    'y': evt.xy.y + tolerance
                });
                
                var bounds = new OpenLayers.Bounds();
                bounds.extend(topLeft);
                bounds.extend(bottomRight);
                
                var boundsGeometry = bounds.toGeometry();
                
                //
                var result = [];
                var records = [];
                this.fileFieldsCounter = 0;
                
                for (var a = 0; a < gt.control.layers.records.length; a++) {
                    var layerSettings = gt.control.layers.records[a];
                    
                    if (! layerSettings.offline || 
                        ! layerSettings.select || 
                        ! layerSettings.view ||
                        layerType['layer_' + layerSettings.id] != 'vector'
                        ) {
                        continue;
                    }
                    
                    var layer = gt.layer[layerSettings['class']];
                    
                    var layerRecord = {
                        'html': layerSettings.name,
                        'children': []
                    };

                    var fileNames = gt.control.offline.getFileElementsNames(editForm['layer_' + layerSettings.id]);
                    
                    for (var b = 0; b < layer.features.length; b++) {
                        var feature = layer.features[b];
                        if (! feature.getVisibility()) {
                            continue;
                        }
                        
                        if (bounds.intersectsBounds(feature.geometry.bounds) &&
                            boundsGeometry.intersects(feature.geometry)) {
                             
                            var featureRecord = {
                                'html': feature.attributes[layerSettings.default_field],
                                'layerClass': layerSettings['class'],
                                'featureId': feature.fid,
                                'children': []
                            };
                            
                            //
                            for (var key in gt.map.removeReservedAttributes(feature.attributes)) {
                                var fieldRecord = {
                                    'html': key + ': ' + feature.attributes[key]
                                }
                                featureRecord.children.push(fieldRecord);
                                
                                if (Giscuit.Util.inArray(fileNames, key) && feature.attributes[key]) {
                                    this.fileFieldsCounter++;
                                    if (typeof(feature.attributes[key]) == 'object') {
                                        var fileId = feature.attributes[key].id;
                                    } else {
                                        var fileId = feature.attributes[key];
                                    }
                                    this.setFileField(layerSettings.id, fileId, key, fieldRecord);
                                }
                            }

                            layerRecord.children.push(featureRecord);
                        }
                    }
                       
                    if(layerRecord.children.length) {   
                        records.push(layerRecord);
                    }
                }
                
                this.offlineData = {};
                this.offlineData.records = records;
                if (! this.fileFieldsCounter) {
                    this.set(this.offlineData);
                }
                
            }, this)}, 
            {}
        );
        
        control.events.on({
            "activate": function() {
                control.handlers.feature.deactivate();
                
                control.handlers.click.setMap(control.map);
                control.handlers.click.activate();
            },
            "deactivate": function() {
                control.handlers.click.deactivate();
            }
        });
    },
    
    setFileField: function (layerId, fileId, fileField, fieldRecord) {
        var filesTable = new Giscuit.Util.Db.Files({tableName: 'files_' + layerId});
        
        filesTable.read({
            id: fileId,
            record: fieldRecord,
            callback: Giscuit.Util.bind(function(result) {
                //
                var fileInfo = result.files[0];
                var extension = fileInfo['name'].split('.').pop().toLowerCase();
                var fieldRecord = result.record;
                
                if (Giscuit.Util.inArray(Giscuit.Util.imageExtensions, extension)) {
                    fieldRecord.html = '<a href=\"javascript: void(0);" ' +
                        'onclick=\"javascript:Giscuit.Util.mobilePhotobox(' +
                        '\'' + fileInfo['name'] + '\', ' + fileInfo['id'] + ', ' + layerId + ');" ' +
                        'title="' + fileInfo['name'] + '">' + fileField + ': ' + fileInfo['name'] + '</a>';
                } else {
                    fieldRecord.html = fileField + ': ' + fileInfo['name'] + 
                        ' (' + gt.translate._('notAvailableInOffline') + ')';
                }
                
                this.fileFieldsCounter--;

                if (! this.fileFieldsCounter) {
                    this.set(this.offlineData);
                }
            }, this)
        });
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Info"
});