/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Coords.js 2744 2012-06-12 07:27:23Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Mobile.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.jobs = new Giscuit.Mobile.Control.Jobs();
        gt.control.jobs.displayJob();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Jobs
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Jobs = OpenLayers.Class(Giscuit.Control, {
    
    records: [],
    
    completedRecords: [],
    
    possibleActions: {},
    
    offlineActions: [],
    
    completed: false,
    
    completedOffset: 0,
    
    completedRecordsPerRequest: 20,
    
    loadMoreButton: true,
    
    layer: null,
    
    currentId: null,
    
    timeoutId: null,
    
    lastSelectedSortId: null,
    
    lastSelectedSortType: 1,
    
    lastSelectedFeature: null,
    
    lastSelectedAction: null,
    
    zoomLevel: null,
    
    acJobId: false,
    
    initialize: function (options) {
        
        if (! gt.pluginParams.jobsPluginIsAllowedWorker) {
            return;
        }
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        this.zoomLevel = gt.config.resolutions.length - 3;
        
        //Add control button
        $("#gtMapControls").append('<div>'
            + '<div class="ui-bar ui-bar-c" id="gtJobsButton">'
            + '<div class="gtMapControlsElImage" style="background-image: url(\'' + gt.config.httpImagePath + '/jobs_mobile.png\');"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('jobs') + '</div>'
            + '</div></div>');
        
        //Show jobs page
        $("#gtJobsButton").on(gt.config.clickEvt, this, function(evt) {
            //
            evt.stopImmediatePropagation();
            
            if(! gt.control.geolocation.point) {
                gt.control.geolocation.start();
            }
            
            //Hide map controls
            gt.init.layout.hideMapControls();
            
            $.mobile.showPageLoadingMsg();
            
            //Render page
            if(! $('#gtJobs').length) {
                evt.data.render();
            }
            
            //Add layer
            evt.data.addLayer();
            
            $.mobile.changePage($('#gtJobs'), {
                'changeHash': true
            });
            
            //Synchronize offline actions
            if (gt.control.offline.isOnline) {
                var offlineActions = Giscuit.Util.getStorageObject('gtJobsActions');
                if (typeof(offlineActions.length) != 'undefined' && offlineActions.length) {
                    var url = gt.config.httpMobilePath + '/jobs/updateaction';
                    
                    $.ajax({
                        'type': 'POST',
                        'url': url,
                        'data': {'actions': offlineActions},
                        'success': evt.data.synchronizeSuccess,
                        'error': Giscuit.Util.jqmAjaxError ,
                        'dataType': 'json' ,
                        'context': evt.data
                    });
                }
            }
            
            return false;
        });
        
        //Offline
        gt.control.offline.events.register('offline', this, function() {
            if (this.layer) {
                gt.control.layers.removeLayer('Jobs');
                this.layer = null;
            }
            $('#gtJobsSortCompleted').addClass('ui-disabled');
        });
        
        //Online
        gt.control.offline.events.register('online', this, function() {
            //FIXME:
            if (this.layer) {
                gt.control.layers.removeLayer('Jobs');
                this.layer = null;
            }
            $('#gtJobsSortCompleted').removeClass('ui-disabled');
        });
    },
    
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        if (! deactivated) {
            if(this.lastSelectedFeature) {
                this.lastSelectedFeature.layer.drawFeature(this.lastSelectedFeature, 'default');
            }
            
            if (this.lastSelectedAction) {
                this.layer.removeFeatures([this.lastSelectedAction]);
            }
        }
        
        return deactivated;
    },
    
    render: function() {
        //Add css
        var css = '.gtJobsLeftDiv {float: left; width: 33%;}' +
            '.gtJobsRightDiv {float:left; width: 66%;}' +
            '.gtJobsList {padding:0px; margin-top:0px}' +
            '.gtJobsList > li {text-overflow: ellipsis; white-space: nowrap; overflow: hidden;}' +
            '.gtJobsHeaderDiv {margin-right: 35px;}' +
            '.gtJobsPopupDiv {padding:10px 20px;}' +
            '.gtJobsHeading {margin:0; padding:0; padding-top:0.8em; clear:both;}' +
            '.gtJobsPriorityHigh {color: #CC5050; font-weight: bold;}' +
            '.gtJobsPriorityMedium {color: #E69138; font-weight: bold;}' +
            '.gtJobsPriorityLow {color: #38761D; font-weight: bold;}';
        $('<style>' + css + '</style>').appendTo('body');
        
        //Add html
        var html = '<div data-role="page" id="gtJobs" data-dom-cache="true" data-url="gtJobs">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('jobs') + '</h1>' +
                '<a href="javascript:void(0);" id="gtJobsRefreshButton" data-icon="refresh" data-direction="reverse" class="ui-btn-left jqm-home">' +
                    gt.translate._('refresh') +
                '</a>' + 
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' + 
            '</div>' +
            '<div data-role="content">' + 
                '<div data-role="navbar">' +
                    '<ul id="gtJobsTypeSelect">' +
                        '<li><a id="gtJobsSortDate" class="ui-btn-active" data-sort="1">' + gt.translate._('date') + '</a></li>' +
                        '<li><a id="gtJobsSortDistance" data-sort="2">' + gt.translate._('distance') + '</a></li>' +
                        '<li><a id="gtJobsSortPriority" data-sort="3">' + gt.translate._('priority') + '</a></li>' +
                        '<li><a id="gtJobsSortCompleted" data-sort="4"><strong>' + gt.translate._('completed') + '</strong></a></li>' +
                    '</ul>' +
                '</div>' +
                '<div id="gtJobsList">' +
    
                '</div>' + 
            '</div>' + 
        '</div>' +
        '<div data-role="page" id="gtJobsDetailed" data-dom-cache="true" data-url="gtJobsDetailed">' +
            '<div data-role="header">' +
                '<h1></h1>' +
                /*
                '<a href="#gtJobs" data-icon="arrow-l" data-iconpos="notext" data-direction="reverse" class="ui-btn-left jqm-home">' +
                    gt.translate._('back') +
                '</a>' + 
                */
                '<a id="gtJobsShowOnMapButton" class="ui-btn-right">' +
                    gt.translate._('showOnMap') +
                '</a>' + 
            '</div>' +
            '<div data-role="content">' +

            '</div>' +   
            '<div data-role="popup" id="gtJobsDetailedComment" data-theme="a" class="ui-corner-all">' +
                '<a href="javascript:void(0);" data-rel="back" data-role="button" data-theme="a" data-icon="delete"' +
                ' data-iconpos="notext" class="ui-btn-right">' + gt.translate._('close') + '</a>' +
                '<div class="gtJobsPopupDiv">' +
                    '<form name="gtJobsActionCommentForm">' +
                    '<h3>' + gt.translate._('enterComment') + '</h3>' +
                    '<input id="gtJobsActionComment" type="text">' +
                    '<button type="submit" data-role="button" id="gtJobsActionCommentSubmit"></button>' +
                    '</form>' +
                '</div>' +
            '</div>' +
            '<div data-role="popup" id="gtJobsDetailedText">' +
                '<p>.</p>' +
            '</div>' +
        '</div>';
    
        $.mobile.pageContainer.append(html);
        $('#gtJobs').page();

        //Events
        $('#gtJobs').on(gt.config.clickEvt, '#gtJobsRefreshButton', this, function (evt) {
            $.mobile.showPageLoadingMsg();
            
            if (evt.data.completed) {
                evt.data.completedOffset = 0;
                evt.data.loadMoreButton = true;
            }
            
            if (! evt.data.layer.visibility) {
                var layerId = gt.control.layers.getLayerIdByClassName('Jobs');
                gt.control.layers.setSettings('view', layerId, true);
            } else {
                evt.data.layer.load();
            }
        });
        
        $('#gtJobs').on('pageshow', this, function(evt) {
            //Set last selected sort option
            if (evt.data.lastSelectedSortId) {
                $('#gtJobsTypeSelect > li > a').removeClass('ui-btn-active');
                $('#' + evt.data.lastSelectedSortId).addClass('ui-btn-active');
            }
            
            //Remove selected elements from map
            if (evt.data.lastSelectedFeature) {
                gt.layer.Jobs.drawFeature(evt.data.lastSelectedFeature, 'default');
            }
            
            if (evt.data.lastSelectedAction) {
                gt.layer.Jobs.removeFeatures([evt.data.lastSelectedAction]);
            }
        });
        
        $('#gtJobsDetailed').on('pageshow', this, function(evt) {
            //Remove selected elements from map
            if (evt.data.lastSelectedFeature) {
                gt.layer.Jobs.drawFeature(evt.data.lastSelectedFeature, 'default');
            }
            
            if (evt.data.lastSelectedAction) {
                gt.layer.Jobs.removeFeatures([evt.data.lastSelectedAction]);
            }
        });
        
        $('#gtJobs').on(gt.config.clickEvt, '#gtJobsLoadMoreButton', this, function (evt) {
            $.mobile.showPageLoadingMsg();
            
            evt.data.completedOffset += evt.data.completedRecordsPerRequest;
            evt.data.layer.load();
        });
        
        $('#gtJobsList').on(gt.config.clickEvt, '.gtJobsItem', this, function (evt) {
            evt.data.currentId = $(this).jqmData('id');
            evt.data.populateDetails();
        });
        
        var detailedPageContent = $("#gtJobsDetailed > div[data-role='content']");
        detailedPageContent.on(gt.config.clickEvt, 'a[data-action]', this, function(evt) {
            if(! gt.control.geolocation.point) {
                Giscuit.Util.jqmMessage(gt.translate._('enableGeolocationTool'));
                return false;
            }
            
            $('#gtJobsActionComment').val('');
            var commentSubmit = $('#gtJobsActionCommentSubmit');
            commentSubmit.html($(this).find('.ui-btn-text').html());
            commentSubmit.jqmData('action', $(this).jqmData('action'));
            commentSubmit.jqmData('reserved', $(this).jqmData('reserved'));
            commentSubmit.button('refresh');
        });
        
        detailedPageContent.on('taphold', '.gtJobsRightDiv li', this, function(evt) {
            $('#gtJobsDetailedText').html('<p>' + $(this).html() + '</p>');
            
            $('#gtJobsDetailedText').popup('open');
            
            return false;
        });
        
        $("form[name=gtJobsActionCommentForm]").on('submit', this, function(evt) {
            $("#gtJobsDetailedComment").popup("close");
            $("#gtJobsDetailed a[data-action]").addClass('ui-disabled');
            var button = $('#gtJobsActionCommentSubmit');
            
            var data = {
                'action': button.jqmData('action'),
                'reserved': button.jqmData('reserved'),
                'name': button.html(),
                'id': evt.data.currentId,
                'comment': $('#gtJobsActionComment').val(),
                'geometry': gt.control.geolocation.point.geometry.toString()
            };
            
            if (gt.control.offline.isOnline) {
                var url = gt.config.httpMobilePath + '/jobs/updateaction';
                
                $.ajax({
                    'type': 'POST',
                    'url': url,
                    'data': {'actions': [data]},
                    'success': function(data) {

                    },
                    'error': Giscuit.Util.jqmAjaxError ,
                    'dataType': 'json' ,
                    'context': evt.data
                });
            } else {
                evt.data.offlineActions.push(data);
                Giscuit.Util.setStorageObject('gtJobsActions', evt.data.offlineActions);
            }
            
            evt.data.addAction(data);
            
            return false;
        });

        $("#gtJobsDetailed").on(gt.config.clickEvt, '#gtJobsShowOnMapButton', this, function(evt) {
            var pointGeom = OpenLayers.Geometry.fromWKT($(this).jqmData('geom'));
            
            if (evt.data.lastSelectedFeature) {
                gt.layer.Jobs.drawFeature(evt.data.lastSelectedFeature, 'default');
            }
            
            if (evt.data.lastSelectedAction) {
                gt.layer.Jobs.removeFeatures([evt.data.lastSelectedAction]);
            }
            
            evt.data.lastSelectedFeature = gt.layer.Jobs.getFeatureBy('geometry', pointGeom.toString());
            gt.layer.Jobs.drawFeature(evt.data.lastSelectedFeature, 'select'); 
            
            gt.map.zoomToExtent(pointGeom.getBounds());
            gt.map.zoomTo(evt.data.zoomLevel);
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });
        
        $("#gtJobsDetailed").on(gt.config.clickEvt, '.gtJobsDistance', this, function(evt) {
            var pointGeom = OpenLayers.Geometry.fromWKT($(this).jqmData('geom'));
            var feature = new OpenLayers.Feature.Vector(pointGeom);
            
            
            if (evt.data.lastSelectedFeature) {
                gt.layer.Jobs.drawFeature(evt.data.lastSelectedFeature, 'default');
            }
            
            if (evt.data.lastSelectedAction) {
                gt.layer.Jobs.removeFeatures([evt.data.lastSelectedAction]);
            }
            
            evt.data.lastSelectedAction = feature;
            gt.layer.Jobs.addFeatures([evt.data.lastSelectedAction]); 
            gt.layer.Jobs.drawFeature(evt.data.lastSelectedAction, 'select'); 
            
            gt.map.zoomToExtent(pointGeom.getBounds());
            gt.map.zoomTo(evt.data.zoomLevel);
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });
        
        $("#gtJobsTypeSelect").on(gt.config.clickEvt, 'a', this, function(evt) {
            if ($(this).hasClass('ui-btn-active')) {
                return;
            }
            
            evt.data.lastSelectedSortId = this.id;
            evt.data.lastSelectedSortType = $(this).jqmData('sort');
            
            $.mobile.showPageLoadingMsg();
            if (evt.data.lastSelectedSortType == 4) {
                evt.data.completed = true;
                evt.data.layer.load();
            } else {
                evt.data.completed = false;
                evt.data.completedOffset = 0;
                evt.data.loadMoreButton = true;
                
                evt.data.populate();
                evt.data.layer.add(evt.data.records);
            }
        });
        
        //E-mail
        detailedPageContent.on(gt.config.clickEvt, '#gtJobsEmailButton', this, function (evt) {
            //Set the selected job records
            if (evt.data.completed) {
                var records = evt.data.completedRecords;
            } else {
                var records = evt.data.records; 
            }
            
            //Get current job
            var currentJob = {};
            for (var i = 0; i < records.length; i++) {
                if (records[i].id == evt.data.currentId) {
                    currentJob = records[i];
                }
            }
            
            var subj = encodeURIComponent(currentJob.title);
            var body = encodeURIComponent(currentJob.desc);
            window.location = 'mailto:?subject=' + subj + '&body=' + body;
            
            return false;
        });
        
    },
    
    displayJob: function () {
        var pathName = window.location.pathname.split('/');

        for (var i=0; i < pathName.length; i++) {
            if(pathName[i] == 'job') {
                this.acJobId = parseInt(pathName[i + 1]);
                $('#gtJobsButton').triggerHandler(gt.config.clickEvt);
            }
        }

    },
    
    populate: function() {
        if (! $('#gtJobs').length) {
            return;
        }
        
        var self = this;
        if (this.completed) {
            var records = this.completedRecords.slice(0);
        } else {
            var records = this.records.slice(0); 
        }
        var list = $("#gtJobsList");
        
        //
        for (var i = 0; i < records.length; i++) {
            if (records[i].priority == 'high') {
                records[i].priorityNumber = 1;
            } else if (records[i].priority == 'medium') {
                records[i].priorityNumber = 2;
            } else if (records[i].priority == 'low') {
                records[i].priorityNumber = 3;
            }
        }
        
        //Sort records by selected criteria
        if (this.lastSelectedSortType == 1) {
            //Date
            records.sort(function(a, b) {
                return (a.date > b.date) ? -1 : (a.date < b.date) ? 1 : 0;
            });
        } else if (this.lastSelectedSortType == 2) {
            //Distance
            records.sort(function(a, b) {
                var distanceA = self.calculateDistance(a.the_geom);
                var distanceB = self.calculateDistance(b.the_geom);

                return (distanceA < distanceB) ? -1 : (distanceA > distanceB) ? 1 : 0;
            });
        } else if (this.lastSelectedSortType == 3) {
            //Priority
            records.sort(function(a, b) {
                return (a.priorityNumber < b.priorityNumber) ? -1 : (a.priorityNumber > b.priorityNumber) ? 1 : 
                    (a.date > b.date) ? -1 : (a.date < b.date) ? 1 : 0;
            });
        } else if (this.lastSelectedSortType == 4) {
            //Completed by date
            records.sort(function(a, b) {
                return (a.date > b.date) ? -1 : (a.date < b.date) ? 1 : 0;
            });
        }

        //html
        var html = '<ul data-role="listview" data-inset="true">';
        for (var i = 0; i < records.length; i++) {
            if (! this.completed && (records[i].status == 'declined' || records[i].status == 'completed')) {
                continue;
            }
            
            //Define priority color
            if (records[i].priority == 'high') {
                var priorityClass = 'gtJobsPriorityHigh';
            } else if (records[i].priority == 'medium') {
                var priorityClass = 'gtJobsPriorityMedium';
            } else if (records[i].priority == 'low') {
                var priorityClass = 'gtJobsPriorityLow';
            }
            
            html += '<li class="gtJobsItem" data-id="' + records[i].id + '"><a href="">' +
                '<h2>' + records[i].title + '</h2>' +
                '<p data-distance="' + records[i].the_geom + '">' + 
                    gt.translate._('priority') + ': <span class="' + priorityClass + '">' + records[i].priority + '</span></p>' +
                '<p class="ui-li-aside" data-time="' + records[i].date + '"></p>' +
            '</a></li>';
        }
        html += '</ul>';
        
        if (this.completed && this.loadMoreButton) {
            html += '<button type="button" id="gtJobsLoadMoreButton">' + gt.translate._('loadMore') + '</button>';
        }
        
        list.html(html);
        list.trigger('create');
        
        this.updateTimeAndDistance();

        $.mobile.hidePageLoadingMsg();
        
        //Display detailed job if selected
        if (this.acJobId) {
            this.currentId = this.acJobId;
            
            this.acJobId = false;
            this.populateDetails();
        }
    },

    populateDetails: function() {
        Giscuit.Util.jqmCleanPage('gtJobsDetailed');
        
        //Set the selected job records
        if (this.completed) {
            var records = this.completedRecords;
        } else {
            var records = this.records; 
        }
        
        //Get current job
        var currentJob = {};
        for (var i = 0; i < records.length; i++) {
            if (records[i].id == this.currentId) {
                currentJob = records[i];
            }
        }
        
        //Set available actions
        this.updateAvailableActions(currentJob);
        
        //Set title
        var content = $("#gtJobsDetailed > div[data-role='content']");
        var title = $("#gtJobsDetailed > div[data-role='header'] > h1");
        $('#gtJobsShowOnMapButton').jqmData('geom', currentJob.the_geom);
        title.html(currentJob.title);        
        
        //Parse content
        var waitingTime = Giscuit.Util.prettyDate(currentJob.date * 1000);
        var distance = this.calculateDistance(currentJob.the_geom);
        
        var deadlineDate = new Date(currentJob.deadline * 1000);
        var currentDay = ( deadlineDate.getDate() < 10 ? "0" : "" ) + deadlineDate.getDate();
        var currentMonth = ( (deadlineDate.getMonth() + 1) < 10 ? "0" : "" ) + (deadlineDate.getMonth() + 1);
        var deadline = deadlineDate.getFullYear() + '-' + currentMonth + '-' + currentDay;
            
        distance = this.formatDistance(distance);
        currentJob.availableActions.sort(function(a, b) {
            return a.order > b.order;
        });
        var descendingActions = currentJob.actions.slice(0).reverse();
        
        //html
        var html = '';
        //html += '<button type="button" id="gtJobsEmailButton">' + gt.translate._('emailDetails') + '</button>';
        if (! this.completed) {
            for (var i = 0; i < currentJob.availableActions.length; i++) {
                var action = currentJob.availableActions[i];
                html += '<a href="#gtJobsDetailedComment" data-rel="popup" data-transition="pop" data-role="button" data-theme="' + action.theme + '"' + 
                    ' data-action="' + action.id + '" data-reserved="' + action.reserved + '" data-inline="true" data-position-to="window">' + action.name + '</a>';
            }
            
            //Comment action
            if (currentJob.status != 'declined' && currentJob.status != 'completed') {
                html += '<a href="#gtJobsDetailedComment" data-rel="popup" data-transition="pop" data-role="button" data-theme="c"' + 
                    ' data-action="-1" data-reserved="4" data-inline="true" data-position-to="window">' + gt.translate._('comment') + '</a>';
            }
        }

        //Build details section
        html += '</div><h3 class="gtJobsHeading">' + gt.translate._('details') + ':</h3>' +
        '<div class="gtJobsLeftDiv">' +
            '<ul class="gtJobsList">' +
                '<li>' + gt.translate._('description') + ':</li>' +
                '<li>' + gt.translate._('address') + ':</li>' +
                '<li>' + gt.translate._('contact') + ':</li>' +
                '<li>' + gt.translate._('waiting') + ':</li>' +
                '<li>' + gt.translate._('distance') + ':</li>' +
                '<li>' + gt.translate._('deadline') + ':</li>' +
            '</ul>' +
        '</div>' +
        '<div class="gtJobsRightDiv">' +
            '<ul class="gtJobsList">' +
                '<li>' + currentJob.desc + '</li>' +
                '<li>' + currentJob.contact_address + '</li>' +
                '<li>' + currentJob.contact_name + '</li>' +
                '<li>' + waitingTime + '</li>' +
                '<li>' + distance + '</li>' +
                '<li>' + deadline + '</li>' +
            '</ul>' +
        '</div>';

        //Build history section
        if (descendingActions.length) {
            html += '<h3 class="gtJobsHeading">' + gt.translate._('history') + ':</h3>' +
            '<div class="gtJobsLeftDiv">' +
                '<ul class="gtJobsList">';
            
            //Calculate time and date of actions
            var lastDay = 0;
            for (var i = 0; i < descendingActions.length; i++) {
                //Get date of current action
                var entry = descendingActions[i];
                var entryTime = new Date(entry.date * 1000);
                var currentDay = entryTime.getDate();
                
                //If it's the first action or current action is on a different day from previous action
                if (! lastDay || currentDay != lastDay) {
                    lastDay = currentDay;
                    
                    //Show day in the yyyy-mm-dd format
                    currentDay = ( currentDay < 10 ? "0" : "" ) + currentDay;
                    var currentMonth = ( (entryTime.getMonth() + 1) < 10 ? "0" : "" ) + (entryTime.getMonth() + 1);
                    html += '<li><strong>' + entryTime.getFullYear() + '-' + currentMonth + '-' + currentDay + '</strong></li>';
                    
                    //Add an empty entry to array so the day won't be considered as action when building history
                    descendingActions.splice(i, 0, null);
                    i++;
                }
                
                //Calculate minutes and seconds for the action
                var currentHours = ( entryTime.getHours() < 10 ? "0" : "" ) + entryTime.getHours();
                var currentMinutes = ( entryTime.getMinutes() < 10 ? "0" : "" ) + entryTime.getMinutes();
                html += '<li>' + currentHours + ':' + currentMinutes + '</li>';
            }
            
            html += '</ul></div>' +
            '<div class="gtJobsRightDiv">' +
                '<ul class="gtJobsList">';
            
            //Show action info
            for (var i = 0; i < descendingActions.length; i++) {
                if (descendingActions[i]) {
                    var entry = descendingActions[i];
                    var actionName = this.getActionName(entry.action, currentJob['workflow_id']);
                    var distance = this.formatDistance(this.calculateDistance(currentJob.the_geom, entry.the_geom));
                    html += '<li><i>' + actionName + '</i> ' +
                    '<a href="javascript:void(0);" class="gtJobsDistance" data-geom="' + entry.the_geom + '">(' + distance + ')</a>';
                    if (entry.comment != '') {
                        html += ', ' + entry.comment;
                    }
                    html += '</li>';
                } else {
                    html += '<li></li>';
                }
            }
            
            html += '</ul></div>';
        }
        
        content.html(html);
        content.trigger('create');

        $.mobile.changePage($('#gtJobsDetailed'), {
            'changeHash': true
        });
    },
    
    updateTimeAndDistance: function() {
        //Update time elapsed
        $(".gtJobsItem p[data-time]").each(function() {
            $(this).html(Giscuit.Util.prettyDate(parseFloat($(this).jqmData('time')) * 1000) + ' ' + gt.translate._('ago'));
        });
        
        //Update distance to target
        var self = this;
        $(".gtJobsItem p[data-distance]").each(function() {
            var priority = $(this).html();
            if (typeof(priority.split(',')[1]) != 'undefined') {
                priority = priority.split(',')[1];
            }
            var distance = self.calculateDistance($(this).jqmData('distance'));
            distance = self.formatDistance(distance);
            
            var distanceHtml = gt.translate._('distance') + ': ' + distance + ', ' + priority;

            $(this).html(distanceHtml);
        });
        
        //Update each 15 seconds
        if (this.timeoutId) {
            clearTimeout(this.timeoutId);
        }
        this.timeoutId = setTimeout(Giscuit.Util.bind(this.updateTimeAndDistance, this), 15000);
    },
    
    calculateDistance: function(targetGeom, startGeom) {
        //Same as desktop
        if (typeof(startGeom) == 'undefined') {
            if (gt.control.geolocation.point && gt.control.geolocation.point.geometry) {
                startGeom = gt.control.geolocation.point.geometry;
            } else {
                return NaN;
            }
        } else {
            startGeom = OpenLayers.Geometry.fromWKT(startGeom);
        }
        
        var wgs = new OpenLayers.Projection("EPSG:4326");
        
        var currentPoint = new OpenLayers.LonLat(startGeom.x, startGeom.y);
        currentPoint.transform(gt.map.projection, wgs);
        
        var targetPointGeom = OpenLayers.Geometry.fromWKT(targetGeom);
        var targetPoint = new OpenLayers.LonLat(targetPointGeom.x, targetPointGeom.y);
        targetPoint.transform(gt.map.projection, wgs);
        
        return OpenLayers.Util.distVincenty(currentPoint, targetPoint);
    },
    
    formatDistance: function(distance) {
        //Same as desktop
        if (! distance) {
            return gt.translate._('unknown');
        }
        
        if (distance < 1) {
            formatedDistance = parseInt(distance * 1000) + 'm';
        } else {
            formatedDistance = distance.toFixed(2) + 'km';
        }
        
        return formatedDistance;
    },
    
    addLayer: function() {
        if (! this.layer) {
            this.layer = new Giscuit.Mobile.Layer.Jobs();
            gt.control.layers.addLayer(this.layer, {name: gt.translate._('jobs'), select: true}, 'overlay');
        } else if (! this.layer.visibility) {
            var layerId = gt.control.layers.getLayerIdByClassName('Jobs');
            gt.control.layers.setSettings('view', layerId, true);
        }
    },
    
    addAction: function(data) {
        $("#gtJobsDetailed a[data-action]").removeClass('ui-disabled');

        //Select current job
        var currentJob = {};
        for (var i = 0; i < this.records.length; i++) {
            if (this.records[i].id == data.id) {
                currentJob = this.records[i];
            }
        }

        //Update job record
        currentJob.actions.push({
            action: data.action,
            name: data.name,
            comment: data.comment,
            date: Math.round(new Date().getTime() / 1000),
            the_geom: data.geometry
        });
        
        //Update job status
        if (data.reserved == 2) {
            currentJob.status = 'declined';
        } else if (data.reserved == 3) {
            currentJob.status = 'completed';
        }
        
        //Update records in storage
        Giscuit.Util.setStorageObject('gtJobsRecords', {
            'records': this.records,
            'actions': this.possibleActions
        });
        
        //Refresh job page
        if (data.reserved == 3 || data.reserved == 2) {
            $.mobile.navigate('#gtJobs');
            this.layer.load();
        } else {
            this.populateDetails();
        }
    },
    
    synchronizeSuccess: function(data) {
        this.offlineActions.length = 0;
        Giscuit.Util.setStorageObject('gtJobsActions', this.offlineActions);
        
        if (! this.layer.visibility) {
            var layerId = gt.control.layers.getLayerIdByClassName('Jobs');
            gt.control.layers.setSettings('view', layerId, true);
        } else {
            this.layer.load(); 
        }
        //Giscuit.Util.jqmMessage(gt.translate._('synchronizeSuccess'));
    },
    
    updateAvailableActions: function(job) {
        var availableActions = [];
        var executedActions = [];
        var filteredActions = [];
        var possibleActions = this.possibleActions[job['workflow_id']];
        
        //Populate executed actions array
        for (var i = 0; i < job['actions'].length; i++) {
            if (job['actions'][i]['action'] != -1) {
                executedActions.push(job['actions'][i]['action']);
            }
        }
        
        //Populate available actions array
        for (var i = 0; i < possibleActions.length; i++) {
            var contains = true;
            var parentActions = possibleActions[i]['parent'].split(',');
            
            //Check if all parent actions were executed
            for (var j = 0; j < parentActions.length; j++) {
                if (! Giscuit.Util.inArray(executedActions, parentActions[j])) {
                    contains = false;
                }
            }

            var action = {
                'id': possibleActions[i]['id'],
                'order': possibleActions[i]['order'],
                'theme': possibleActions[i]['theme'],
                'name': possibleActions[i]['name'],
                'reserved': possibleActions[i]['reserved']
            };
            
            //Add action if parents were executed
            if (contains) {
                availableActions.push(action);
            }
            //Or if it's a starting action
            else if (! executedActions.length && (possibleActions[i]['reserved'] == 1 || possibleActions[i]['reserved'] == 2)) {
                availableActions.push(action);
            }
        }
        
        //Filter already executed actions
        filteredActions = $.grep(availableActions,function (item) {
            return ! Giscuit.Util.inArray(executedActions, item.id);
        });
        
        //Add starting actions in case job was reassigned after decline
        if (job['status'] == 'pending' && ! filteredActions.length) {
            for (var i = 0; i < possibleActions.length; i++) {
                if (possibleActions[i]['reserved'] == 1 || possibleActions[i]['reserved'] == 2) {
                    filteredActions.push({
                        'id': possibleActions[i]['id'],
                        'order': possibleActions[i]['order'],
                        'theme': possibleActions[i]['theme'],
                        'name': possibleActions[i]['name'],
                        'reserved': possibleActions[i]['reserved']
                    });
                }
            }
        }
        
        job.availableActions = filteredActions;
    },
    
    getActionName: function(actionId, workflowId) {
        var possibleActions = this.possibleActions[workflowId];
        
        //Comment action
        if (actionId == -1) {
            return gt.translate._('comment');
        }
        
        //Any other action
        for (var i = 0; i < possibleActions.length; i++) {
            if (possibleActions[i]['id'] == actionId) {
                return possibleActions[i]['name'];
            }
        }
        
        return '';
    },

    CLASS_NAME: "Giscuit.Mobile.Control.Jobs"
});

