/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Info.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Control.Offline
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Offline = OpenLayers.Class(Giscuit.Control, {
    
    isOnline: null,
    
    db: null,
    
    maxDownloadTiles: 2000,
    
    restrictedLayers: ['Rastercut'],
    
    
    downloadIds: [],
    
    currDownloadId: null,
    
    
    overwriteIds: {},
    
    skipIds: {},
    
    
    tiles: [],
    
    files: [],
    
    
    synchronizeIds: [],
    
    currSynchronizeId: null,
    
    updatedFeatures: [],
    
    updatedFiles: [],
    
    stopSynchronize: false,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        //
        if(typeof (window.openDatabase) !== "function" ) {
            this.isOnline = true;
            return;
        }
          
        //
        this.getStatus();
        
        //
        $(document).ready(Giscuit.Util.bind(function() {
            if(gt.config.guest == 't') {
                return;
            }
            
            this.addOfflineButton();
            
            //Activate
            $('#gtMapControls').on(gt.config.downClickEvt, '#gtOfflineButton', this, function(evt) {
                //
                evt.stopImmediatePropagation();
                
                //
                var content = $("#gtOffline > div[data-role='content']");
                
                //Hide map controls
                gt.init.layout.hideMapControls();
                
                //Render
                var page = '#gtOffline';
                var content = $(page + " > div[data-role='content']");
                if(! content.children().length) {
                    $.mobile.showPageLoadingMsg();
                    
                    setTimeout(Giscuit.Util.bind(function() {
                        evt.data.render();
                        $.mobile.changePage($(page));
                        
                        $.mobile.hidePageLoadingMsg();
                    }, evt.data), 5);
                } else {
                    evt.data.updateMaxDownloadScaleLabel();
                    $.mobile.changePage($(page));
                }
                
                return false;
            });
            
            
        }, this));
    },
    
    render: function() {
        var records = gt.control.layers.records;
        
        var notInOrder = true;
        var rootNode = $('<ul data-role="listview" data-split-icon="delete" id="gtOfflineMenu">');
        
        //Content
        while(notInOrder) {
            notInOrder = false;
        
            for(var i = 0; i < records.length; i++) {
                var liClass = '', 
                    nameClass = '',
                    settingsClass = '',
                    parentNode = null;
                    
                var rec = records[i];
                
                //Skip javascript layers
                if (rec.js_layer) {
                    continue;
                }
                
                //Skip restricted layers
                if (Giscuit.Util.inArray(this.restrictedLayers, rec['class'])) {
                    continue;
                }
                
                //Skip already added records
                if(rootNode.find('#gtOfflineMenu-' + rec.id).length) {
                    continue;
                }
                
                //Root
                if(! rec.parent_id) {
                    parentNode = rootNode;
                    
                //Non root
                } else {
                    parentNode = rootNode.find('#gtOfflineMenu-' + rec.parent_id);
                    
                    //Skip records if their parent was not added yet
                    if(! parentNode.length) {
                        notInOrder = true;
                        continue;
                    }
                }
                                                                
                //Container
                if(! rec['class']) {
                    liClass += 'gtOfflineMenuContainer ';
                    
                    parentNode.append('<li class="' + liClass + '">' +
                        '<div>' + rec.name + '</div>' + 
                        '<ul id="gtOfflineMenu-' + rec.id + '" data-split-icon="delete"></ul></li>');
                
                //Layer
                } else {
                    nameClass += 'gtOfflineMenuName ';
                    settingsClass += 'gtOfflineMenuDelete ';
                    
                    parentNode.append('<li class="' + liClass + '" id="gtOfflineMenu-' + rec.id + '" data-layer-id="' + rec.id + '">' +
                        '<a class="' + nameClass + '">' +
                        rec.name + 
                        '<span class="ui-li-count">0</span>' +
                        '<a class="' + settingsClass + '"></a>' +
                        '</a></li>');
                }
            }
        }
        
        //Page
        var html = '<div data-role="page" id="gtOffline" data-dom-cache="true" data-url="gtOffline">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('offline') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') +
                '</a>' + 
            '</div>' +
     
            '<div data-role="content" >' +
                '<div class="offline">' +
                    '<button type="button" id="gtOfflineGoOnlineButton" data-theme="e">' + gt.translate._('goOnline') + '</button><br>' +
                    '<ul id="gtOfflineEditErrors" class="ui-listview" data-role="listview"></ul>' +
                '</div>' +
            
                '<div class="online">' +
                    '<button type="button" id="gtOfflineGoOfflineButton" data-theme="e">' + gt.translate._('goOffline') + '</button><br>' +
                
                    '<div id="gtOfflineMenuPH"></div><br>' +
                    
                    '<div data-role="popup" id="gtOfflineProgressDialog">' + 
                        '<div data-role="header"><h1 role="heading">' + gt.translate._('downloading') + '</h1></div>' + 
                        '<div data-role="content">' +
                            '<label id="gtOfflineDownloadProgressLabel" for="gtOfflineDownloadProgress"></label>' +
                            '<input type="range" name="gtOfflineDownloadProgress" id="gtOfflineDownloadProgress" value="0" min="0" max="100" data-highlight="true" disabled/>' +
                            '<button type="button" id="gtOfflineDownloadCancel">' + gt.translate._('cancel') + '</button>' +
                        '</div>' +
                    '</div>' +

                    '<p>' +            
                        '<label id="gtOfflineMaxDownloadScaleLabel">' + gt.translate._('maximumDownloadScale') + ':</label>' +
                        '<select id="gtOfflineMaxDownloadScale"></select>' +
                    '</p>' +
                
                    '<p><button type="button" id="gtOfflineDownloadButton">' + gt.translate._('download') + '</button></p>' +
                '</div>' +
            '</div>' +        
        '</div>' +

        '<div data-role="page" id="gtOfflineConflict" data-dom-cache="true" data-url="gtOfflineConflict">' +
              '<div data-role="content">' +
                   '<h3>' + gt.translate._('resolveConflict') + '</h3>' +
                   '<a id="gtOfflineConflictShowButton" data-role="button">' + gt.translate._('showResultsOnMap') + '</a>' +
                   '<a id="gtOfflineConflictOverwriteButton" data-role="button">' + gt.translate._('commitMyVersion') + '</a>' +     
                   '<a id="gtOfflineConflictSkipButton" data-role="button">' + gt.translate._('discardMyVersion') + '</a>' +       
                   '<a id="gtOfflineConflictCancelButton" data-role="button">' + gt.translate._('leaveUnresolved') + '</a>' +
               '</div>' +                
        '</div>';
        
        //Append HTML
        $.mobile.pageContainer.append(html);
        $('#gtOfflineMenuPH').append(rootNode);
        $('#gtOffline').page();
        
        //Add slider percentage div
        var sliderEl = $('#gtOfflineDownloadProgress');
        sliderEl.next().append('<div id="gtOfflineDownloadPercent">0%</div>');
        
        //
        this.changeOfflineLayout(this.isOnline);
        
        //Maximum download scale
        var select = $('#gtOfflineMaxDownloadScale');
        var resolutions = gt.map.resolutions;
        var units = gt.map.getUnits();

        select.html();
        for(var i = 0; i < resolutions.length; i++) {
            var scale = Math.round(OpenLayers.Util.getScaleFromResolution(resolutions[i], units));
            select.append('<option value="' + i + '">1:' + scale + '</option>');
        }
        select.val(gt.map.zoom);
        select.selectmenu('refresh');
        
        //
        this.updateMaxDownloadScaleLabel();
        
        //EVENTS:
        
        //Download
        $('#gtOfflineDownloadButton').on(gt.config.clickEvt, this, function(evt) {
            evt.data.downloadInitialize();
            
            return false;
        });
        
        //Offline
        $('#gtOfflineGoOfflineButton').on(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
            
            evt.data.setStatus(false);
            
            return false;
        });
        
        //Online
        $('#gtOfflineGoOnlineButton').on(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
            
            evt.data.setStatus(true);
            
            return false;
        });
        
        //Count layer features
        $('#gtOffline').on('pagebeforeshow', this, function(evt) {
            if(evt.data.isOnline) {
                evt.data.updateFeaturesCount();
            } else {
                evt.data.setEditErrors();
            }
        });
        
        //Toggle layer
        $(document).on(gt.config.clickEvt, '.gtOfflineMenuName', this, function(evt) {
            var li = $(this.parentNode.parentNode.parentNode);
            var deleteEl = li.find('.gtOfflineMenuDelete');
            
            if(li.hasClass('ui-btn-active')) {
                li.removeClass('ui-btn-active');
                deleteEl.removeClass('ui-btn-active');
            } else {
                li.addClass('ui-btn-active');
                deleteEl.addClass('ui-btn-active');
            }
            
            return false;
        });
        
        //Delete
        $(document).on(gt.config.clickEvt, '.gtOfflineMenuDelete', this, function(evt) {
            var li =  $(this.parentNode);
            
            var layerId = li.jqmData('layer-id');
            
            if(confirm(gt.translate._('confirmDeleteDbTable'))) {
                evt.data.deleteLayer(layerId);
            }
            
            return false;
        });
        /*
        //Cancel download
        $("#gtOfflineDownloadCancel").on(gt.config.downClickEvt, this, function(evt) {
            evt.stopImmediatePropagation();
            
            evt.data.downloadIds = [];
            evt.data.tiles = [];
            evt.data.files = [];
            $("#gtOfflineProgressDialog").popup("close");
            
            return false;
        });
        */
        //FIXME: Hack to avoid closing popup when clicking outside
        $("#gtOfflineProgressDialog").on({
            popupbeforeposition: function () {
                $('#gtOfflineProgressDialog .ui-popup-screen').off();
            }
        });
        
    },
    
    updateMaxDownloadScaleLabel: function() {
        var resolutions = gt.map.resolutions;
        var units = gt.map.getUnits();
        
        //Maximum download scale label
        $('#gtOfflineMaxDownloadScaleLabel').html(
            gt.translate._('maximumDownloadScale') + 
            ' (' + 
            gt.translate._('currentScale') + 
            ' - 1:' + Math.round(OpenLayers.Util.getScaleFromResolution(resolutions[gt.map.zoom], units)) + 
            ')'
        );
    } ,
    
    addOfflineButton: function() {
        $("#gtMapControls").append('<div>'
            + '<div class="ui-bar ui-bar-c" id="gtOfflineButton">'
            + '<div class="gtMapControlsElImage" style="background-position: -576px 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('offline') + '</div>'
            + '</div></div>');
    } ,
    
    changeOfflineLayout: function(status) {
        if(status) {
            $('#gtOffline .online').css('display', 'block');
            $('#gtOffline .offline').css('display', 'none');
        } else {
            $('#gtOffline .online').css('display', 'none');
            $('#gtOffline .offline').css('display', 'block');
        }
    } ,
           
    getStatus: function () {
        if (navigator.onLine) {
            if(this.isOnline === null) {
                this.setStatus(true);
            }
        } else {
            this.setStatus(false);
        }
    } ,
    
    setStatus: function(status) {
        var lastIsOnline = this.isOnline;
        
        if(status !== lastIsOnline) {
            //Status change
            if(lastIsOnline !== null) {
                if(status) {
                    this.synchronize();
                } else {
                    this.isOnline = false;
                    this.wentOffline();
                }
            //Initial status
            } else {
                if(status) {
                    //Clear layer changes
                    Giscuit.Util.removeStorageObject('gtLayersChanges');
                } else {

                }
                
                this.isOnline = status;
            }
        }
    } ,
    
    wentOnline: function() {
        debug.info('Online');
        
        //
        gt.map.deactivateAll();
        //gt.control.loading.setVisible(true);
                
        //Layers
        gt.map.removeAll();
        gt.control.layers.add();
        gt.control.layers.addLazyLayers();
        
        //Page
        Giscuit.Util.jqmCleanPage('gtLayers');
        $('#gtLayers').off().remove();
        $('#gtLayersOrder').off().remove();
        
        //Buttons
        this.changeOfflineLayout(true);
        gt.control.home.addHomeButton();
        
        //
        this.events.triggerEvent('online');
        gt.init.layout.renderMapControls();
        
        //
        $('#gtOfflineGoOnlineButton').button('enable');
        
        //
        $.mobile.changePage($('#gtMap'));
    } ,
    
    wentOffline: function() {
        debug.info('Offline');
        
        //
        gt.map.deactivateAll();
        //gt.control.loading.setVisible(false);
        
        //Layers
        gt.map.removeAll();
        gt.control.layers.add();

        //Page
        Giscuit.Util.jqmCleanPage('gtLayers');
        $('#gtLayers').off().remove();
        $('#gtLayersOrder').off().remove();
        
        //Buttons
        gt.control.home.deleteHomeButton();
        this.changeOfflineLayout(false);
                
        //
        $('#gtOfflineGoOfflineButton').button('enable');
        
        $.mobile.changePage($('#gtMap'));
    } ,
    
    synchronize: function() {
        //Cancel synchronize event
        $("#gtOfflineDownloadCancel").off(gt.config.downClickEvt).on(gt.config.downClickEvt, this, function(evt) {
            evt.stopImmediatePropagation();
            
            evt.data.stopSynchronize = true;
            $("#gtOfflineProgressDialog").popup("close");
            $('#gtOfflineGoOnlineButton').button('enable');
            
            return false;
        });
        
        //Set download progress popup width and open it
        $("#gtOfflineProgressDialog").width($("#gtOfflineGoOnlineButton").parent().width());
        $("#gtOfflineProgressDialog h1").html(gt.translate._('synchronizing'));
        $("#gtOfflineProgressDialog").popup("open");
        
        this.synchronizeLayers();
    } ,
    
    synchronizeLayers: function() {
        var records = gt.control.layers.records;
        var items = ['id', 'view', 'select', 'label', 'edit', 'label_size', 'label_color', 'label_fields', 'order'];
        var boolItems = ['view', 'select', 'label', 'edit'];
        var result = [];
        
        //Progress bar
        var sliderEl = $('#gtOfflineDownloadProgress');
        //Value
        sliderEl.attr('value', 0);
        sliderEl.attr('max', 1);
        sliderEl.slider('refresh');
        //Percentage
        $("#gtOfflineDownloadPercent").html('0%');
        //Label
        $("#gtOfflineDownloadProgressLabel").html(gt.translate._('userData'));
        
        for(var i=0; i < records.length; i++) {
            if(typeof(records[i].changed) != 'undefined' && records[i].changed === true) {
                var row = {}; 
                for(var obj in records[i]) {
                    if(Giscuit.Util.inArray(boolItems, obj)) {
                        row[obj] = (records[i][obj] === true) ? 't' : 'f' ;
                    } else if(Giscuit.Util.inArray(items, obj)) {
                        row[obj] = records[i][obj];
                    }
                }
                result.push(row);
            }
        }

        var jsonFormat = new OpenLayers.Format.JSON();
        var url =  gt.config.httpMobilePath + '/privatelayers/settingsbulk';
        
        $.ajax({
            'cache': false ,
            'timeout': 5000 ,
            'type': 'POST',
            'url': url,
            'data': {data: jsonFormat.write(result)},
            'success': this.synchronizeLayersSuccess ,
            'error': function(evt) {
                if(evt.status == 401) {
                    Giscuit.Util.jqmMessage(gt.translate._('accessForbidden'));
                    window.location = gt.config.httpMobilePath + '/menu';
                    
                    return;
                }
                
                Giscuit.Util.jqmMessage(gt.translate._('noInternetConnection'));
                $('#gtOfflineGoOnlineButton').button('enable');
            } ,
            'dataType': 'json' ,
            'context': this
        });
    } ,
    
    synchronizeLayersSuccess: function(data) {
        //FIXME:
        gt.control.editgeom.deactivate();
        
        //
        //this.files = [];
        //this.synchronizeIds = [];
        var layerType = Giscuit.Util.getStorageObject('gtLayerType');
        var self = this;
        
        var editErrors = Giscuit.Util.getStorageObject('gtEditErrors');
        editErrors = {};
        Giscuit.Util.setStorageObject('gtEditErrors', editErrors);
        
        //Progress bar
        var sliderEl = $('#gtOfflineDownloadProgress');
        //Value
        sliderEl.attr('value', 1);
        sliderEl.slider('refresh');
        //Percentage
        $("#gtOfflineDownloadPercent").html('100%');
        
        if (this.files.length) {
            sliderEl.attr('value', 0);
            sliderEl.attr('max', this.files.length + 1);
            sliderEl.slider('refresh');
            //Percentage
            $("#gtOfflineDownloadPercent").html('0%');
            
            this.synchronizeFiles();
            
            return;
        }
        
        Giscuit.Util.Db.getMetadata(function (metadata) {
            for(var obj in metadata) {
                if(layerType[metadata[obj].name] != 'vector') {
                    continue;
                }
                
                self.synchronizeIds.push(obj);
            }
            
            if (self.synchronizeIds.length) {
                self.synchronizeEditing();
            } else {
                self.synchronizeEditingComplete();
            }
        });
    },
    
    synchronizeEditing: function() {
        //Cancel synchronization
        if (this.stopSynchronize) {
            this.stopSynchronize = false;
            
            return;
        }
        
        var self = this;
        this.currSynchronizeId = this.synchronizeIds.pop();
                
        var protocol = new Giscuit.Util.Db.Vector({tableName: 'layer_' + this.currSynchronizeId});
        protocol.read({
            withHStatus: true,
            callback: function(result) {

                var layerId = self.currSynchronizeId;
                var modifiedFeatures = [];
                var layerRecord = gt.control.layers.getLayerRecordById(layerId);
                
                //Progress bar
                var sliderEl = $('#gtOfflineDownloadProgress');
                //Value
                sliderEl.attr('value', 0);
                sliderEl.slider('refresh');
                //Percentage
                $("#gtOfflineDownloadPercent").html('0%');
                //Label
                $("#gtOfflineDownloadProgressLabel").html(layerRecord.name);
                        
                var len = result.features.length, i;
                for (i = 0; i < len; i++) {
                     var row = result.features[i].attributes;
                     row.the_geom = result.features[i].geometry.toString();
                     row.__fid = result.features[i].fid;
                     modifiedFeatures.push(row);
                }
                        
                if(! modifiedFeatures.length) {
                    self.synchronizeEditingComplete();
                    
                    return;
                }
                        
                var overwriteIdsValue = '/overwrite';
                if(typeof(self.overwriteIds[layerId]) != 'undefined') {
                    overwriteIdsValue += '/' + self.overwriteIds[layerId].join(','); 
                }
                        
                var skipIdsValue = '/skip';
                if(typeof(self.skipIds[layerId]) != 'undefined') {
                    skipIdsValue += '/' + self.skipIds[layerId].join(','); 
                }
                        
                //FIXME: Unused code
                //editErrors[layerId] = false;
                                        
                var jsonFormat = new OpenLayers.Format.JSON();
                var url =  gt.config.httpMobilePath + 
                    '/editgeom/bulk/id/' + layerId + overwriteIdsValue + skipIdsValue;
                
                $.ajax({
                    'cache': false ,
                    'type': 'POST',
                    'url': url,
                    'data': {data: jsonFormat.write(modifiedFeatures)},
                    'success': self.synchronizeEditingSuccess ,
                    'error': Giscuit.Util.jqmAjaxError ,
                    'dataType': 'json' ,
                    'context': self
                });
            }
        });
    } ,
        
    synchronizeEditingSuccess: function(data) {
        var editErrors = Giscuit.Util.getStorageObject('gtEditErrors');
        var format = new OpenLayers.Format.GeoJSON();
        var editForm = Giscuit.Util.getStorageObject('gtEditForm');
        var fileNames = gt.control.offline.getFileElementsNames(editForm['layer_' + data.layerId]);
        this.updatedFeatures = [];
        
        if(data.errors) {
            editErrors[data.layerId] = data.errors;
        } else {
            delete editErrors[data.layerId];
        }
        
        Giscuit.Util.setStorageObject('gtEditErrors', editErrors);
        
        var filesTable = new Giscuit.Util.Db.Files({tableName: 'files_' + data.layerId});
        var protocol = new Giscuit.Util.Db.Vector({tableName: 'layer_' + data.layerId});
        protocol.read({
            withHStatus: true,
            callback: Giscuit.Util.bind(function(result) {
                //
                var len = result.features.length, a;
                this.updatedFeatures = result.features;
                for (a = 0; a < len; a++) {
                   var feature = result.features[a];
                   
                   if(typeof(editErrors[data.layerId]) == 'undefined' ||
                       typeof(editErrors[data.layerId][feature.fid]) == 'undefined') {
                       
                       if(feature.attributes.hstatus == 'D') {
                           protocol['delete'](feature);
                       } else if (feature.attributes.hstatus == 'C' ||
                           //FIXME: Hack for updating gid when feature was locally updated and deleted on server
                           data.createRecords[feature.fid]) {
                           
                           feature.attributes.gid = data.createRecords[feature.fid];
                           feature.attributes.__hdate = data.timestamp;
                           delete feature.attributes.hstatus;
                           
                           protocol.update(feature);
                       } else if (feature.attributes.hstatus == 'U') {
                           feature.attributes.__hdate = data.timestamp;
                           delete feature.attributes.hstatus;
                           
                           protocol.update(feature);
                       }

                       if (fileNames.length) {
                           if (typeof(this.skipIds[data.layerId]) == 'undefined' ||
                               (typeof(this.skipIds[data.layerId]) != 'undefined' &&
                               ! Giscuit.Util.inArray(this.skipIds[data.layerId], feature.fid))) {

                               for (var i = 0; i < fileNames.length; i++) {
                                   if (feature.attributes[fileNames[i]] < 0 ||
                                       (typeof(this.overwriteIds[data.layerId]) != 'undefined' &&
                                       Giscuit.Util.inArray(this.overwriteIds[data.layerId], feature.fid) && 
                                       feature.attributes[fileNames[i]]) ) {
                                       
                                       var fileData = {
                                           id: feature.attributes[fileNames[i]],
                                           field: fileNames[i],
                                           gid: feature.attributes.gid
                                       };
                                       this.files.push(fileData);
                                   }
                               }
                           }
                       }
                       
                   }
                   
                   //Update features
                   for(var b=0; b < data.updateRecords.length; b++) {
                       if(data.updateRecords[b].properties.gid == feature.attributes.gid) {
                           var newFeature = format.read(data.updateRecords[b], "Feature");
                           newFeature.attributes.__hdate = data.timestamp;
                           newFeature.fid = feature.fid;
                           protocol.update(newFeature);
                           
                           for (var i = 0; i < fileNames.length; i++) {
                               //Remove old feature files
                               if (feature.attributes[fileNames[i]]) {
                                   filesTable['delete'](feature.attributes[fileNames[i]]);
                               }
                               //Set files for redownload
                               if (newFeature.attributes[fileNames[i]]) {
                                   this.updatedFiles.push(newFeature.attributes[fileNames[i]]);
                               }
                           }
                           
                           break;
                       }
                   }
                   
                   //Delete features
                   for(var b=0; b < data.deleteRecords.length; b++) {
                       if(data.deleteRecords[b] == feature.attributes.gid) {
                           //Delete file
                           for (var i = 0; i < fileNames.length; i++) {
                               if (feature.attributes[fileNames[i]]) {
                                   filesTable['delete'](feature.attributes[fileNames[i]]);
                               }
                           }
                           
                           //Delete feature
                           protocol['delete'](feature);
                           
                           break;
                       }
                   }
                }
                
                //Progress bar
                var sliderEl = $('#gtOfflineDownloadProgress');
                //Max value
                sliderEl.attr('max', this.files.length + this.updatedFiles.length + 1);

                //
                if(typeof(this.skipIds[data.layerId]) != 'undefined') {    
                    delete this.skipIds[data.layerId];
                }
                if(typeof(this.overwriteIds[data.layerId]) != 'undefined') {
                    delete this.overwriteIds[data.layerId];
                }
                
                this.synchronizeFiles();
            }, this)
        });

    },
    
    synchronizeFiles: function() {
        //Cancel synchronization
        if (this.stopSynchronize) {
            this.stopSynchronize = false;
            
            return;
        }
        
        var self = this;
        var layerRecord = gt.control.layers.getLayerRecordById(this.currSynchronizeId);
        
        //Progress bar
        var sliderEl = $('#gtOfflineDownloadProgress');
        //Value
        sliderEl.attr('value', parseInt(sliderEl.val()) + 1);
        sliderEl.slider('refresh');
        //Percentage
        $("#gtOfflineDownloadPercent").html(Giscuit.Util.getSliderPercentage(sliderEl) + '%');
        //Label
        $("#gtOfflineDownloadProgressLabel").html(layerRecord.name + ' (' + sliderEl.val() + '/' + sliderEl.attr('max') + ')');
        
        if (this.files.length) {
            var fileData = this.files[0];
        } else if (this.updatedFiles.length) {
            this.synchronizeConflictedFiles();
            
            return;
        } else {
            this.synchronizeEditingComplete();
            
            return;
        }
        
        var filesTable = new Giscuit.Util.Db.Files({tableName: 'files_' + this.currSynchronizeId});
        filesTable.read({
            id: fileData.id,
            callback: function(result) {
                
                var data = '/layerId/' + self.currSynchronizeId +
                    '/field/' + fileData.field + 
                    '/featureId/' + fileData.gid +
                    '/?filename=' + result.files[0].name;
                
                var url =  gt.config.httpMobilePath + '/editgeom/uploadfile' + data;
                
                $.ajax(url, {
                    'type': 'POST',
                    'data': result.files[0].data,
                    'processData': false,
                    'contentType': 'text/plain',
                    'success': self.synchronizeFilesSuccess ,
                    'error': Giscuit.Util.jqmAjaxError ,
                    'context': self
                });
                
            }
        });
    },
    
    synchronizeFilesSuccess: function(data) {
        //Update local values
        var filesTable = new Giscuit.Util.Db.Files({tableName: 'files_' + this.currSynchronizeId});
        var protocol = new Giscuit.Util.Db.Vector({tableName: 'layer_' + this.currSynchronizeId});
        
        for (var i = 0; i < this.updatedFeatures.length; i++) {
            if (parseInt(this.updatedFeatures[i].attributes.gid) == data.fileData.featureId) {
                filesTable.update({
                    id: data.fileData.fileId
                }, {
                    id: this.updatedFeatures[i].attributes[data.fileData.fileField]
                });
                
                this.updatedFeatures[i].attributes[data.fileData.fileField] = data.fileData.fileId;
                protocol.update(this.updatedFeatures[i]);
            }
        }
        
        this.files.shift();
        
        this.synchronizeFiles();
    },
    
    synchronizeEditingComplete: function(data) {
        
        if (this.synchronizeIds.length) {
            this.synchronizeEditing();
            
            return;
        }
        
        var editErrors = Giscuit.Util.getStorageObject('gtEditErrors');
        var format = new OpenLayers.Format.GeoJSON();

        //
        this.isOnline = true;
        this.wentOnline();
        
        //Finished
        for (var error in editErrors) {
            var errorsExist = true;
        }
        if (typeof errorsExist == 'undefined') {
            Giscuit.Util.jqmMessage(gt.translate._('synchronizeSuccess'));
        } else {
            Giscuit.Util.jqmMessage(gt.translate._('synchronizeFailed'));
        }

        gt.map.refresh();
    },
    
    downloadInitialize: function() {
        
        $('#gtOfflineDownloadButton').button('disable');
        $('.gtOfflineMenuDelete').each(function(index, value) {
            $(this).addClass('ui-disabled');
        });
        
        //Cancel download event
        $("#gtOfflineDownloadCancel").off(gt.config.downClickEvt).on(gt.config.downClickEvt, this, function(evt) {
            evt.stopImmediatePropagation();
            
            evt.data.downloadIds = [];
            evt.data.tiles = [];
            evt.data.files = [];
            $("#gtOfflineProgressDialog").popup("close");
            
            return false;
        });
        
        this.downloadIds = [];
        
        //Show only layers available for offline
        $('.gtOfflineMenuName').each(Giscuit.Util.bind(function(index, value) {
            var li = $(value.parentNode.parentNode.parentNode);
            
            if(li.hasClass('ui-btn-active')) {
                this.downloadIds.push(li.jqmData('layer-id'));
            }
            
        }, this));

        //Set download progress popup width and open it
        $("#gtOfflineProgressDialog").width($("#gtOfflineDownloadButton").parent().width());
        $("#gtOfflineProgressDialog h1").html(gt.translate._('downloading'));
        $("#gtOfflineProgressDialog").popup("open");
        
        this.downloadGet();
    } ,
    
    downloadGet: function() {
        //
        if (! this.downloadIds.length && ! this.tiles.length) {
            this.downloadComplete();
            return;
        }
        
        //
        var layer = gt.control.layers.getLayerRecordById(id);
        
        //
        var id = this.downloadIds.shift();
        var where = gt.map.getExtent().toBBOX();
        var url =  gt.config.httpMobilePath + 
            '/offline/download/id/' + id + '/where/' + where;

        this.currDownloadId = id;
        
        //Progress bar
        var sliderEl = $('#gtOfflineDownloadProgress');
        //Value
        sliderEl.attr('value', 0);
        sliderEl.slider('refresh');
        //Percentage
        $("#gtOfflineDownloadPercent").html('0%');
        //Label
        $("#gtOfflineDownloadProgressLabel").html(layer.name);
        
        $.ajax({
            'type': 'GET' ,
            'url': url ,
            'success': this.downloadSet ,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this 
        });
    } ,
    
    
    
    downloadSet: function(data) {
        var sliderEl = $('#gtOfflineDownloadProgress');
        var self = this;
        var record = gt.control.layers.getLayerRecordById(this.currDownloadId);
        
        //Delete previous synchronize errors
        var editErrors = Giscuit.Util.getStorageObject('gtEditErrors');
        delete editErrors[this.currDownloadId];
        Giscuit.Util.setStorageObject('gtEditErrors', editErrors);
        
        $('.gtOfflineMenuName').parent().parent().parent().removeClass('ui-btn-active');
        $('.gtOfflineMenuName').parent().parent().next().removeClass('ui-btn-active');
        
        if(data.tilecache) {
            this.tiles = [];

            var maxZoomLevel = parseInt($('#gtOfflineMaxDownloadScale').val()) + 1;
            if(maxZoomLevel <= gt.map.zoom) {
                maxZoomLevel = gt.map.zoom + 1;
            }            
            
            for(var zoom = gt.map.zoom; zoom < maxZoomLevel; zoom++) {
                var tiles = this.getTileURLs(gt.map.getExtent(), zoom);
                this.tiles = $.merge(this.tiles, tiles); 
            }
                                    
            if(tiles.length > this.maxDownloadTiles) {
                var error = record.name + ' ' + gt.translate._('wasNotDownloaded') + ', ' + gt.translate._('tooManyFeaturesRequested') + '.';
                //FIXME:
                alert(error);
                
                this.currDownloadId = null;
                this.tiles = [];
                this.downloadGet();
                return;
            }
            
            //
            var protocol = new Giscuit.Util.Db.Raster({tableName: 'layer_' + record.id});
            protocol.clear({
                callback: function() {
                    self.downloadTiles(record, data.tilecache.tmpname);
                }
            });

            //Layer type
            var layerType = Giscuit.Util.getStorageObject('gtLayerType');
            layerType['layer_' + record.id] = 'raster';
            Giscuit.Util.setStorageObject('gtLayerType', layerType);
            
            //Value
            sliderEl.attr('max', tiles.length);
            sliderEl.slider('refresh');
            //Percentage
            //Label
            $("#gtOfflineDownloadProgressLabel").html(record.name + ' (0/' + tiles.length + ')');
            
        } else {
            if(data.error) {
                var error = record.name + ' ' + gt.translate._('wasNotDownloaded') + ', ' + gt.translate._(data.error) + '.';
                //FIXME:
                alert(error);
                
                this.currDownloadId = null;
                this.downloadGet();
                return;
            }
            
            //Records
            var protocol = new Giscuit.Util.Db.Vector({tableName: 'layer_' + record.id});
            protocol.clear();
            protocol.initialCreate(data.records, {
                timestamp: data.timestamp
            });
            
            //Style
            var mapSLD = Giscuit.Util.getStorageObject('gtMapSLD');
            mapSLD['layer_' + record.id] = data.sld;
            Giscuit.Util.setStorageObject('gtMapSLD', mapSLD);
            
            //Form
            if(data.form) {
                var editForm = Giscuit.Util.getStorageObject('gtEditForm');
                editForm['layer_' + record.id] = data.form;
                Giscuit.Util.setStorageObject('gtEditForm', editForm);
                
                var fileNames = this.getFileElementsNames(data.form);
                
                if (fileNames.length) {
                    for (var i = 0; i < data.records.length; i++) {
                        for (var j = 0; j < fileNames.length; j++) {
                            if (data.records[i].properties[fileNames[j]]) {
                                this.files.push(data.records[i].properties[fileNames[j]]);
                            }
                        }
                    }
                }
            }
            
            //Geometry type
            var mapGeometryType = Giscuit.Util.getStorageObject('gtMapGeometryType');
            mapGeometryType['layer_' + record.id] = data.geometryType;
            Giscuit.Util.setStorageObject('gtMapGeometryType', mapGeometryType);
                    
            //Legend
            var layersLegend = Giscuit.Util.getStorageObject('gtLayersLegend');
            layersLegend['layer_' + record.id] = data.legend;
            Giscuit.Util.setStorageObject('gtLayersLegend', layersLegend);
            
            //Layer type
            var layerType = Giscuit.Util.getStorageObject('gtLayerType');
            layerType['layer_' + record.id] = 'vector';
            Giscuit.Util.setStorageObject('gtLayerType', layerType);
            
            //Update downloading layer label and progress
            $("#gtOfflineDownloadProgressLabel").html(record.name);
            
            //Value
            sliderEl.attr('value', parseInt(sliderEl.val()) + 1);
            sliderEl.attr('max', this.files.length + 1);
            sliderEl.slider('refresh');
            //Percentage
            $("#gtOfflineDownloadPercent").html(Giscuit.Util.getSliderPercentage(sliderEl) + '%');
            //Label
            $("#gtOfflineDownloadProgressLabel").html(record.name + ' (1/' + sliderEl.attr('max') + ')');
            
            //Download attached files or continue to next layer
            if (this.files.length) {
                //
                var protocol = new Giscuit.Util.Db.Files({tableName: 'files_' + record.id});
                protocol.clear({
                    callback: function() {
                        self.downloadFiles();
                    }
                });
            } else {
                this.currDownloadId = null;
                this.downloadGet();
            }
        }
    } ,
    
    downloadTiles: function(record, tmpname) {
        var self = this;
        var url = this.tiles.pop();
        
        if(! url) {
            this.downloadTilesComplete(tmpname);
            
        } else {
            //
            if (gt.config.staticserverUrls instanceof Array) {
                var httpTilesPath = gt.config.staticserverUrls[Math.floor(Math.random() * gt.config.staticserverUrls.length)];
            } else {
                httpTilesPath = gt.config.staticserverUrls;
            }
            var req = httpTilesPath + '/' + tmpname + '/' + url;
            
            //
            $.ajax({
                timeout: 30000 ,
                type: 'GET',
                url: req,
                success: this.downloadTilesSuccess ,
                error: this.downloadTilesSuccess ,
                context: {
                    self: this,
                    record: record,
                    tmpname: tmpname,
                    url: url
                },
                mimeType: 'text/plain; charset=x-user-defined'
            });
        }
    },
    
    downloadTilesSuccess: function(data, status, xhr) {
        xhr.responseText = '';
        
        var self = this.self;
        var record = this.record;
        var tmpname = this.tmpname;
        var url = this.url;
        
        var protocol = new  Giscuit.Util.Db.Raster({tableName: 'layer_' + record.id});
        var dataType = xhr.getResponseHeader('content-type');
                
        //
        var binary = '';
        for (var i = 0; i < data.length; i+=1) {
            binary += String.fromCharCode(data.charCodeAt(i) & 0xff);
        }
        data = '';
        //TODO: btoa not supportted in IE
        var text = 'data:' + dataType + ';base64,' + btoa(binary);
        binary = '';

        //
        protocol.create(url, text);
        
        //Progress bar
        var sliderEl = $('#gtOfflineDownloadProgress');
        //Value
        sliderEl.attr('value', parseInt(sliderEl.val()) + 1);
        sliderEl.slider('refresh');
        //Percentage
        $("#gtOfflineDownloadPercent").html(Giscuit.Util.getSliderPercentage(sliderEl) + '%');
        //Label
        $("#gtOfflineDownloadProgressLabel").html(record.name + ' (' + sliderEl.val() + '/' + sliderEl.attr('max') + ')');
        
        //
        self.downloadTiles(record, tmpname); 
    },
    
    downloadTilesComplete: function(tmpname) {
        var url = gt.config.httpMobilePath + 
            '/offline/downloadtilessuccess/tmpname/' + tmpname;
    
        $.ajax({
            'type': 'GET',
            'url': url,
            'success': function(data) {
                
            } ,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this
        });
        
        //
        this.currDownloadId = null;
        this.downloadGet();
        
        this.updateFeaturesCount();
    },
    
    downloadComplete: function() {
        $("#gtOfflineProgressDialog").popup("close");
        $('#gtOfflineDownloadButton').button('enable');
        $('.gtOfflineMenuDelete').each(function(index, value) {
            $(this).removeClass('ui-disabled');
        });
    },
    
    deleteLayer: function(id) {
        var self = this;

        //
        var protocol = new OpenLayers.Protocol.SQL.WebSQL({
            tableName: 'layer_' + id
        });
        protocol.drop({
            callback: function() {
                self.updateFeaturesCount();
            }
        });
        
        //Style
        var mapSLD = Giscuit.Util.getStorageObject('gtMapSLD');
        if(mapSLD['layer_' + id]) {
            delete mapSLD['layer_' + id];
        }
        Giscuit.Util.setStorageObject('gtMapSLD', mapSLD);
        
        //Form
        var editForm = Giscuit.Util.getStorageObject('gtEditForm');
        if(editForm['layer_' + id]) {
            delete editForm['layer_' + id];
        }
        Giscuit.Util.setStorageObject('gtEditForm', editForm);
        
        //Geometry Type
        var mapGeometryType = Giscuit.Util.getStorageObject('gtMapGeometryType');
        if(mapGeometryType['layer_' + id]) {
            delete mapGeometryType['layer_' + id];
        }
        Giscuit.Util.setStorageObject('gtMapGeometryType', mapGeometryType);
    },
        
    updateFeaturesCount: function(type) {
        Giscuit.Util.Db.getMetadata(function (metadata) {
            $('.gtOfflineMenuName').each(Giscuit.Util.bind(function(index, value) {
                var li = $(value.parentNode.parentNode.parentNode);
                var layerId = li.jqmData('layer-id');
                var countEl = li.find('.ui-li-count');
                
                if(typeof(metadata[layerId]) == 'undefined' || ! metadata[layerId].count) {
                    countEl.css('display', 'none');
                } else {
                    countEl.html(metadata[layerId].count);
                    countEl.css('display', 'block');
                }
            }, this));
        });
    },
    
    setEditErrors: function() {
        var self = this;
        var editErrors = Giscuit.Util.getStorageObject('gtEditErrors');
        
        var html = '';
        for (var layerId in editErrors) {
            var layer = gt.control.layers.getLayerRecordById(layerId);
            html += '<li data-role="list-divider">' + layer.name + '</li>';
            for (var feature in editErrors[layerId]) {
                for (var field in editErrors[layerId][feature]) {
                    var errors = '';
                    for (var error in editErrors[layerId][feature][field]) {
                        var errorData = editErrors[layerId][feature][field][error];
                        
                        if(errorData instanceof Array && errorData.length == 2) {
                            html += '<li data-type="' + errorData[1] + '" data-layer="'+ layerId +'" data-feature="'+ feature +'">' + 
                                '<a>' + errorData[0] + '</a>' +
                                '</li>';
                        } else {
                            html += '<li data-type="attribute" data-layer="'+ layerId +'" data-feature="'+ feature +'">' + 
                                '<a>' + field + ' - ' +  errorData + '</a>' +
                                '</li>';
                        }
                        
                    }
                }
            }
        }

        $('#gtOfflineEditErrors')
            .off()
            .html(html)
            .listview('refresh');
                
        $('#gtOfflineEditErrors li').on(gt.config.delayedClickEvt, this, function(evt) {
            //
            var currEl = this;
            evt.stopImmediatePropagation();
            
            //Type
            if (! $(this).jqmData('type')) {
                return false;
            }
            
            //Set editable layer
            var layerSettings = gt.control.layers.getLayerRecordById($(this).jqmData('layer'));
            for (var i = 0; i < gt.control.layers.records.length; i++) {
                gt.control.layers.records[i].edit = false;
            }
            layerSettings.edit = true;
            
            //Activate editgeom control
            if(gt.control.editgeom.active) {
                gt.control.editgeom.deactivate(function() {
                    gt.control.editgeom.activate();
                    self.showEditErrors.apply(currEl, [evt]);
                });
            } else {
                gt.control.editgeom.activate();
                self.showEditErrors.apply(currEl, [evt]);
            }

            return false;
        });
    },
    
    showEditErrors: function (evt) {
        //Feature
        var layerSettings = gt.control.layers.getLayerRecordById($(this).jqmData('layer'));
        var layer = gt.layer[layerSettings['class']];
        var feature = layer.getFeatureByFid($(this).jqmData('feature'));
        
        //Type
        var type = $(this).jqmData('type');
        
        if(type == 'attribute') {
            if(feature) {
                gt.control.editgeom.controls.editattrControl.select(feature);
            }
        } else if (type == 'geometry') {
            if(feature) {
                gt.control.editgeom.controls.editControl.selectFeature(feature);
                $.mobile.changePage('#gtMap');
            }
        } else {
            var featureId = $(this).jqmData('feature');
            var layerId = $(this).jqmData('layer');
            
            if(typeof(evt.data.overwriteIds[layerId]) == 'undefined') {
                evt.data.overwriteIds[layerId] = [];
            }
            if(typeof(evt.data.skipIds[layerId]) == 'undefined') {
                evt.data.skipIds[layerId] = [];
            }
            
            $('#gtOfflineConflictOverwriteButton').off(gt.config.delayedClickEvt).on(gt.config.delayedClickEvt, evt.data, function(evt) {
                evt.stopImmediatePropagation();
            
                evt.data.overwriteIds[layerId].push(featureId);
                evt.data.skipIds[layerId] = jQuery.grep(evt.data.skipIds[layerId], function(value) {
                    return value != featureId;
                });
                $.mobile.changePage('#gtOffline');
                
                return false;
            });
            
            $('#gtOfflineConflictSkipButton').off(gt.config.delayedClickEvt).on(gt.config.delayedClickEvt, evt.data, function(evt) {
                evt.stopImmediatePropagation();
            
                evt.data.skipIds[layerId].push(featureId);
                evt.data.overwriteIds[layerId] = jQuery.grep(evt.data.overwriteIds[layerId], function(value) {
                    return value != featureId;
                });
                $.mobile.changePage('#gtOffline');
                
                return false;
            });
            
            $('#gtOfflineConflictCancelButton').off(gt.config.delayedClickEvt).on(gt.config.delayedClickEvt, evt.data, function(evt) {
                evt.stopImmediatePropagation();
            
                evt.data.overwriteIds[layerId] = jQuery.grep(evt.data.overwriteIds[layerId], function(value) {
                    return value != featureId;
                });
                evt.data.skipIds[layerId] = jQuery.grep(evt.data.skipIds[layerId], function(value) {
                    return value != featureId;
                });
                $.mobile.changePage('#gtOffline');
                
                return false;
            });
            
            $('#gtOfflineConflictShowButton').off(gt.config.delayedClickEvt).on(gt.config.delayedClickEvt, evt.data, function(evt) {
                evt.stopImmediatePropagation();
            
                if(feature) {
                    gt.control.editgeom.controls.editControl.selectFeature(feature);
                    $.mobile.changePage('#gtMap');
                }
                
                return false;
            });
            
            $('#gtMetadata').css('min-height', '100%');
            $.mobile.changePage('#gtOfflineConflict', {'role': 'dialog', 'transition': 'none'});
        }
    },
    
    getTileURLs: function (bounds, zoom) {
        var result = [];

        function getTileOrigin() {
            var extent = gt.map.maxExtent;
            var edges = ({
                "tl": ["left", "top"],
                "tr": ["right", "top"],
                "bl": ["left", "bottom"],
                "br": ["right", "bottom"]
            })["bl"];
            var origin = new OpenLayers.LonLat(extent[edges[0]], extent[edges[1]]);
            
            return origin;
        }
        
        function calculateGridLayout (bounds, origin, resolution) {
            //FIXME: buffer removed
            var buffer = 0;
            
            var tilelon = resolution * gt.map.tileSize.w;
            var tilelat = resolution * gt.map.tileSize.h;
            
            var offsetlon = bounds.left - origin.lon;
            var tilecol = Math.floor(offsetlon/tilelon) - buffer;
            var tilecolremain = offsetlon/tilelon - tilecol;
            var tileoffsetx = -tilecolremain * gt.map.tileSize.w;
            var tileoffsetlon = origin.lon + tilecol * tilelon;
            
            var offsetlat = bounds.top - (origin.lat + tilelat);  
            var tilerow = Math.ceil(offsetlat/tilelat) + buffer;
            var tilerowremain = tilerow - offsetlat/tilelat;
            var tileoffsety = -tilerowremain * gt.map.tileSize.h;
            var tileoffsetlat = origin.lat + tilerow * tilelat;
            
            return { 
              tilelon: tilelon, tilelat: tilelat,
              tileoffsetlon: tileoffsetlon, tileoffsetlat: tileoffsetlat,
              tileoffsetx: tileoffsetx, tileoffsety: tileoffsety
            };
        }
        
        function getURL (bounds) {
            var tileOrigin = getTileOrigin();
            
            var res = gt.map.resolutions[zoom];
            var x = Math.round((bounds.left - tileOrigin.lon) / (res * gt.map.tileSize.w));
            var y = Math.round((bounds.bottom - tileOrigin.lat) / (res * gt.map.tileSize.h));
                
            return zoom + "/" + x + "/" + y + '.jpeg'; 
        }
        
        function getTiles (bounds) {
            var origin = getTileOrigin();
            var resolution = gt.map.resolutions[zoom];
            
            var tileLayout = calculateGridLayout(bounds, origin, resolution);

            var tileoffsetx = Math.round(tileLayout.tileoffsetx);
            var tileoffsety = Math.round(tileLayout.tileoffsety);
            
            var tileoffsetlon = tileLayout.tileoffsetlon;
            var tileoffsetlat = tileLayout.tileoffsetlat;

            var tilelon = tileLayout.tilelon;
            var tilelat = tileLayout.tilelat;

            this.origin = new OpenLayers.Pixel(tileoffsetx, tileoffsety);

            var startX = tileoffsetx;
            var startLon = tileoffsetlon;
            
            do {
                tileoffsetlon = startLon;
                tileoffsetx = startX;

                do {
                    var tileBounds = new OpenLayers.Bounds(tileoffsetlon, tileoffsetlat, tileoffsetlon + tilelon, tileoffsetlat + tilelat);
                    result.push(getURL(tileBounds));

                    tileoffsetlon += tilelon;
                    tileoffsetx += gt.map.tileSize.w;
                //FIXME: buffer removed
                } while ((tileoffsetlon <= bounds.right));

                tileoffsetlat -= tilelat;
                tileoffsety += gt.map.tileSize.h;
            //FIXME: buffer removed
            } while ((tileoffsetlat >= bounds.bottom - tilelat));

        }

        getTiles(bounds, zoom);

        return result;
    },

    getFileElementsNames: function (formHtml) {
        var fileInputs = $(formHtml).find('input[type="file"]'), fileNames = [];

        if (fileInputs.length) {
            fileInputs.each(function(index, value) {
                fileNames.push($(value).attr('name'));
            });
        }
        
        return fileNames;
    },
    
    downloadFiles: function() {
        var fileId = this.files.pop();
        var self = this;
        
        $.ajax({
            url: gt.config.httpPath + '/uploads/index/id/' + fileId,
            mimeType: 'text/plain; charset=x-user-defined'
        }).done(function (data, status, xhr) {
            
            var record = gt.control.layers.getLayerRecordById(self.currDownloadId);
            self.saveFile(data, status, xhr, fileId, record.id);
            
            //Progress bar
            var sliderEl = $('#gtOfflineDownloadProgress');
            //Value
            sliderEl.attr('value', parseInt(sliderEl.val()) + 1);
            sliderEl.slider('refresh');
            //Percentage
            $("#gtOfflineDownloadPercent").html(Giscuit.Util.getSliderPercentage(sliderEl) + '%');
            //Label
            $("#gtOfflineDownloadProgressLabel").html(record.name + ' (' + sliderEl.val() + '/' + sliderEl.attr('max') + ')');
            
            //
            if (self.files.length) {
                self.downloadFiles();
            } else {
                self.currDownloadId = null;
                self.downloadGet();
            }
        });
    },
    
    synchronizeConflictedFiles: function() {
        
        if (this.updatedFiles.length) {
            var fileId = this.updatedFiles.pop();
        } else {
            this.synchronizeEditingComplete();
            
            return;
        }
        var self = this;
        
        $.ajax({
            url: gt.config.httpPath + '/uploads/index/id/' + fileId,
            mimeType: 'text/plain; charset=x-user-defined'
        }).done(function (data, status, xhr) {
            
            var record = gt.control.layers.getLayerRecordById(self.currSynchronizeId);
            self.saveFile(data, status, xhr, fileId, record.id);
            
            //Progress bar
            var sliderEl = $('#gtOfflineDownloadProgress');
            //Value
            sliderEl.attr('value', parseInt(sliderEl.val()) + 1);
            sliderEl.slider('refresh');
            //Percentage
            $("#gtOfflineDownloadPercent").html(Giscuit.Util.getSliderPercentage(sliderEl) + '%');
            //Label
            $("#gtOfflineDownloadProgressLabel").html(record.name + ' (' + sliderEl.val() + '/' + sliderEl.attr('max') + ')');

            self.synchronizeConflictedFiles();
        });
    },
    
    saveFile: function(data, status, xhr, fileId, layerId) {
        xhr.responseText = ''; 
        
        //Get file name
        var dataType = xhr.getResponseHeader('content-type');
        var contentDisposition = xhr.getResponseHeader('content-disposition');
        if(contentDisposition) {
            var fileName = contentDisposition.match(/filename="([^"\\]*(?:\\.[^"\\]*)*)"/i)[1];
        }
            
        //Get file content
        var binary = '';
        for (var i = 0; i < data.length; i+=1) {
            binary += String.fromCharCode(data.charCodeAt(i) & 0xff);
        }
        data = ''; 
        //TODO: btoa not supportted in IE
        var text = 'data:' + dataType + ';base64,' + btoa(binary);
        binary = '';

        //Insert file
        var protocol = new Giscuit.Util.Db.Files({tableName: 'files_' + layerId});
        protocol.create({
            id: fileId, 
            data: text, 
            name: fileName,
            hstatus: ''
        });
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Offline"
});

gt.control.offline = new Giscuit.Mobile.Control.Offline();