<?php

class WfsControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{

    protected $_layerName = 'Worldboundaries';

    protected $_wfsParams = array(
        'TYPENAME' => 'Worldboundaries' , 
        'SERVICE' => 'wfs' , 
        'VERSION' => '1.0.0' , 
        'REQUEST' => 'GetFeature' 
    );
    
    protected $_validLogin = array(
        'account' => 'test@giscuit.com' , 
        'password' => '123456'
    );

    public function testPublicGetCapabilities()
    {
        $this->dispatch('/' . $this->_language . '/default/wfs/public?SERVICE=WFS&REQUEST=GetCapabilities&VERSION=1.0.0');
        
        $response = $this->getResponse()->getBody();
        
        $this->assertContains('<Name>Worldboundaries</Name>', $response);
    }
    
    public function testPrivateGetCapabilities()
    {
        $url = '/' . $this->_language . '/default/wfs/private?SERVICE=WFS&REQUEST=GetCapabilities&VERSION=1.0.0&';

        $this->_validLogin['password'] = md5($this->_validLogin['password']);
        foreach ($this->_validLogin as $key => $value) {
            $url .= "{$key}={$value}&";
        }
        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        
        $this->assertContains('<Name>Worldboundaries</Name>', $response);
    }
    
    public function testDescribeFeatureType()
    {
        $url = '/' . $this->_language . '/default/wfs/public/?SERVICE=WFS&VERSION=1.0.0&REQUEST=DescribeFeatureType&TYPENAME=' . $this->_layerName;
        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        
        $this->assertContains('<element name="iso_a2" type="string"/>', $response);
    }
    
    public function testPublicAction ()
    {
        $queryString = '';
        foreach ($this->_wfsParams as $key => $value) {
            $queryString .= "{$key}={$value}&";
        }
        $url = '/' . $this->_language . '/default/wfs/public?' . $queryString;
        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        $this->assertContains('<gml:coordinates>27.820039,48.416270', $response);
    }
    
    public function testPrivateAction()
    {
        $queryString = '';
        foreach ($this->_wfsParams as $key => $value) {
            $queryString .= "{$key}={$value}&";
        }
        $this->_validLogin['password'] = md5($this->_validLogin['password']);
        foreach ($this->_validLogin as $key => $value) {
            $queryString .= "{$key}={$value}&";
        }
        
        $url = '/' . $this->_language . '/default/wfs/private?' . $queryString;
        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        $this->assertContains('<gml:coordinates>27.820039,48.416270', $response);
    }
}