<?php
/**
 * Giscuit
 *
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 Giscuit
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for borders identification
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 Giscuit
 */
class Giscuit_Controller_Plugin_Borders extends Giscuit_Controller_Plugin_Abstract
{

    //Translations for the new keywords
    protected $_defaultTranslations = array(
        'borders' => 'Borders' , 
        'borderingCountriesWith' => 'Bordering countries with' ,
        'noNeighboursFound' => 'No neighbours found'
    );

    /**
     * List of the javascript file to include
     * 
     * @return array
     */
    public function getJsFilesList ()
    {
        return array(
            'Control/Borders.js'
        );
    }
    
    /**
     * List of the javascript files to include for mobile
     * 
     * @return array
     */
    public function getMobileJsFilesList ()
    {
        return array(
        	'Mobile/Control/Borders.js'
        );
    }
    
    /**
     * Install plugin
     *
     * Here we will make our plugin accessible to registered users only.
     * This will be used to limit access to our controller that will be created (Tool_BordersController) 
     * and to control our button visibility in the getJsInitCode method 
     * of the Giscuit_Controller_Plugin_Borders class.
     * Also we will set a limit parameter to the number of results returned from the database.
     * 
     */
    public function install($newConfig) 
    {
        $rulesTable = new Giscuit_Db_Table_Rules();
        $data = array(
            'role' => 'registered',
            'resource' => 'tool_borders_controller',
            'privilege' => '',
            'group' => 'controller'
        );
        $rulesTable->insert($data);
        
        $pluginName = $this->name;
        $newConfig->production->plugins->$pluginName->limit = 10;
        
        //Add translations for the new keywords
        $defaultTranslations = array(
            'borders' => 'Borders' , 
            'borderingCountriesWith' => 'Bordering countries with' , 
            'noNeighboursFound' => 'No neighbours found'
        );
        
        //Add default section translations
        $defaultLanguageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.en.tmx';
        $translate = new Zend_Translate('tmx', $defaultLanguageFile, 'en');
        
        $newMessages = array_merge($translate->getMessages(), $this->_defaultTranslations);
        $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx('en', $newMessages);
        file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.en.tmx', $newTmx);
    }
    
    /**
     * Uninstall plugin
     *  
     */
    public function uninstall($newConfig) 
    {
        $rulesTable = new Giscuit_Db_Table_Rules();
        $where = $rulesTable->getAdapter()->quoteInto('resource = ?', 'tool_borders_controller');
        $rulesTable->delete($where);
		
        //Delete translations
        $defaultLanguageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.en.tmx';
        $translate = new Zend_Translate('tmx', $defaultLanguageFile, 'en');
        
        $translations = $translate->getMessages();
        foreach ($this->_defaultTranslations as $key => $value) {
            unset($translations[$key]);
        }
        
        $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx('en', $translations);
        file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.en.tmx', $newTmx);
    }

    /**
     * Parse settings in form
     *
     * Here we can add additional configuration parameters to the plugin form.
     * In this case we will add the limit parameter, setting required validators and current value.
     *  
     * @return boolean
     */
    public function parseSettingsForm($form) 
    {
        $settings = $this->getSettings();
        $limit = $settings['limit'];
    
        $form->addElement('text', $this->name . '_limit', array(
            'validators' => array(
                'Digits'
            ) ,
            'required' => true , 
            'label' => 'limit' ,
            'value' => $limit
        ));
    
        return true;
    }
    
    /**
     * Parse settings from received parameters.
     * This is the place where we can save received parameters to config file
     *  
     * @return boolean
     */
    public function parseSettingsParams($newConfig, $form) 
    {
        $pluginName = $this->name;
    
        $newConfig->production->plugins->$pluginName->limit = $form->getValue($pluginName . '_limit');
        
        return true;
    }

}