Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.borders = new Giscuit.Control.Borders();
    }
});

/**
 * Namespace: Giscuit.Control.Borders
 */
Giscuit.Control.Borders = OpenLayers.Class(Giscuit.Control, {

    panel: null,
    
    clickControl: null,
    
    button: null,

    initialize: function(options) {
    	Giscuit.Control.prototype.initialize.apply(this, [options]);

    	var self = this;
        this.button = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon('view_statistics.png'), 
            id: "gtBordersControl", 
            onclick: {fn: function() {
                if(self.active) {
                    self.deactivate();
                } else {
                    self.activate();
                }
            }},
            title: gt.translate._('borders'), 
            container: "gtMapControls",
            type: 'checkbox'
        });
    }, 
    
    activate: function () {
        //Deactivate all active controls
        gt.map.deactivateAll();
        
        var activated = Giscuit.Control.prototype.activate.call(this);
        if (activated == true) {
        	//Create, add and activate a click control
            this.clickControl = new Giscuit.Control.Click({
                'clickCallback': Giscuit.Util.bind(this.trigger, this)
            });
            gt.map.addControl(this.clickControl);
            this.clickControl.activate();
        }

        return activated;
    },

    deactivate: function () {
        
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        if (deactivated == true) {
            //Destroy control
            this.clickControl.destroy();
            this.clickControl = null;
        }
        
        return deactivated;
    },
    
    /**
     * Called when we recive a response from the server
     */
    getSuccess: function (o) {
        
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
        }
        
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }

        if(responseText.records.length > 0) {
            //Construct html from received data
            var html = '<ul>' + gt.translate._('borderingCountriesWith') +  
                ' ' +
                responseText.name +  
                ': <li>' + 
                responseText.records.join('</li><li>') +
                '</li></ul>';
        } else {
            var html = '<p>' + gt.translate._('noNeighboursFound') + '</p>';
        }

        //Populate our accordion with prepared html
        gt.accord.setBody(this.getPanelIndex(), html);
         
    },

    /**
     * Called when user clicks on the map
     */
    trigger: function (e) {
        var lonlat = gt.map.getLonLatFromViewPortPx(e.xy);
        
        //Create a new accordion or open an existing one with a loading image
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('borders'),
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
        } 
        gt.accord.openPanel(this.getPanelIndex());
        
        //On success call the getSuccess method
        var getCallback = {
            success: this.getSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        //Make an async request to the server with previously collected coordinates
        var request = YAHOO.util.Connect.asyncRequest(
            'GET',
            gt.config.httpToolPath + "/borders/get" + 
                "/lon/" + lonlat.lon + "/lat/" + lonlat.lat,
            getCallback
        );
        
    },
    
    CLASS_NAME: "Giscuit.Control.Borders"
});